unit block;

{
    Vector graphical editor.
    Version: 10.
    Written on FreePascal (https://freepascal.org/).
    Copyright (C) 2024  Warfalomay

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
}

//Функции для работы с блоками

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils;

//Собрать блок
procedure MakeBlock;
//Собрать новый блок при копировании
procedure MakeBlock(primitives : array of integer); 
//Проверка по массиву блоков
function Prov_Sel_Block(num : integer):integer;
//Номер линии в массиве линий по ID
function FindLineNumber(num : integer) : integer;
//Номер окружности в массиве окружностей по ID
function FindCircleNumber(num : integer) : integer;
//Номер точки в массиве точек по ID
function FindPointNumber(num : integer) : integer;
//Номер точки в массиве точек по ID
function FindTextNumber(num : integer) : integer;
//Номер стрелки в массиве стрелок по ID
function FindArrowNumber(num : integer) : integer;
//Номер дуги в массиве дуг по ID
function FindArcNumber(num : integer) : integer;
//Номер размера в массиве размеров по ID
function FindDimNumber(num : integer) : integer;
//выбрать блок
procedure Select_block(block_num : integer);
//удаление блока из массива блоков
procedure DeleteBlock(num : integer);

implementation

uses unit1, view, primitives, Graphics, Dialogs, SpecFunction;

//собрать блок
procedure MakeBlock;

var
  count : integer;
  res : boolean = false;

begin
    //добавление блока в массив
    SetLength(blocks, Length(blocks)+1);
    //присвоение номера блока
    blocks[High(blocks)].num := High(blocks);
    //линии
    for count := 0 to High(multy_selected_line) do
    begin
       SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
       blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0] :=
                     lines[multy_selected_line[count]].id;
       blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1] := Line; //тип - линия
       res := true;
    end;
    //окружности
    for count := 0 to High(multy_selected_circle) do
    begin
       SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
       blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0] :=
                     circles[multy_selected_circle[count]].id;
       blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1] := Circle; //тип - окружность
       res := true;
    end;
    //точки
    for count := 0 to High(multy_selected_point) do
    begin
       SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
       blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0] :=
                     points[multy_selected_point[count]].id;
       blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1] := Dot; //тип - точка
       res := true;
    end;
    //надписи
    for count := 0 to High(multy_selected_text) do
    begin
       SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
       blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0] :=
                     nadp[multy_selected_text[count]].id;
       blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1] := txt; //тип - надпись
       res := true;
    end;
    //стрелки
    for count := 0 to High(multy_selected_arrow) do
    begin
       SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
       blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0] :=
                     arrow[multy_selected_arrow[count]].id;
       blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1] := Arr; //тип - стрелка
       res := true;
    end;

    //дуги
    for count := 0 to High(multy_selected_arcs) do
    begin
       SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
       blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0] :=
                     arcs[multy_selected_arcs[count]].id;
       blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1] := Arc; //тип - дуга
       res := true;
    end;

    //размеры
    for count := 0 to High(multy_selected_dim) do
    begin
       SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
       blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0] :=
                     DimAr[multy_selected_dim[count]].id;
       blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1] := Dim; //тип - размер
       res := true;
    end;

    if (res) then showmessage('Создан блок №' + IntToStr(blocks[High(blocks)].num));

    Reset_mode;
    Clear;
end;

//Собрать новый блок при копировании
procedure MakeBlock(primitives : array of integer);

var
  count : integer;
  FindedNum : integer;

begin
   //добавление блока в массив
    SetLength(blocks, Length(blocks)+1);
    //присвоение номера блока
    blocks[High(blocks)].num := High(blocks);

    //линии
    for count := 0 to High(primitives) do
    begin
      //найдена линия
      FindedNum := FindLineNumber(primitives[count]);

      if FindedNum <> -1 then
      begin
         SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0] :=
                       lines[FindedNum].id;
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1] := Line; //тип - линия

         continue;
      end;

      //найдена окружность
      FindedNum := FindCircleNumber(primitives[count]);

      if FindedNum <> -1 then
      begin
         SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0] :=
                       circles[FindedNum].id;
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1] := circle; //тип - окружность

         continue;
      end;

      //найдена точка
      FindedNum := FindPointNumber(primitives[count]);

      if FindedNum <> -1 then
      begin
         SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0] :=
                       points[FindedNum].id;
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1] := dot; //тип - точка

         continue;
      end;

      //найден текст
      FindedNum := FindTextNumber(primitives[count]);

      if FindedNum <> -1 then
      begin
         SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0] :=
                       nadp[FindedNum].id;
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1] := txt; //тип - текст

         continue;
      end;

      //найдена стрелка
      FindedNum := FindArrowNumber(primitives[count]);

      if FindedNum <> -1 then
      begin
         SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0] :=
                       arrow[FindedNum].id;
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1] := arr; //тип - стрелка

         continue;
      end;

      //найдена дуга
      FindedNum := FindArcNumber(primitives[count]);

      if FindedNum <> -1 then
      begin
         SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0] :=
                       arcs[FindedNum].id;
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1] := arc; //тип - дуга

         continue;
      end;

      //найден размер
      FindedNum := FindDimNumber(primitives[count]);

      if FindedNum <> -1 then
      begin
         SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0] :=
                       DimAr[FindedNum].id;
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1] := dim; //тип - размер

         continue;
      end;

    end;
end;       

//Проверка по массиву блоков
function Prov_Sel_Block(num : integer):integer;

var
  count, count2 : integer;

begin
   result := -1;

   for count := High(blocks) downto 0 do
        for count2 := 0 to High(blocks[count].id_num) do
        if (blocks[count].id_num[count2,0] = num) then
        begin
             //наден блок, в который входит объект
             result := count;
             exit;
        end;
end;

//Номер линии в массиве линий по ID
function FindLineNumber(num : integer) : integer;

var
  count : integer;

begin
   result := -1;

   for count := 0 to High(lines) do
   if lines[count].id = num then
   begin
        result := count;
        exit;
   end;
end;

//Номер окружности в массиве окружностей по ID
function FindCircleNumber(num : integer) : integer;

var
  count : integer;

begin
   result := -1;

   for count := 0 to High(circles) do
   if circles[count].id = num then
   begin
        result := count;
        exit;
   end;
end;

//Номер точки в массиве точек по ID
function FindPointNumber(num : integer) : integer;

var
  count : integer;

begin
   result := -1;

   for count := 0 to High(points) do
   if points[count].id = num then
   begin
        result := count;
        exit;
   end;
end;

//Номер текста в массиве точек по ID
function FindTextNumber(num : integer) : integer;

var
  count : integer;

begin
   result := -1;

   for count := 0 to High(nadp) do
   if nadp[count].id = num then
   begin
        result := count;
        exit;
   end;
end;

//Номер стрелки в массиве стрелок по ID
function FindArrowNumber(num : integer) : integer;

var
  count : integer;

begin
   result := -1;

   for count := 0 to High(arrow) do
   if arrow[count].id = num then
   begin
        result := count;
        exit;
   end;
end;

//Номер дуги в массиве дуг по ID
function FindArcNumber(num : integer) : integer;

var
  count : integer;

begin
   result := -1;

   for count := 0 to High(arcs) do
   if arcs[count].id = num then
   begin
        result := count;
        exit;
   end;
end;

//Номер размера в массиве размеров по ID
function FindDimNumber(num : integer) : integer;

var
  count : integer;

begin
   result := -1;

   for count := 0 to High(DimAr) do
   if DimAr[count].id = num then
   begin
        result := count;
        exit;
   end;
end;

//выбрать блок
procedure Select_block(block_num : integer);

var
    block_counter : integer;
    vremNum : integer;

begin
    //Clear;
    For block_counter := 0 to High(blocks[block_num].id_num) do
       begin
         If blocks[block_num].id_num[block_counter,1] = line then //найдена линия в блоке
         begin
             //найти номер линии в массиве линий по ее id
             vremNum := FindLineNumber(blocks[block_num].id_num[block_counter,0]);

             if vremNum<>-1 then
                 begin
                     //выделение линии, входящей в блок
                     form1.WS.Canvas.Ellipse(Round(Lines[vremNum].x1)-5+Screen_x,
                                             Round(Lines[vremNum].y1)-5+screen_y,
                                             Round(Lines[vremNum].x1)+5+Screen_x,
                                             Round(Lines[vremNum].y1)+5+Screen_y);
                     form1.WS.Canvas.Ellipse(Round(Lines[vremNum].x2)-5+Screen_x,
                                             Round(Lines[vremNum].y2)-5+Screen_y,
                                             Round(Lines[vremNum].x2)+5+Screen_x,
                                             Round(Lines[vremNum].y2)+5+Screen_y);

                     //Занести линии в массив мультивыделения
                     setLength(multy_selected_line,Length(multy_selected_line)+1);
                     multy_selected_line[High(multy_selected_line)] := vremNum;
                 end;
         end;



         If blocks[block_num].id_num[block_counter,1] = circle then //найдена окружность в блоке
         begin
             //найти номер окружности в массиве кругов по ее id
             vremNum := FindCircleNumber(blocks[block_num].id_num[block_counter,0]);

             //выделение окружности, входящей в блок
             Form1.WS.Canvas.Ellipse(Round(Circles[vremNum].x-Circles[vremNum].r)-5+Screen_x,
                                     Round(Circles[vremNum].y+Circles[vremNum].r)+5+Screen_y,
                                     Round(Circles[vremNum].x-Circles[vremNum].r)+5+Screen_x,
                                     Round(Circles[vremNum].y+Circles[vremNum].r)-5+Screen_y);
             Form1.WS.Canvas.Ellipse(Round(Circles[vremNum].x-Circles[vremNum].r)-5+Screen_x,
                                     Round(Circles[vremNum].y-Circles[vremNum].r)+5+Screen_y,
                                     Round(Circles[vremNum].x-Circles[vremNum].r)+5+Screen_x,
                                     Round(Circles[vremNum].y-Circles[vremNum].r)-5+Screen_y);
             Form1.WS.Canvas.Ellipse(Round(Circles[vremNum].x+Circles[vremNum].r)-5+Screen_x,
                                     Round(Circles[vremNum].y+Circles[vremNum].r)+5+Screen_y,
                                     Round(Circles[vremNum].x+Circles[vremNum].r)+5+Screen_x,
                                     Round(Circles[vremNum].y+Circles[vremNum].r)-5+Screen_y);
             Form1.WS.Canvas.Ellipse(Round(Circles[vremNum].x+Circles[vremNum].r)-5+Screen_x,
                                     Round(Circles[vremNum].y-Circles[vremNum].r)+5+Screen_y,
                                     Round(Circles[vremNum].x+Circles[vremNum].r)+5+Screen_x,
                                     Round(Circles[vremNum].y-Circles[vremNum].r)-5+Screen_y);
             Form1.WS.Canvas.Ellipse(Round(Circles[vremNum].x-3+Screen_x),
                                     Round(Circles[vremNum].y-3+Screen_y),
                                     Round(Circles[vremNum].x+3+Screen_x),
                                     Round(Circles[vremNum].y+3+Screen_y));

             //Занести окружность в массив мультивыделения
             setLength(multy_selected_circle,Length(multy_selected_circle)+1);
             multy_selected_circle[High(multy_selected_circle)] := vremNum;
         end;



         If blocks[block_num].id_num[block_counter,1] = dot then //найдена точка в блоке
         begin
             //найти номер точки в массиве точек по ее id
             vremNum := FindPointNumber(blocks[block_num].id_num[block_counter,0]);

             Form1.WS.Canvas.Ellipse(Round(points[vremNum].x)-15+Screen_x,
                                     Round(points[vremNum].y)+15+Screen_y,
                                     Round(points[vremNum].x)+15+Screen_x,
                                     Round(points[vremNum].y)-15+Screen_y);

             //Занести точку в массив мультивыделения
             setLength(multy_selected_point,Length(multy_selected_point)+1);
             multy_selected_point[High(multy_selected_point)] := vremNum;
         end;



         If blocks[block_num].id_num[block_counter,1] = txt then //найден текст в блоке
         begin
             //найти номер текста в массиве дуг по ее id
             vremNum := FindTextNumber(blocks[block_num].id_num[block_counter,0]);

             //выделение тескта, входящего в блок
             Form1.WS.Canvas.Font.Size:=Round(nadp[vremNum].h);
             Form1.WS.Canvas.Font.Color:= clRed;
             Form1.WS.Canvas.TextOut(Round(nadp[vremNum].x+Screen_x),
                                     Round(nadp[vremNum].y+Screen_y),
                                     nadp[vremNum].s);

             //Занести текст в массив мультивыделения
             setLength(multy_selected_text,Length(multy_selected_text)+1);
             multy_selected_text[High(multy_selected_text)] := vremNum;
         end;



         If blocks[block_num].id_num[block_counter,1] = arr then //найдена стрелка в блоке
         begin
             //найти номер стрелки в массиве стрелок по ее id
             vremNum := FindArrowNumber(blocks[block_num].id_num[block_counter,0]);

             //выделение стрелки, входящей в блок
             Form1.WS.Canvas.Ellipse(Round(arrow[vremNum].x-5),
                                     Round(arrow[vremNum].y+5),
                                     Round(arrow[vremNum].x+5),
                                     Round(arrow[vremNum].y-5));
             if arrow[vremNum].t = 'left' then
                  Form1.WS.Canvas.Ellipse(Round(arrow[vremNum].x+30-5+Screen_x),
                                          Round(arrow[vremNum].y+5+Screen_y),
                                          Round(arrow[vremNum].x+30+5+Screen_x),
                                          Round(arrow[vremNum].y-5+Screen_y));
             if arrow[vremNum].t = 'right' then
                  Form1.WS.Canvas.Ellipse(Round(arrow[vremNum].x-30-5+Screen_x),
                                          Round(arrow[vremNum].y+5+Screen_y),
                                          Round(arrow[vremNum].x-30+5+Screen_x),
                                          Round(arrow[vremNum].y-5+Screen_y));
             if arrow[vremNum].t = 'up' then
                  Form1.WS.Canvas.Ellipse(Round(arrow[vremNum].x-5+Screen_x),
                                          Round(arrow[vremNum].y+30+5+Screen_y),
                                          Round(arrow[vremNum].x+5+Screen_x),
                                          Round(arrow[vremNum].y+30-5+Screen_y));
             if arrow[vremNum].t = 'down' then
                  Form1.WS.Canvas.Ellipse(Round(arrow[vremNum].x-5+Screen_x),
                                          Round(arrow[vremNum].y-30+5+Screen_y),
                                          Round(arrow[vremNum].x+5+Screen_x),
                                          Round(arrow[vremNum].y-30-5+Screen_y));
             //Занести стрелку в массив мультивыделения
             setLength(multy_selected_arrow,Length(multy_selected_arrow)+1);
             multy_selected_arrow[High(multy_selected_arrow)] := vremNum;
         end;



         If blocks[block_num].id_num[block_counter,1] = arc then //найдена дуга в блоке
         begin
             //найти номер дуги в массиве дуг по ее id
             vremNum := FindArcNumber(blocks[block_num].id_num[block_counter,0]);

             //выделение дуги, входящей в блок
             Form1.WS.Canvas.Ellipse(Round(Arcs[vremNum].x1-5+Screen_x),
                                     Round(Arcs[vremNum].y1-5+Screen_y),
                                     Round(Arcs[vremNum].x1+5+Screen_x),
                                     Round(Arcs[vremNum].y1+5+Screen_y));
             Form1.WS.Canvas.Ellipse(Round(Arcs[vremNum].x2-5+Screen_x),
                                     Round(Arcs[vremNum].y2-5+Screen_y),
                                     Round(Arcs[vremNum].x2+5+Screen_x),
                                     Round(Arcs[vremNum].y2+5+Screen_y));

             //Занести дугу в массив мультивыделения
             setLength(multy_selected_arcs,Length(multy_selected_arcs)+1);
             multy_selected_arcs[High(multy_selected_arcs)] := vremNum;
         end;

         If blocks[block_num].id_num[block_counter,1] = Dim then //найден размер в блоке
         begin
             //найти номер размера в массиве размеров по его id
             vremNum := FindDimNumber(blocks[block_num].id_num[block_counter,0]);

             //выделение размера, входящего в блок
             Form1.WS.Canvas.Ellipse(Round(DimAr[vremNum].x3-5), Round(DimAr[vremNum].y3-5),
                                     Round(DimAr[vremNum].x3+5), Round(DimAr[vremNum].y3+5));
             Form1.WS.Canvas.Ellipse(Round(DimAr[vremNum].x4-5), Round(DimAr[vremNum].y4-5),
                                     Round(DimAr[vremNum].x4+5), Round(DimAr[vremNum].y4+5));

             Form1.WS.Canvas.Ellipse(Round(DimAr[vremNum].x1-5), Round(DimAr[vremNum].y1-5),
                                     Round(DimAr[vremNum].x1+5), Round(DimAr[vremNum].y1+5));
             Form1.WS.Canvas.Ellipse(Round(DimAr[vremNum].x2-5), Round(DimAr[vremNum].y2-5),
                                     Round(DimAr[vremNum].x2+5), Round(DimAr[vremNum].y2+5));

             //Занести размер в массив мультивыделения
             setLength(multy_selected_dim,Length(multy_selected_dim)+1);
             multy_selected_dim[High(multy_selected_dim)] := vremNum;
         end;

       end;

end;

//удаление блока из массива блоков
procedure DeleteBlock(num : integer);
begin
    Delete(blocks,num,1);
end;

end.

