unit calc;

{
    Vector graphical editor.
    Version: 10.
    Written on FreePascal (https://freepascal.org/).
    Copyright (C) 2024  Warfalomay

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
}

//вспомогательные расчетные функции

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, math, MyTypes, Graphics, Drawing_unit;

//Расчет угла отрезка по 2 точкам
function CalcAngle(x1,y1,x2,y2 : double) : double;
//Вычисление площади, периметра и суммарной изм.длины
function Calc3Par : FreeVal;
//Площадь треугольника по 3 сторонам
function Area3(a,b,c : double) : double;
//С какой стороны прямой точка
function SideDot(x1,y1,x2,y2,x3,y3 : double) : double;
//Расчет зеркальной точки
function MirrorCalc(x1,y1,x2,y2,x3,y3 : double) : MyTPoint;

implementation

uses Unit1, view;

//Расчет зеркальной точки
function MirrorCalc(x1,y1,x2,y2,x3,y3 : double) : MyTPoint;

var
  xr, yr : double;

begin
    xr := ((x2-x1)*(y2-y1)*(y3-y1)+x1*sqr(y2-y1)+x3*sqr(x2-x1))/(sqr(y2-y1)+sqr(x2-x1));
    if (x2 <> x1) then
       yr := (y2-y1)*(xr-x1)/(x2-x1)+y1
    else
      yr := y3;

    if xr>x3 then result.x := xr + abs(x3-xr)
    else result.x := xr - abs(x3-xr);
    if yr > y3 then result.y := yr + abs(y3-yr)
    else result.y := yr - abs(y3-yr);
end;

//Расчет угла наклона отрезка по 2 точкам для экспорта дуг
function CalcAngle(x1,y1,x2,y2 : double) : double;
begin
    result := DegToRad(270)+ArcTan2(x2-x1,y2-y1);
end;

//Вычисление площади, периметра и суммарной изм.длины
function Calc3Par : FreeVal;

var
   Sarea : double = 0;
   Lper  : double = 0;
   LWay  : double = 0;

   i : integer;

   vrem_pen_color : TColor;
   vrem_brush_color : TColor;
   vremPointList : array of TPoint;
   countPoint : integer;

begin
    for i := 0 to High(point_list)-1 do
    begin
      Sarea := (Sarea+point_list[i].X*point_list[i+1].y-point_list[i+1].X*point_list[i].y);
      Lper := Lper + sqrt(sqr(point_list[i].x-point_list[i+1].x)+
                          sqr(point_list[i].y-point_list[i+1].y));
    end;

    Lway := Lper;
    Sarea := Sarea + point_list[High(point_list)].X*point_list[0].y-point_list[0].X*point_list[High(point_list)].y;
    Lper := Lper + sqrt(sqr(point_list[High(point_list)].x-point_list[0].x)+
                        sqr(point_list[High(point_list)].y-point_list[0].y));

    Sarea := abs(Sarea/2);


    clear();
    vrem_pen_color := Form1.WS.Canvas.Pen.Color;
    vrem_brush_color := Form1.WS.Canvas.Brush.Color;
    Form1.WS.Canvas.Pen.Color:=clPurple;
    Form1.WS.Canvas.Brush.Style := bsCross;
    Form1.WS.Canvas.Brush.Color := clPurple;

    SetLength(vremPointList,Length(point_list));
    for countPoint := 0 to High(point_list) do
    begin
        vremPointList[countPoint].X := Round(point_List[countPoint].X);
        vremPointList[countPoint].Y := Round(point_List[countPoint].Y);
    end;

    Form1.WS.Canvas.Polygon(vremPointList);
    Form1.WS.Canvas.Pen.Color := vrem_pen_color;
    Form1.WS.Canvas.Brush.Color := vrem_brush_color;
    Form1.WS.Canvas.Brush.Style := bsClear;

    Result.S := Sarea;
    Result.L1 := Lper;
    Result.L2 := Lway;

    SetLength(vremPointList,0);
end;

//Площадь треугольника по 3 сторонам
function Area3(a,b,c : double) : double;

var
   p : double; //Полупериметр

begin
   p := (a+b+c)/2;
   result := sqrt(abs(p*(p-a)*(p-b)*(p-c)));
end;

//С какой стороны прямой точка
//res = 0 - на прямой, <0 - слева, >0 - справа
function SideDot(x1,y1,x2,y2,x3,y3 : double) : double;
begin
   result := (x3 - x1) * (y2 - y1) - (y3 - y1) * (x2 - x1);
end;


end.

