unit Drawing_unit;

{
    Vector graphical editor.
    Version: 10.
    Written on FreePascal (https://freepascal.org/).
    Copyright (C) 2024  Warfalomay

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
}

//Функции создания объектов

{$mode ObjFPC}{$H+}

interface
uses Classes, Dialogs, Graphics, math;

type
  MyTPoint = record
    x,y : double;
  end;

  myTRect = record
    left,
    right,
    top,
    bottom : double;
  end;

//Расчет и построение точки середины отрезка
Procedure Midle_point();
//Рисование прямоугольника
Procedure DrawRect(x_vrem, y_vrem : double);
//Рисование отрезка
Procedure DrawLine(x_vrem, y_vrem : double);
//Рисование окружности
Procedure DrawCirc(x_vrem, y_vrem : double);
//Дуга
Procedure Draw_arc(Point1,Point2 : MyTPoint; GetRad : double);
//Окружность по 2 ординатам
Procedure Draw_circle_2_point(Point1,Point2 : MyTPoint);
//Окружность по 3 точкам (вписаная в треугольник)
Procedure Draw_circle_3_point(Point1, Point2, Point3 : MyTPoint);
//Вписаный многоугольник
Procedure InMultAngle(x,y : double);
//Описаный многоугольник
Procedure OutMultAngle(x,y : double);
//Направляющая через 2 точки
Procedure Ray2Point(x_vrem,y_vrem : double);
//Направляющая под углом
Procedure AngleRay(x_vrem, y_vrem, vrem_angle : double);
//Рисовать стрелку
Procedure DrawArrow(x_vrem,y_vrem : double);
//Добавить размер
Procedure AddDimension(x,y : double);
//Добавить фаску между 2 отрезками из массива мультивыделения
Procedure AddBevel;
//Добавить скругление между 2 отрезками из массива мультивыделения
procedure RoundEdge();


implementation

uses
  SysUtils, Unit1, View, my_modify, Intersection_unit, primitives, calc,
  SpecFunction, block;


//Добавить фаску между 2 отрезками из массива мультивыделения
Procedure AddBevel;

var
  CrossPoint : MyTPoint;
  EndPoint1, EndPoint2 : MyTPoint;
  BevelSize : double;
  VremAngle : double = 0;

begin
  CrossPoint := PointCross2Line(multy_selected_line[0],
                                 multy_selected_line[1], false);

   //Проверка  что точка пересечения находится на конце отрезков
    if ((abs(CrossPoint.x - lines[multy_selected_line[0]].x1) > 0.0001) and
        (abs(CrossPoint.y - lines[multy_selected_line[0]].y1) > 0.0001)) and
       ((abs(CrossPoint.x - lines[multy_selected_line[0]].x2) > 0.0001) and
        (abs(CrossPoint.x - lines[multy_selected_line[0]].y2) > 0.0001)) and
       ((abs(CrossPoint.x - lines[multy_selected_line[1]].x1) > 0.0001) and
        (abs(CrossPoint.y - lines[multy_selected_line[1]].y1) > 0.0001)) and
       ((abs(CrossPoint.x - lines[multy_selected_line[1]].x2) > 0.0001) and
        (abs(CrossPoint.y - lines[multy_selected_line[1]].y2) > 0.0001)) then
      begin
        showmessage('Отрезки должны иметь 1 общую КОНЕЧНУЮ точку');
        exit;
      end;

   /////////////////////////////////
  BevelSize := StrToFloat(InputBox('Построение фаски', 'Укажи размер фаски','0'));
  if BevelSize <=0 then exit;

  //Для 1 отрезка
  VremAngle := ArcTan2((lines[multy_selected_line[0]].y2-lines[multy_selected_line[0]].y1),
                      (lines[multy_selected_line[0]].x2-lines[multy_selected_line[0]].x1));

  if (abs(CrossPoint.x - lines[multy_selected_line[0]].x2) < 0.0001) and
     (abs(CrossPoint.y - lines[multy_selected_line[0]].y2) < 0.0001) then
  begin
     EndPoint1.x := lines[multy_selected_line[0]].x2 - BevelSize * cos(VremAngle);
     EndPoint1.y := lines[multy_selected_line[0]].y2 - BevelSize * sin(VremAngle);
     lines[multy_selected_line[0]].x2 := EndPoint1.x;
     lines[multy_selected_line[0]].y2 := EndPoint1.y;
     //Form1.WS.Canvas.Ellipse(Round(EndPoint1.x-5),Round(EndPoint1.y-5),Round(EndPoint1.x+5),Round(EndPoint1.y+5));
  end;

  if (abs(CrossPoint.x - lines[multy_selected_line[0]].x1) < 0.0001) and
     (abs(CrossPoint.y - lines[multy_selected_line[0]].y1) < 0.0001) then
  begin
     EndPoint1.x := lines[multy_selected_line[0]].x1 + BevelSize * cos(VremAngle);
     EndPoint1.y := lines[multy_selected_line[0]].y1 + BevelSize * sin(VremAngle);
     lines[multy_selected_line[0]].x1 := EndPoint1.x;
     lines[multy_selected_line[0]].y1 := EndPoint1.y;
     //Form1.WS.Canvas.Ellipse(Round(EndPoint1.x-5),Round(EndPoint1.y-5),Round(EndPoint1.x+5),Round(EndPoint1.y+5));
  end;

  //Для 2 отрезка
  VremAngle := ArcTan2((lines[multy_selected_line[1]].y2-lines[multy_selected_line[1]].y1),
                      (lines[multy_selected_line[1]].x2-lines[multy_selected_line[1]].x1));

  if (abs(CrossPoint.x - lines[multy_selected_line[1]].x2) < 0.0001) and
     (abs(CrossPoint.y - lines[multy_selected_line[1]].y2) < 0.0001) then
  begin
     EndPoint2.x := lines[multy_selected_line[1]].x2 - BevelSize * cos(VremAngle);
     EndPoint2.y := lines[multy_selected_line[1]].y2 - BevelSize * sin(VremAngle);
     lines[multy_selected_line[1]].x2 := EndPoint2.x;
     lines[multy_selected_line[1]].y2 := EndPoint2.y;
     //Form1.WS.Canvas.Ellipse(Round(EndPoint2.x-5),Round(EndPoint2.y-5),Round(EndPoint2.x+5),Round(EndPoint2.y+5));
  end;

  if (abs(CrossPoint.x - lines[multy_selected_line[1]].x1) < 0.0001) and
     (abs(CrossPoint.y - lines[multy_selected_line[1]].y1) < 0.0001) then
  begin
     EndPoint2.x := lines[multy_selected_line[1]].x1 + BevelSize * cos(VremAngle);
     EndPoint2.y := lines[multy_selected_line[1]].y1 + BevelSize * sin(VremAngle);
     lines[multy_selected_line[1]].x1 := EndPoint2.x;
     lines[multy_selected_line[1]].y1 := EndPoint2.y;
     //Form1.WS.Canvas.Ellipse(Round(EndPoint2.x-5),Round(EndPoint2.y-5),Round(EndPoint2.x+5),Round(EndPoint2.y+5));
  end;

  //Создать новый отрезок
  lines[k].x1 := EndPoint1.x;
  lines[k].x2 := EndPoint2.x;
  lines[k].y1 := EndPoint1.y;
  lines[k].y2 := EndPoint2.y;
  lines[k].col := lines[multy_selected_line[1]].col;
  lines[k].style := lines[multy_selected_line[1]].style;
  lines[k].Width := lines[multy_selected_line[1]].Width;
  lines[k].id := id;

  k+=1;
  id+=1;

  clear;
end;

//Добавить скругление между 2 отрезками из массива мультивыделения
procedure RoundEdge();

var
  CrossPoint : MyTPoint;
  rad : double;

  LKas : double; //расстояние от точки пересечения до касания окружности

  C : MyTPoint; //центр дуги скругления

  x1,y1,x2,y2 : double; //точки 1 пар. отрезка
  x3,y3,x4,y4 : double; //точки 2 пар. отрезка
  angle_vrem : double; //врем. угол наклона отрезка
  EndPoint1, EndPoint2 : MyTPoint; //Точки касания отрезков окружностью

begin
   CrossPoint := PointCross2Line(multy_selected_line[0],
                                 multy_selected_line[1], false);

  //Проверка  что точка пересечения находится на конце отрезков
   if ((abs(CrossPoint.x - lines[multy_selected_line[0]].x1) > 0.0001) and
       (abs(CrossPoint.y - lines[multy_selected_line[0]].y1) > 0.0001)) and
      ((abs(CrossPoint.x - lines[multy_selected_line[0]].x2) > 0.0001) and
       (abs(CrossPoint.x - lines[multy_selected_line[0]].y2) > 0.0001)) and
      ((abs(CrossPoint.x - lines[multy_selected_line[1]].x1) > 0.0001) and
       (abs(CrossPoint.y - lines[multy_selected_line[1]].y1) > 0.0001)) and
      ((abs(CrossPoint.x - lines[multy_selected_line[1]].x2) > 0.0001) and
       (abs(CrossPoint.y - lines[multy_selected_line[1]].y2) > 0.0001)) then
     begin
       showmessage('Отрезки должны иметь 1 общую КОНЕЧНУЮ точку');
       exit;
     end;
   ////////////////////////////////////////////////////////////////////////////////

   rad := StrToFloat(InputBox('Построение скругления', 'Укажи радиус','0'));
   if rad <= 0 then exit;

   /////////////////////////////////////////////////////////////////////////////////

   //1 парал. отрезок
   angle_vrem := ArcTan2(lines[multy_selected_line[0]].y2-lines[multy_selected_line[0]].y1,
                         lines[multy_selected_line[0]].x2-lines[multy_selected_line[0]].x1)
                        + DegToRad(90);

   if (SideDot(lines[multy_selected_line[0]].x1,
               lines[multy_selected_line[0]].y1,
               lines[multy_selected_line[0]].x2,
               lines[multy_selected_line[0]].y2,
               lines[multy_selected_line[1]].x1,
               lines[multy_selected_line[1]].y1) < 0) or
      (SideDot(lines[multy_selected_line[0]].x1,
               lines[multy_selected_line[0]].y1,
               lines[multy_selected_line[0]].x2,
               lines[multy_selected_line[0]].y2,
               lines[multy_selected_line[1]].x2,
               lines[multy_selected_line[1]].y2) < 0) then
      begin
             x1 := lines[multy_selected_line[0]].x1 + rad * cos(angle_vrem);
             y1 := lines[multy_selected_line[0]].y1 + rad * sin(angle_vrem);
             x2 := lines[multy_selected_line[0]].x2 + rad * cos(angle_vrem);
             y2 := lines[multy_selected_line[0]].y2 + rad * sin(angle_vrem);
      end;

   if (SideDot(lines[multy_selected_line[0]].x1,
               lines[multy_selected_line[0]].y1,
               lines[multy_selected_line[0]].x2,
               lines[multy_selected_line[0]].y2,
               lines[multy_selected_line[1]].x1,
               lines[multy_selected_line[1]].y1) > 0) or
      (SideDot(lines[multy_selected_line[0]].x1,
               lines[multy_selected_line[0]].y1,
               lines[multy_selected_line[0]].x2,
               lines[multy_selected_line[0]].y2,
               lines[multy_selected_line[1]].x2,
               lines[multy_selected_line[1]].y2) > 0) then
      begin
             x1 := lines[multy_selected_line[0]].x1 - rad * cos(angle_vrem);
             y1 := lines[multy_selected_line[0]].y1 - rad * sin(angle_vrem);
             x2 := lines[multy_selected_line[0]].x2 - rad * cos(angle_vrem);
             y2 := lines[multy_selected_line[0]].y2 - rad * sin(angle_vrem);
      end;


   //Form1.WS.Canvas.Line(round(x1),round(y1),round(x2),round(y2));

   ///////////////////////////////////////////////////////////////////////////////////////

   //2 парал. отрезок
   angle_vrem := ArcTan2(lines[multy_selected_line[1]].y2-lines[multy_selected_line[1]].y1,
                         lines[multy_selected_line[1]].x2-lines[multy_selected_line[1]].x1)
                        + DegToRad(90);

   if (SideDot(lines[multy_selected_line[1]].x1,
               lines[multy_selected_line[1]].y1,
               lines[multy_selected_line[1]].x2,
               lines[multy_selected_line[1]].y2,
               lines[multy_selected_line[0]].x1,
               lines[multy_selected_line[0]].y1) < 0) or
      (SideDot(lines[multy_selected_line[1]].x1,
               lines[multy_selected_line[1]].y1,
               lines[multy_selected_line[1]].x2,
               lines[multy_selected_line[1]].y2,
               lines[multy_selected_line[0]].x2,
               lines[multy_selected_line[0]].y2) < 0) then
      begin
             x3 := lines[multy_selected_line[1]].x1 + rad * cos(angle_vrem);
             y3 := lines[multy_selected_line[1]].y1 + rad * sin(angle_vrem);
             x4 := lines[multy_selected_line[1]].x2 + rad * cos(angle_vrem);
             y4 := lines[multy_selected_line[1]].y2 + rad * sin(angle_vrem);
      end;

   if (SideDot(lines[multy_selected_line[1]].x1,
               lines[multy_selected_line[1]].y1,
               lines[multy_selected_line[1]].x2,
               lines[multy_selected_line[1]].y2,
               lines[multy_selected_line[0]].x1,
               lines[multy_selected_line[0]].y1) > 0) or
      (SideDot(lines[multy_selected_line[1]].x1,
               lines[multy_selected_line[1]].y1,
               lines[multy_selected_line[1]].x2,
               lines[multy_selected_line[1]].y2,
               lines[multy_selected_line[0]].x2,
               lines[multy_selected_line[0]].y2) > 0) then
      begin
             x3 := lines[multy_selected_line[1]].x1 - rad * cos(angle_vrem);
             y3 := lines[multy_selected_line[1]].y1 - rad * sin(angle_vrem);
             x4 := lines[multy_selected_line[1]].x2 - rad * cos(angle_vrem);
             y4 := lines[multy_selected_line[1]].y2 - rad * sin(angle_vrem);
      end;

   //Form1.WS.Canvas.Line(round(x3),round(y3),round(x4),round(y4));

   ///////////////////////////////////////////////////////////////////////////

   C := PointCross2Line(x1,y1,x2,y2,x3,y3,x4,y4);

   Lkas := sqrt((sqr(c.x-CrossPoint.x) + sqr(c.y-CrossPoint.y))-sqr(rad));

   /////////////////////////////////////////////////////////////////////////////

   //Для 1 отрезка
    angle_vrem := ArcTan2((lines[multy_selected_line[0]].y2-lines[multy_selected_line[0]].y1),
                          (lines[multy_selected_line[0]].x2-lines[multy_selected_line[0]].x1));

    //Наверное, какие-то ошибки с числами с большим кол-вом знаков, если
    //просто сравнивать числа (a = b) , то они не будут одинаковые
    if (abs(CrossPoint.x - lines[multy_selected_line[0]].x2) < 0.0001) and
       (abs(CrossPoint.y - lines[multy_selected_line[0]].y2) < 0.0001) then
    begin
       EndPoint1.x := lines[multy_selected_line[0]].x2 - Lkas * cos(angle_vrem);
       EndPoint1.y := lines[multy_selected_line[0]].y2 - Lkas * sin(angle_vrem);
       lines[multy_selected_line[0]].x2 := EndPoint1.x;
       lines[multy_selected_line[0]].y2 := EndPoint1.y;
       //Form1.WS.Canvas.Ellipse(Round(EndPoint1.x-5),Round(EndPoint1.y-5),Round(EndPoint1.x+5),Round(EndPoint1.y+5));
    end;

    if (abs(CrossPoint.x - lines[multy_selected_line[0]].x1) < 0.0001) and
       (abs(CrossPoint.y - lines[multy_selected_line[0]].y1) < 0.0001) then
    begin
       EndPoint1.x := lines[multy_selected_line[0]].x1 + Lkas * cos(angle_vrem);
       EndPoint1.y := lines[multy_selected_line[0]].y1 + Lkas * sin(angle_vrem);
       lines[multy_selected_line[0]].x1 := EndPoint1.x;
       lines[multy_selected_line[0]].y1 := EndPoint1.y;
       //Form1.WS.Canvas.Ellipse(Round(EndPoint1.x-5),Round(EndPoint1.y-5),Round(EndPoint1.x+5),Round(EndPoint1.y+5));
    end;

    //Для 2 отрезка
    angle_vrem := ArcTan2((lines[multy_selected_line[1]].y2-lines[multy_selected_line[1]].y1),
                          (lines[multy_selected_line[1]].x2-lines[multy_selected_line[1]].x1));

    if (abs(CrossPoint.x - lines[multy_selected_line[1]].x2) < 0.0001) and
       (abs(CrossPoint.y - lines[multy_selected_line[1]].y2) < 0.0001) then
    begin
       EndPoint2.x := lines[multy_selected_line[1]].x2 - Lkas * cos(angle_vrem);
       EndPoint2.y := lines[multy_selected_line[1]].y2 - Lkas * sin(angle_vrem);
       lines[multy_selected_line[1]].x2 := EndPoint2.x;
       lines[multy_selected_line[1]].y2 := EndPoint2.y;
       //Form1.WS.Canvas.Ellipse(Round(EndPoint1.x-5),Round(EndPoint1.y-5),Round(EndPoint1.x+5),Round(EndPoint1.y+5));
    end;

    if (abs(CrossPoint.x - lines[multy_selected_line[1]].x1) < 0.0001) and
       (abs(CrossPoint.y - lines[multy_selected_line[1]].y1) < 0.0001) then
    begin
       EndPoint2.x := lines[multy_selected_line[1]].x1 + Lkas * cos(angle_vrem);
       EndPoint2.y := lines[multy_selected_line[1]].y1 + Lkas * sin(angle_vrem);
       lines[multy_selected_line[1]].x1 := EndPoint2.x;
       lines[multy_selected_line[1]].y1 := EndPoint2.y;
       //Form1.WS.Canvas.Ellipse(Round(EndPoint1.x-5),Round(EndPoint1.y-5),Round(EndPoint1.x+5),Round(EndPoint1.y+5));
    end;

    RoundMode := true; //чтобы не спрашивала направление дуги

    //Направление дуги
    if (c.y > CrossPoint.y) then
    begin
      if (EndPoint2.x < EndPoint1.x) then
         Drawing_unit.Draw_arc(myPoint(EndPoint1.x,EndPoint1.y),myPoint(EndPoint2.x,EndPoint2.y),rad)
      else
         Drawing_unit.Draw_arc(myPoint(EndPoint2.x,EndPoint2.y),myPoint(EndPoint1.x,EndPoint1.y),rad);
    end;

    if (c.y < CrossPoint.y) then
    begin
      if (EndPoint2.x < EndPoint1.x) then
         Drawing_unit.Draw_arc(myPoint(EndPoint2.x,EndPoint2.y),myPoint(EndPoint1.x,EndPoint1.y),rad)
      else
         Drawing_unit.Draw_arc(myPoint(EndPoint1.x,EndPoint1.y),myPoint(EndPoint2.x,EndPoint2.y),rad);
    end;

    if (c.y = CrossPoint.y) then
    begin
      if (CrossPoint.x < c.x) then
         begin
            if (EndPoint2.y < EndPoint1.y) then
            Drawing_unit.Draw_arc(myPoint(EndPoint2.x,EndPoint2.y),myPoint(EndPoint1.x,EndPoint1.y),rad)
            else
            Drawing_unit.Draw_arc(myPoint(EndPoint1.x,EndPoint1.y),myPoint(EndPoint2.x,EndPoint2.y),rad);
         end;
      if (CrossPoint.x > c.x) then
         begin
            if (EndPoint2.y < EndPoint1.y) then
            Drawing_unit.Draw_arc(myPoint(EndPoint1.x,EndPoint1.y),myPoint(EndPoint2.x,EndPoint2.y),rad)
            else
            Drawing_unit.Draw_arc(myPoint(EndPoint2.x,EndPoint2.y),myPoint(EndPoint1.x,EndPoint1.y),rad);
         end;
    end;

    RoundMode := false;
    clear;

   //Form1.WS.Canvas.Ellipse(Round(c.x-rad),Round(c.y-rad),Round(c.x+rad),Round(c.y+rad));

end;

//Добавить размер
Procedure AddDimension(x,y : double);

var
   vremA, vremB, vremC : double;
   vremDist : double;
   VremAngle : double;
   VremStr : string = '3';

begin
   If Length(Free_point_ar) = 0 then
   begin
      SetLength(Free_point_ar,1);
      Free_point_ar[0].x := x;
      Free_point_ar[0].y := y;

      Form1.WS.Canvas.Ellipse(Round(x+Screen_x-5), Round(y+Screen_y-5),
                              Round(x+Screen_x+5), Round(y+Screen_y+5));
      Form1.StatusBar1.Panels.Items[0].Text:='Укажи вторую точку';

      exit;
   end;
   If Length(Free_point_ar) = 1 then
   begin
      SetLength(Free_point_ar,2);
      Free_point_ar[1].x := x;
      Free_point_ar[1].y := y;

      Form1.WS.Canvas.Ellipse(Round(x+Screen_x-5), Round(y+Screen_y-5),
                              Round(x+Screen_x+5), Round(y+Screen_y+5));

      Form1.StatusBar1.Panels.Items[0].Text:='Укажи положение размерной линии';
      if NeedHelp then
                  ShowMessage('Укажи положение размерной линии');

      exit;
   end;
   If Length(Free_point_ar) = 2 then
   begin
      save_prev();

      SetLength(Free_point_ar,3);
      Free_point_ar[2].x := x;
      Free_point_ar[2].y := y;

      //Построение размера
      SetLength(DimAr, Length(DimAr)+1);

      DimAr[High(DimAr)].x1 := Free_point_ar[0].x;
      DimAr[High(DimAr)].y1 := Free_point_ar[0].y;

      DimAr[High(DimAr)].x2 := Free_point_ar[1].x;
      DimAr[High(DimAr)].y2 := Free_point_ar[1].y;

      //Определение точек размерной линии//////////////////////////////////
      //длина перпендикуляра до размерной линии
      vremA := sqrt(sqr(Free_point_ar[0].x-Free_point_ar[2].x)+
                    sqr(Free_point_ar[0].y-Free_point_ar[2].y));
      vremB := sqrt(sqr(Free_point_ar[1].x-Free_point_ar[2].x)+
                    sqr(Free_point_ar[1].y-Free_point_ar[2].y));
      vremC := sqrt(sqr(Free_point_ar[0].x-Free_point_ar[1].x)+
                    sqr(Free_point_ar[0].y-Free_point_ar[1].y));
      vremDist := 2*Area3(vremA,vremB,vremC)/vremC;

      //угол наклона перпендикуляра
      if not OrtoDim then
      begin
          VremAngle := CalcAngle(Free_point_ar[0].x,Free_point_ar[0].y,
                                 Free_point_ar[1].x,Free_point_ar[1].y)-DegToRad(180);

          DimAr[High(DimAr)].x3 := Free_point_ar[0].x+vremDist*sin(VremAngle);
          DimAr[High(DimAr)].y3 := Free_point_ar[0].y+vremDist*cos(VremAngle);
      end
      else
      begin
          if (Free_point_ar[2].x > Free_point_ar[0].x) and
             (Free_point_ar[2].x > Free_point_ar[1].x) then
             VremAngle := DegToRad(90)
             else
          if (Free_point_ar[2].x < Free_point_ar[0].x) and
             (Free_point_ar[2].x < Free_point_ar[1].x) then
             VremAngle := DegToRad(270)
             else
          if (Free_point_ar[2].y > Free_point_ar[0].y) and
             (Free_point_ar[2].y > Free_point_ar[1].y) then
             VremAngle := DegToRad(0)
             else
             VremAngle := DegToRad(180);
      end;

      if (OrtoDim) then
      begin
         if (VremAngle = DegToRad(180)) or
            (VremAngle = DegToRad(0)) then
            vremDist := abs(Free_point_ar[2].y - Free_point_ar[0].y);
         if (VremAngle = DegToRad(90)) or
            (VremAngle = DegToRad(270)) then
            vremDist := abs(Free_point_ar[2].x - Free_point_ar[0].x);
      end;

      //Установить угол в зависимости от указанного направления (+180 град)
      If (SideDot(Free_point_ar[0].x,Free_point_ar[0].y,
                 Free_point_ar[1].x, Free_point_ar[1].y,
                 Free_point_ar[2].x, Free_point_ar[2].y) < 0) and
         (SideDot(Free_point_ar[0].x,Free_point_ar[0].y,
                 Free_point_ar[1].x, Free_point_ar[1].y,
                 Free_point_ar[0].x+vremDist*sin(VremAngle),
                 Free_point_ar[0].y+vremDist*cos(VremAngle)) > 0)
         then VremAngle += DegToRad(180);

      //Конечные точки размерной линии
      DimAr[High(DimAr)].x3 := Free_point_ar[0].x+vremDist*sin(VremAngle);
      DimAr[High(DimAr)].y3 := Free_point_ar[0].y+vremDist*cos(VremAngle);
      if not OrtoDim then
      begin
          DimAr[High(DimAr)].x4 := Free_point_ar[1].x+vremDist*sin(VremAngle);
          DimAr[High(DimAr)].y4 := Free_point_ar[1].y+vremDist*cos(VremAngle);
      end
      else
      begin
          if (VremAngle = DegToRad(0)) or
             (VremAngle = DegToRad(180)) then
             begin
                 DimAr[High(DimAr)].y4 := DimAr[High(DimAr)].y3;
                 DimAr[High(DimAr)].x4 := DimAr[High(DimAr)].x2
             end
          else
             begin
                 DimAr[High(DimAr)].y4 := DimAr[High(DimAr)].y2;
                 DimAr[High(DimAr)].x4 := DimAr[High(DimAr)].x3;
             end;
      end;

      //Середина размерной линии
      DimAr[High(DimAr)].nadpCX := min(DimAr[High(DimAr)].x3,DimAr[High(DimAr)].x4) +
                            abs(DimAr[High(DimAr)].x3 - DimAr[High(DimAr)].x4)/2;
      DimAr[High(DimAr)].nadpCY := min(DimAr[High(DimAr)].y3,DimAr[High(DimAr)].y4) +
                            abs(DimAr[High(DimAr)].y3 - DimAr[High(DimAr)].y4)/2;


      //Предварительная отрисовка размера///////////////////////////////////////////
      //Размерная линия
      Form1.WS.Canvas.Line(Round(DimAr[High(DimAr)].x3+Screen_x),Round(DimAr[High(DimAr)].y3+Screen_y),
                           Round(DimAr[High(DimAr)].x4+Screen_x),Round(DimAr[High(DimAr)].y4+Screen_y));
      //Выносные линии
      Form1.WS.Canvas.Line(Round(DimAr[High(DimAr)].x1+Screen_x),Round(DimAr[High(DimAr)].y1+Screen_y),
                           Round(DimAr[High(DimAr)].x3+Screen_x),Round(DimAr[High(DimAr)].y3+Screen_y));
      Form1.WS.Canvas.Line(Round(DimAr[High(DimAr)].x2+Screen_x),Round(DimAr[High(DimAr)].y2+Screen_y),
                           Round(DimAr[High(DimAr)].x4+Screen_x),Round(DimAr[High(DimAr)].y4+Screen_y));
      ///////////////////////////////////////////////////////////////////////////////

      if not OrtoDim then
      DimAr[High(DimAr)].nadp := vremC
      else
      begin
          if (DimAr[High(DimAr)].x3=DimAr[High(DimAr)].x1) or
             (DimAr[High(DimAr)].x4=DimAr[High(DimAr)].x2)
          then
              DimAr[High(DimAr)].nadp := abs(DimAr[High(DimAr)].x2 - DimAr[High(DimAr)].x1)
          else
              DimAr[High(DimAr)].nadp := abs(DimAr[High(DimAr)].y2 - DimAr[High(DimAr)].y1);
      end;

      //Точность отображения

      if inputQuery('Нанесение размера','Укажи количество знаков'+
                    ' после запятой для отображения размера',VremStr) then

      //DimAr[High(DimAr)].NumDigit := StrToInt(InputBox('Нанесение размера','Укажи количество знаков'+
      //                                                 ' после запятой для отображения размера','3'));

          DimAr[High(DimAr)].NumDigit := StrToInt(VremStr)
      else
      begin
          Delete(DimAr,High(DimAr),1);
          clear;
          Reset_mode;
          drawDimensionMode := true;//Режим добавления размера
          Form1.StatusBar1.Panels.Items[0].Text:='Выбери первую точку';
          exit;
      end;

      DimAr[High(DimAr)].id := Id;
      id += 1;

      SetLength(Free_point_ar,0);
      Clear;
      exit;
   end;
end;

//Описаный многоугольник
Procedure OutMultAngle(x,y : double);

var
   r : double; //Радиус окружности
   n : integer; //Сколько граней

   UgolSegm : double;
   i : integer;

begin
   r := StrToFloat(InputBox('Радиус окружности','Введи радиус вписаной окружности','0'));
   if r = 0 then exit;
   n := StrToInt(InputBox('Количество граней','Введи количество граней','0'));
   if n = 0 then exit;

   save_prev(); //Сохранить предыдущее состояние

   r := r / (cos(DegToRad(180/n)));

   UgolSegm := 360 / n;

   SetLength(multy_selected_line,0);

   for i := 0 to n-1 do
   begin
     lines[k].x1 := x+r*sin(DegToRad(UgolSegm));
     lines[k].y1 := y+r*cos(DegToRad(UgolSegm));
     UgolSegm += 360 / n;
     lines[k].x2 := x+r*sin(DegToRad(UgolSegm));
     lines[k].y2 := y+r*cos(DegToRad(UgolSegm));
     lines[k].col := Cvet;
     lines[k].style := LineStyle;
     lines[k].Width := LineWeight;
     lines[k].id := id;

     If Form1.Need_intersect_point.Checked then
      begin
          //проверка пересечения линий
          prov_intersect();
          //проверка пересечения с окружностью
          intersection_line_circ(k);
      end;

     //Занести созданную линию в массив мультивыделения
     setlength(multy_selected_line,length(multy_selected_line)+1);
     multy_selected_line[High(multy_selected_line)] := lines[k].id;

     id += 1;
     k+=1;
   end;

   //Создать точку в центре многоугольника
   points[k_points].x := x;
   points[k_points].y := y;
   points[k_points].id := id;
   id += 1;

   SetLength(multy_selected_point,1);
   multy_selected_point[0] := k_points;

   k_points += 1;

   //Собрать блок
   MakeBlock;

   Clear();
end;

//Вписаный многоугольник
Procedure InMultAngle(x,y : double);

var
   r : double; //Радиус окружности
   n : integer; //Сколько граней

   UgolSegm : double;
   i : integer;

   lastPoint : MyTPoint;

begin
   r := StrToFloat(InputBox('Радиус окружности','Введи радиус описанной окружности','0'));
   if r = 0 then exit;
   n := StrToInt(InputBox('Количество граней','Введи количество граней','0'));
   if n <= 0 then exit;

   save_prev(); //Сохранить предыдущее состояние

   UgolSegm := 360 / n;

   lastPoint.x := -1;
   lastPoint.y := -1;

   for i := 0 to n-1 do
   begin
     if (lastPoint.x = -1) and
        (lastPoint.y = -1) then
     begin
         lines[k].x1 := x+r*sin(DegToRad(UgolSegm));
         lines[k].y1 := y+r*cos(DegToRad(UgolSegm));
     end
     else
     begin
         lines[k].x1 := lastPoint.x;
         lines[k].y1 := lastPoint.y;
     end;

     UgolSegm += 360 / n;
     lines[k].x2 := x+r*sin(DegToRad(UgolSegm));
     lines[k].y2 := y+r*cos(DegToRad(UgolSegm));
     lines[k].col := Cvet;
     lines[k].style := LineStyle;
     lines[k].Width := LineWeight;
     lines[k].id := id;

     If Form1.Need_intersect_point.Checked then
      begin
          //проверка пересечения линий
          prov_intersect();
          //проверка пересечения с окружностью
          intersection_line_circ(k);
      end;

     lastPoint.x := lines[k].x2;
     lastPoint.y := lines[k].y2;

     //Занести созданную линию в массив мультивыделения
     setlength(multy_selected_line,length(multy_selected_line)+1);
     multy_selected_line[High(multy_selected_line)] := lines[k].id;

     id+=1;
     k+=1;
   end;

   //Создать точку в центре многоугольника
   points[k_points].x := x;
   points[k_points].y := y;
   points[k_points].id := id;
   id += 1;

   SetLength(multy_selected_point,1);
   multy_selected_point[0] := k_points;

   k_points += 1;

   //Собрать блок
   MakeBlock;

   Clear();
end;

//Расчет и построение точки середины отрезка
Procedure Midle_point();

var
  x10,y10,x20,y20:float;//старые координаты
  x_ser,y_ser:float;//точка середины

  vrem:float;

begin
   If (selected=10000) then
   begin
     If (NeedHelp) then
     Showmessage('Сначала необходимо выбрать отрезок');
     Exit;
   end;

  If (Selected <> 10000) and (Select_type = line) then
  begin
      x10:=Lines[Selected].x1;
      y10:=Lines[Selected].y1;
      x20:=Lines[Selected].x2;
      y20:=Lines[Selected].y2;

      if x10>x20 then
      begin
      vrem:=x10;
      x10:=x20;
      x20:=vrem;

      vrem:=y10;
      y10:=y20;
      y20:=vrem;
      end;

      //Найти точку середины

      If (x10<>x20) then
      begin
           x_ser:=abs((x20-x10) / 2);
           if x10<x20 then x_ser+=x10 else x_ser+=x20;
      end

      else
      begin
           x_ser:=x10;
      end;

      If (y10<>y20) then
      begin
           y_ser:=abs((y20-y10) / 2);
           if y10<y20 then y_ser+=y10 else y_ser+=y20;
      end

      else
      begin
           y_ser:=y10;
      end;

      Points[k_points].x:=x_ser;
      Points[k_points].y:=y_ser;

      Points[k_points].id := id;
      id += 1;
      k_points+=1;

      Selected:=10000;
      Clear;
  end;
  If selected=10000 then
  begin
    Form1.StatusBar1.Panels.Items[0].Text:='Ничего не выбрано';
  end;
end;

//Нахождение центра окружности, часть которой составляет дуга
//Вспомогательная функция
function GetCenter(Point1,point2 : MyTPoint; R : double) : MyTPoint;

var
  x1,y1,
  x2,y2,

  L, t, xc, yc, vt,
  xa, ya: double;

begin
   x1 := Point1.X; y1:= Point1.Y;
   x2 := Point2.X; y2 := Point2.Y;

    L := sqr(x1 - x2) + sqr(y1 - y2);
    vt := sqr(2*R);

    xc := 1 / (2 * l);
    t := (l - sqr(R) + sqr(R)) * xc;
    l := sqrt(abs(((vt - l) * (l - sqr(R - R))))) * xc;
    xc := (x2 - x1) * t + x1;
    yc := (y2 - y1) * t + y1;
    t := (y2 - y1) * l;
    xa := xc + t;
    t := (x1 - x2) * l;
    ya := yc + t;

   Result.X := xa;
   Result.Y := ya;
end;

//Преобразование из координат XY в тип данных MyTPoint
//Вспомогательная функция
function myPoint(x,y : double) : MyTPoint;
begin
  result.x := x;
  result.y := y;
end;

//Создание дуги
Procedure Draw_arc(Point1,Point2 : MyTPoint; GetRad : double);

var
   T_c : MyTPoint;
   Rect : MyTRect;
   Rad : double;
   MinRad : double;
   NeedSwap : integer = 0;
   VremPoint : MyTPoint;

begin
       If (GetRad > 0) then
       Rad := GetRad
       else
           begin
               MinRad := SQRT((sqr(Point1.x - Point2.x)+sqr(Point1.y - Point2.y)))/2;
               Rad := StrToFloat(InputBox('Параметры дуги','Введи радиус (не меньше '+
               FloatToStr(MinRad) + ')',FloatToStr(MinRad)));
               If (Rad = 0) then
               begin
                  Form1.StatusBar1.Panels.Items[0].Text:='Ничего не выбрано';
                  ShowMessage('Радиус не может быть равен 0');

                  Reset_mode;
                  exit;
               end;
               If (Rad < (MinRad - 0.000001)) then
               begin
                  Form1.StatusBar1.Panels.Items[0].Text:='Ничего не выбрано';
                  ShowMessage('Слишком маленький радиус');

                  clear;
                  Reset_mode;
                  exit;
               end;

           end;

       While (NeedSwap <> 1) do
       begin
             T_c := GetCenter(MyPoint(Point1.x,Point1.Y), MyPoint(Point2.x,Point2.Y),Rad);

             Rect.Left := T_c.X-Rad;
             Rect.Right := T_c.X+Rad;
             Rect.Top := T_c.Y-Rad;
             Rect.Bottom := T_c.Y+Rad;

             //Центр дуги и радиус (для экспорта в скрипт)
             arcs[k_arc].CentrX := T_c.x;
             arcs[k_arc].CentrY := T_c.y;
             arcs[k_arc].rad := Rad;
             //Углы начала и конца дуги (для экспорта в скрипт)
             arcs[k_arc].StartAngle := (CalcAngle(T_c.x, T_c.y, Point1.X, Point1.Y));
             arcs[k_arc].EndAngle   := (CalcAngle(T_c.x, T_c.y, Point2.X, Point2.Y));

             arcs[k_arc].l_x:=Rect.Left;
             arcs[k_arc].r_x:=Rect.Right;
             arcs[k_arc].t_y:=Rect.Top;
             arcs[k_arc].b_y:=Rect.Bottom;

             arcs[k_arc].col:=Cvet;
             arcs[k_arc].style:=LineStyle;
             arcs[k_arc].Width:=LineWeight;

             arcs[k_arc].x1:=Point1.X;
             arcs[k_arc].y1:=Point1.Y;
             arcs[k_arc].x2:=Point2.X;
             arcs[k_arc].y2:=Point2.Y;

             Form1.WS.canvas.Arc(Round(Arcs[k_arc].l_x)+Screen_x,Round(Arcs[k_arc].t_y)+Screen_y,
                                 Round(Arcs[k_arc].r_x)+Screen_x,Round(Arcs[k_arc].b_y)+Screen_y,
                                 Round(Arcs[k_arc].x1)+Screen_x,Round(Arcs[k_arc].y1)+Screen_y,
                                 Round(Arcs[k_arc].x2)+Screen_x,Round(Arcs[k_arc].y2)+Screen_y);

             If (RotMode) or
                (RoundMode) or
                (MirMode) then
             begin
                NeedSwap := 1;
                continue;
             end;

             NeedSwap :=    MessageDlg('Направление изгиба дуги',
                            'Правильно ли направлен изгиб дуги?',
                            mtConfirmation, mbOKCancel,0);



             //Если нужно развернуть дугу
             If NeedSwap = 2 then
             begin
                Form1.WS.Canvas.Pen.Color := clWhite;
                Form1.WS.canvas.Arc(Round(Arcs[k_arc].l_x)+Screen_x,Round(Arcs[k_arc].t_y)+Screen_y,
                                 Round(Arcs[k_arc].r_x)+Screen_x,Round(Arcs[k_arc].b_y)+Screen_y,
                                 Round(Arcs[k_arc].x1)+Screen_x,Round(Arcs[k_arc].y1)+Screen_y,
                                 Round(Arcs[k_arc].x2)+Screen_x,Round(Arcs[k_arc].y2)+Screen_y);
                Form1.WS.Canvas.Pen.Color := Cvet;

                VremPoint.x := Point1.x;
                VremPoint.y := Point1.y;
                Point1.x := Point2.x;
                Point1.y := Point2.y;
                Point2.x := VremPoint.x;
                Point2.y := VremPoint.y;
             end;
       end;
       arcs[k_arc].id := id;
       id += 1;

       k_arc +=1;

       Form1.StatusBar1.Panels.Items[0].Text:='Укажи первую точку';

       Clear;
end;

//Окружность по 2 ординатам
Procedure Draw_circle_2_point(Point1,Point2 : MyTPoint);

var
   center : MyTPoint;
   rad : double;

begin
   center.X := abs(Point1.X+Point2.X) / 2;
   center.Y := abs(Point1.Y+Point2.Y) / 2;
   rad := sqrt(sqr(Point1.X-center.x)+sqr(Point1.Y-center.Y));

   Circles[k_cir].x := center.X;
   Circles[k_cir].y := center.Y;
   Circles[k_cir].r := rad;
   Circles[k_cir].col := Cvet;
   Circles[k_cir].style := LineStyle;
   Circles[k_cir].Width := LineWeight;
   Circles[k_cir].id :=id;
   id += 1;

   k_cir += 1;

   Clear;
end;

//Окружность по 3 точкам (вписаная в треугольник)
Procedure Draw_circle_3_point(Point1, Point2, Point3 : MyTPoint);

var
   LA, LB, LC,
   vrem_x, vrem_y,
   Rad,
   a, b, c, p : double;
   Center : MyTPoint;

begin
    LA := sqrt(sqr(Point1.X-Point2.x)+sqr(Point1.Y-Point2.y));
    LB := sqrt(sqr(Point2.X-Point3.x)+sqr(Point2.Y-Point3.y));
    LC := sqrt(sqr(Point3.X-Point1.x)+sqr(Point3.Y-Point1.y));

    vrem_x := (LB * Point1.X + LC * Point2.X + LA * Point3.X) /
                             (LA + LB + LC);
    vrem_y := (LB * Point1.Y + LC * Point2.Y + LA * Point3.Y) /
                             (LA + LB + LC);

    //Радиус круга
    a := sqrt(sqr(vrem_x-Point2.x)+sqr(vrem_y-Point2.y));
    b := sqrt(sqr(vrem_x-Point1.x)+sqr(vrem_y-Point1.y));
    c := sqrt(sqr(Point2.x-Point1.x)+sqr(Point2.y-Point1.y));
    p := (a+b+c)/2;

    Center.X := vrem_x;
    Center.Y := vrem_y;
    Rad := (2/c)*sqrt(p * (p-a)*(p-b)*(p-c));

    Circles[k_cir].x := center.X;
    Circles[k_cir].y := center.Y;
    Circles[k_cir].r := Rad;
    Circles[k_cir].col := Cvet;
    Circles[k_cir].style := LineStyle;
    Circles[k_cir].Width := LineWeight;
    Circles[k_cir].id := id;
    id += 1;

    k_cir += 1;

    Clear;
end;

//Направляющая через 2 точки
Procedure Ray2Point(x_vrem,y_vrem : double);

var
   koef, beta : double;

begin
     If Length(Two_point_ar)<>2 then
     begin
         SetLength(Two_point_ar,2);
         Two_point_ar[0].X := x_vrem;
         Two_point_ar[0].Y := y_vrem;
     end
     else
     begin
         Two_point_ar[1].X := x_vrem;
         Two_point_ar[1].Y := y_vrem;

         if (Two_point_ar[0].X = Two_point_ar[1].X) then //вертикальная линия
         begin
            Lines[k].x1 := Two_point_ar[0].X;
            Lines[k].x2 := Two_point_ar[0].X;
            Lines[k].y1 := 0;
            Lines[k].y2 := World_size_v;
            lines[k].col := clPurple;

             If Form1.Need_intersect_point.Checked then
             begin
                 //проверка пересечения линий
                 prov_intersect();
                 //проверка пересечения с окружностью
                 intersection_line_circ(k);
             end;

             Lines[k].id := id;
             id += 1;

             k+=1;

             clear;
             SetLength(Two_point_ar,0);

             exit;
         end;

         koef := (Two_point_ar[0].Y-Two_point_ar[1].Y)/
                 (Two_point_ar[0].X-Two_point_ar[1].X);
         beta := Two_point_ar[1].Y-koef*Two_point_ar[1].X;

         lines[k].x1 :=0;
         lines[k].y1 := koef*lines[k].x1+beta;
         lines[k].x2 :=World_size_h;
         lines[k].y2 := koef*lines[k].x2+beta;

         if lines[k].y1<0 then
         begin
            lines[k].y1 := 0;
            lines[k].x1 := (lines[k].y1-beta)/koef
         end;
         if lines[k].y1>World_size_v then
         begin
            lines[k].y1 := World_size_v;
            lines[k].x1 := (lines[k].y1-beta)/koef;
         end;
         if lines[k].x1<0 then
         begin
            lines[k].x1 := 0;
            lines[k].y1 := koef*lines[k].x1+beta;
         end;
         if lines[k].x1>World_size_h then
         begin
            lines[k].x1 := World_size_h;
            lines[k].y1 := koef*lines[k].x1+beta;
         end;
         if lines[k].y2<0 then
         begin
            lines[k].y2 := 0;
            lines[k].x2 := (lines[k].y2-beta)/koef;
         end;
         if lines[k].y2>World_size_v then
         begin
            lines[k].y2 := World_size_v;
            lines[k].x2 := (lines[k].y2-beta)/koef;
         end;
         if lines[k].x2<0 then
         begin
            lines[k].x2 := 0;
            lines[k].y2 := koef*lines[k].x2+beta;
         end;
         if lines[k].x2>World_size_h then
         begin
            lines[k].x2 := World_size_h;
            lines[k].y2 := koef*lines[k].x2+beta;
         end;

         lines[k].col := clPurple;

         If Form1.Need_intersect_point.Checked then
         begin
             //проверка пересечения линий
             prov_intersect();
             //проверка пересечения с окружностью
             intersection_line_circ(k);
         end;

         Lines[k].id := id;
         id += 1;

         k+=1;

         clear;
         SetLength(Two_point_ar,0);
     end;
end;

//Направляющая под углом
Procedure AngleRay(x_vrem, y_vrem, vrem_angle : double);

var
   x_cross_1, y_cross_1,
   x_cross_2, y_cross_2,
   k_alpha : double;

begin
   k_alpha := tan(DegToRad(vrem_angle));

   x_cross_1 := 0;
   if (vrem_angle <> -90) and
      (vrem_angle <> 90) then
      y_cross_1 := k_alpha*x_cross_1-k_alpha*x_vrem+y_vrem
   else
     begin
       y_cross_1 := 0;
       x_cross_1:=x_vrem;
     end;

   x_cross_2 := World_size_h;
   if (vrem_angle <> -90) and
      (vrem_angle <> 90) then
      y_cross_2 := k_alpha*x_cross_2-k_alpha*x_vrem+y_vrem
   else
     begin
          y_cross_2 := World_size_v;
          x_cross_2:=x_vrem;
     end;

   lines[k].x1:=X_cross_1;
   lines[k].y1:=Y_cross_1;

   lines[k].x2:=X_cross_2;
   lines[k].y2:=y_cross_2;
   lines[k].col:=clPurple;

   If Form1.Need_intersect_point.Checked then
   begin
       //проверка пересечения линий
       prov_intersect();
       //проверка пересечения с окружностью
       intersection_line_circ(k);
   end;

   Lines[k].id := id;
   id += 1;

   k+=1;

   clear;
end;

//Рисование прямоугольника
Procedure DrawRect(x_vrem, y_vrem : double);
begin
   near_points(x_vrem,y_vrem,4);

   //1 координата
    if stad=false then
    begin
      Form1.StatusBar1.Panels.Items[0].Text:='Укажи вторую точку';

      x_rect := x_vrem; y_rect := y_vrem;
      stad:=true;

      x_vrem := x_vrem + Screen_x;
      y_vrem := y_vrem + Screen_y;
      Form1.WS.Canvas.Pen.Color:=clRed;
      Form1.WS.Canvas.Ellipse(Round(x_vrem-5),Round(y_vrem+5),
                              Round(x_vrem+5),Round(y_vrem-5));
      Form1.WS.Canvas.Pen.Color:=clBlack;
    end
    else    //2 координата
    begin
      save_prev(); //Сохранить предыдущее состояние

      lines[k].x1:=x_rect;
      lines[k].y1:=y_rect;
      lines[k].x2:=x_vrem;
      lines[k].y2:=y_rect;
      lines[k].col:=Cvet;
      lines[k].Width:=LineWeight;
      lines[k].style:=LineStyle;

      If Form1.Need_intersect_point.Checked then
      begin
          //проверка пересечения линий
          prov_intersect();
          //проверка пересечения с окружностью
          intersection_line_circ(k);
      end;

      Lines[k].id := id;
      id += 1;

      k+=1;

      lines[k].x1:=x_vrem;
      lines[k].y1:=y_rect;
      lines[k].x2:=x_vrem;
      lines[k].y2:=y_vrem;
      lines[k].col:=Cvet;
      lines[k].Width:=LineWeight;
      lines[k].style:=LineStyle;

      If Form1.Need_intersect_point.Checked then
      begin
          //проверка пересечения линий
          prov_intersect();
          //проверка пересечения с окружностью
          intersection_line_circ(k);
      end;

      Lines[k].id := id;
      id += 1;

      k+=1;

      lines[k].x1:=x_vrem;
      lines[k].y1:=y_vrem;
      lines[k].x2:=x_rect;
      lines[k].y2:=y_vrem;
      lines[k].col:=Cvet;
      lines[k].Width:=LineWeight;
      lines[k].style:=LineStyle;

      If Form1.Need_intersect_point.Checked then
      begin
          //проверка пересечения линий
          prov_intersect();
          //проверка пересечения с окружностью
          intersection_line_circ(k);
      end;

      Lines[k].id := id;
      id += 1;

      k+=1;

      lines[k].x1:=x_rect;
      lines[k].y1:=y_vrem;
      lines[k].x2:=x_rect;
      lines[k].y2:=y_rect;
      lines[k].col:=Cvet;
      lines[k].Width:=LineWeight;
      lines[k].style:=LineStyle;

      If Form1.Need_intersect_point.Checked then
      begin
          //проверка пересечения линий
          prov_intersect();
          //проверка пересечения с окружностью
          intersection_line_circ(k);
      end;

      Lines[k].id := id;
      id += 1;

      SetLength(multy_selected_line,4);
      multy_selected_line[0]:= k-3;
      multy_selected_line[1]:= k-2;
      multy_selected_line[2]:= k-1;
      multy_selected_line[3]:= k;
      MakeBlock;

      draw_rectangle_mode := true;

      k+=1;

      stad:=false;

      Clear();//Очистка формы
      Form1.StatusBar1.Panels.Items[0].Text:='Укажи первую точку';
    end;
end;

//Рисование отрезка
Procedure DrawLine(x_vrem, y_vrem : double);
begin
   near_points(x_vrem,y_vrem,4);

   if (last_point.X<>-1) and
      (last_point.Y<>-1) and
      contin_line   then
      begin
         save_prev(); //Сохранить предыдущее состояние

         Form1.StatusBar1.Panels.Items[0].Text:='Укажи вторую точку';

         lines[k].x1:=last_point.X;lines[k].y1:=last_point.Y;
         stad:=true;
      end;

   //1 координата
   if stad=false then
   begin
     save_prev(); //Сохранить предыдущее состояние

     Form1.StatusBar1.Panels.Items[0].Text:='Укажи вторую точку';

     lines[k].x1:=x_vrem; lines[k].y1:=y_vrem;
     stad:=true;

     Form1.WS.Canvas.Pen.Color:=clRed;

     x_vrem := x_vrem + screen_x;
     y_vrem := y_vrem + screen_y;

     Form1.WS.Canvas.Ellipse(Round(x_vrem-5),Round(y_vrem+5),
                             Round(x_vrem+5),Round(y_vrem-5));
     Form1.WS.Canvas.Pen.Color:=clBlack;
   end
   //2 координата
   else
   begin
     if (Orto_graph = true) then
     begin
       if abs(x_vrem-Lines[k].x1)<abs(y_vrem-Lines[k].y1) then x_vrem:=Lines[k].x1
       else y_vrem:=Lines[k].y1; //Совсем ОРТО
     end;

     lines[k].x2:=x_vrem;lines[k].y2:=y_vrem;
     lines[k].col:=Cvet;
     lines[k].Width := LineWeight;
     lines[k].style := LineStyle;

     stad:=false;

     If Form1.Need_intersect_point.Checked then
     begin
         //проверка пересечения линий
         prov_intersect();
         //проверка пересечения с окружностью
         intersection_line_circ(k);
     end;

     Clear();//Очистка формы

     Lines[k].id := id;
     id += 1;

      k+=1;

      last_point.X := x_vrem;
      last_point.Y := y_vrem;

      if (lines[k-1].x1 = lines[k-1].x2) and  //Линия нулевой длины
         (lines[k-1].y1 = lines[k-1].y2) then //удаляется из массива
         begin
            k-=1;
            delete(lines,k,1);
            setlength(lines,arr_length);
         end;

      if contin_line then
         Form1.StatusBar1.Panels.Items[0].Text:='Укажи следующую точку'
      else
          Form1.StatusBar1.Panels.Items[0].Text:='Укажи первую точку';
   end;
end;

//Рисование окружности
Procedure DrawCirc(x_vrem, y_vrem : double);
begin
   save_prev(); //Сохранить предыдущее состояние

   If stad_cir=false then //1 координата
   begin
       near_points(x_vrem,y_vrem,4);

       Form1.WS.Canvas.Pen.Color:=clRed;
       Circles[k_cir].x:=x_vrem; Circles[k_cir].y:=y_vrem;
       Circles[k_cir].col:=Cvet;
       Form1.WS.Canvas.Pen.Color:=clBlack;

       x_vrem := x_vrem + screen_x;
       y_vrem := y_vrem + screen_y;

       Form1.WS.Canvas.Ellipse(Round(x_vrem-5),Round(y_vrem+5),
                               Round(x_vrem+5),Round(y_vrem-5));

       stad_cir:=true;
   end
   else   //2 координата
   begin
        near_points(x_vrem,y_vrem,4);

        Circles[k_cir].r:=sqrt(Power((Circles[k_cir].x-x_vrem),2)+
                               Power((Circles[k_cir].y-y_vrem),2));
        Circles[k_cir].Width:=LineWeight;
        Circles[k_cir].style:=LineStyle;

        stad_cir:=false;

        Circles[k_cir].id := id;
        id += 1;

        k_cir+=1;

        If Form1.Need_intersect_point.Checked then
        begin
            intersection_circ_line(k_cir-1);
            intersect_2_cir(Circles[k_cir-1].x, Circles[k_cir-1].y,
            Circles[k_cir-1].r);
        end;

        Clear();//Очистка формы
   end;
end;

//Рисовать стрелку
Procedure DrawArrow(x_vrem,y_vrem : double);
begin
   save_prev(); //Сохранить предыдущее состояние

   arrow[k_arrow].t:=arrow_kind;
   arrow[k_arrow].x:=x_vrem;
   arrow[k_arrow].y:=y_vrem;
   arrow[k_arrow].col:=Cvet;
   arrow[k_arrow].id := id;
   id += 1;

   k_arrow+=1;
   clear();
end;

end.

