unit Load_Save;

{
    Vector graphical editor.
    Version: 10.
    Written on FreePascal (https://freepascal.org/).
    Copyright (C) 2024  Warfalomay

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
}

{$mode ObjFPC}{$H+}

interface
//Добавить к текущему чертежу объекты из указанного файла
Procedure Add_vect(f_name:string);
//Загрузить векторный файл
Procedure Load_vect(f_name_op:string);
//Сохраниить векторный файл
Procedure Save_vect(f_name:string);
//Сохранить скрипт VB-Nanocad
Procedure SaveScript();
//Сохранить растровый фрагмент в буфер
//Создает графичекий файл в буфере обмена, куда отрисовывает выбранные объекты
Procedure SaveToBuf();


implementation
Uses
Graphics,Unit1,Classes, SysUtils, StrUtils, My_modify, Dialogs,
Clipbrd, math;

//Сохранить растровый фрагмент в буфер
//Создает графичекий файл в буфере обмена, куда отрисовывает выбранные объекты
Procedure SaveToBuf();

var
   vremPic, PicX : TBitmap;
   count : integer;
   xMin : float = 100000;
   xMax : float = -1;
   yMin : float = 100000;
   yMax : float = -1;

   LeftBorder, RightBorder,
   TopBorder, BottomBorder : float;

   TextWidth : byte;
   TextHeight : byte;
   TextText : string;

begin
   vremPic := TBitmap.Create;
   PicX := TBitmap.Create;
   //
   vremPic.Width := 1000;
   vremPic.Height := 1000;

   //Расчет размера изображения для буфера
   //По линиям
   for count := 0 to High(multy_selected_line) do
   begin
       xMin := min(Round(lines[multy_selected_line[count]].x1)+Screen_x,xMin);
       xMin := min(Round(lines[multy_selected_line[count]].x2)+Screen_x,xMin);
       yMin := min(Round(lines[multy_selected_line[count]].y1)+Screen_y,yMin);
       yMin := min(Round(lines[multy_selected_line[count]].y2)+Screen_y,yMin);

       xMax := max(Round(lines[multy_selected_line[count]].x1)+Screen_x,xMax);
       xMax := max(Round(lines[multy_selected_line[count]].x2)+Screen_x,xMax);
       yMax := max(Round(lines[multy_selected_line[count]].y1)+Screen_y,yMax);
       yMax := max(Round(lines[multy_selected_line[count]].y2)+Screen_y,yMax);
   end;
   //По окружностям
   for count := 0 to High(multy_selected_circle) do
   with circles[multy_selected_circle[count]] do
   begin
       xMin := min(x-r+Screen_x,xMin);
       xMax := max(x+r+Screen_x,xMax);
       yMin := min(y-r+Screen_y,yMin);
       yMax := max(y+r+Screen_y,yMax);
   end;
   //По точкам
   for count := 0 to High(multy_selected_point) do
   with points[multy_selected_point[count]] do
   begin
       xMin := min(x+Screen_x,xMin);
       xMax := max(x+Screen_x,xMax);
       yMin := min(y+Screen_y,yMin);
       yMax := max(y+Screen_y,yMax);
   end;
   //По дугам
   for count := 0 to High(multy_selected_arcs) do
   with arcs[multy_selected_arcs[count]] do
   begin
       xMin := min(l_x+Screen_x,xMin);
       xMax := max(r_x+Screen_x,xMax);
       yMin := min(t_y+Screen_y,yMin);
       yMax := max(b_y+Screen_y,yMax);
   end;
   //По текстам
   for count := 0 to High(multy_selected_text) do
   with nadp[multy_selected_text[count]] do
   begin
       xMin := min(x+Screen_x,xMin);
       xMax := max(x+h*Length(Utf8ToAnsi(s))+Screen_x,xMax);
       yMin := min(y+Screen_y,yMin);
       yMax := max(y+h+Screen_y,yMax);
   end;
   //По стрелкам
   for count := 0 to High(multy_selected_arrow) do
   With arrow[multy_selected_arrow[count]] do
   begin
       xMin := min(x-30+Screen_x,xMin);
       xMax := max(x+30+Screen_x,xMax);
       yMin := min(y-30+Screen_y,yMin);
       yMax := max(y+30+Screen_y,yMax);
   end;
   //По размерам
   for count := 0 to High(multy_selected_dim) do
   With DimAr[multy_selected_dim[count]] do
   begin
       xMin := min(Round(DimAr[multy_selected_dim[count]].x1)+Screen_x,xMin);
       xMin := min(Round(DimAr[multy_selected_dim[count]].x2)+Screen_x,xMin);
       xMin := min(Round(DimAr[multy_selected_dim[count]].x3)+Screen_x,xMin);
       xMin := min(Round(DimAr[multy_selected_dim[count]].x4)+Screen_x,xMin);

       yMin := min(Round(DimAr[multy_selected_dim[count]].y1)+Screen_y,yMin);
       yMin := min(Round(DimAr[multy_selected_dim[count]].y2)+Screen_y,yMin);
       yMin := min(Round(DimAr[multy_selected_dim[count]].y3)+Screen_y,yMin);
       yMin := min(Round(DimAr[multy_selected_dim[count]].y4)+Screen_y,yMin);

       xMax := max(Round(DimAr[multy_selected_dim[count]].x1)+Screen_x,xMax);
       xMax := max(Round(DimAr[multy_selected_dim[count]].x2)+Screen_x,xMax);
       xMax := max(Round(DimAr[multy_selected_dim[count]].x3)+Screen_x,xMax);
       xMax := max(Round(DimAr[multy_selected_dim[count]].x4)+Screen_x,xMax);

       yMax := max(Round(DimAr[multy_selected_dim[count]].y1)+Screen_y,yMax);
       yMax := max(Round(DimAr[multy_selected_dim[count]].y2)+Screen_y,yMax);
       yMax := max(Round(DimAr[multy_selected_dim[count]].y3)+Screen_y,yMax);
       yMax := max(Round(DimAr[multy_selected_dim[count]].y4)+Screen_y,yMax);
   end;

   LeftBorder := xMin + Screen_x - 10;
   RightBorder := xMax + Screen_x + 10;
   TopBorder := yMin + Screen_y - 10;
   BottomBorder := yMax + Screen_y + 10;

   vremPic.Canvas.Brush.Color := clWhite;
   vremPic.Canvas.Rectangle(0,0,vremPic.Width,vremPic.Height);
   //
   //Отрисовка линий во фрагмент
   for count := 0 to High(multy_selected_line) do
   begin
       vremPic.Canvas.Pen.Color := lines[multy_selected_line[count]].col;
       vremPic.Canvas.Pen.Style := lines[multy_selected_line[count]].style;
       vremPic.Canvas.Pen.Width := lines[multy_selected_line[count]].Width;
       With lines[multy_selected_line[count]] do
            vremPic.Canvas.Line(Round(x1)+Screen_x, Round(y1) + Screen_y, Round(x2)+Screen_x, Round(y2)+Screen_y);
   end;
   //Отрисовка окружностей во фрагмент
   for count := 0 to High(multy_selected_circle) do
   begin
       vremPic.Canvas.Pen.Color := circles[multy_selected_circle[count]].col;
       vremPic.Canvas.Pen.Style := circles[multy_selected_circle[count]].style;
       vremPic.Canvas.Pen.Width := circles[multy_selected_circle[count]].Width;
       vremPic.Canvas.Brush.Style := bsClear;
       With circles[multy_selected_circle[count]] do
            vremPic.Canvas.Ellipse(Round(x-r)+Screen_x, Round(y-r)+Screen_y,
                                   Round(x+r)+Screen_x, Round(y+r)+Screen_y);
   end;
   //Отрисовка точек во фрагмент
   for count := 0 to High(multy_selected_point) do
   begin
       vremPic.Canvas.Ellipse(Round(points[multy_selected_point[count]].x)-3+Screen_x,
                              Round(points[multy_selected_point[count]].y)-3+Screen_y,
                              Round(points[multy_selected_point[count]].x)+3+Screen_x,
                              Round(points[multy_selected_point[count]].y)+3+Screen_y);
   end;
   //Отрисовка текстов во фрагмент
   for count := 0 to High(multy_selected_text) do
   begin
       vremPic.Canvas.Font.Color := nadp[count].col;
       vremPic.Canvas.Font.Size := Round(nadp[count].h);
       vremPic.Canvas.TextOut(Round(nadp[multy_selected_text[count]].x)+Screen_x,
                              Round(nadp[multy_selected_text[count]].y)+Screen_y,
                              nadp[multy_selected_text[count]].s);
   end;
   //Отрисовка стрелок во фрагмент
   for count := 0 to High(multy_selected_arrow) do
   begin
       vremPic.Canvas.Pen.Color := arrow[multy_selected_arrow[count]].col;

       With arrow[multy_selected_arrow[count]] do
       begin
           If arrow[multy_selected_arrow[count]].t='left' then
               begin
                   vremPic.Canvas.Line(Round(x+Screen_x),Round(y+Screen_y),
                                       Round(x+30+Screen_x),Round(y+Screen_y));
                   vremPic.Canvas.Line(Round(x+Screen_x),Round(y+Screen_y),
                                       Round(x+10+Screen_x),Round(y+5+Screen_y));
                   vremPic.Canvas.Line(Round(x+Screen_x),Round(y+Screen_y),
                                       Round(x+10+Screen_x),Round(y-5+Screen_y));
               end;
           If arrow[multy_selected_arrow[count]].t='right' then
               begin
                   vremPic.Canvas.Line(Round(x+Screen_x),Round(y+Screen_y),
                                       Round(x-30+Screen_x),Round(y+Screen_y));
                   vremPic.Canvas.Line(Round(x+Screen_x),Round(y+Screen_y),
                                       Round(x-10+Screen_x),Round(y+5+Screen_y));
                   vremPic.Canvas.Line(Round(x+Screen_x),Round(y+Screen_y),
                                       Round(x-10+Screen_x),Round(y-5+Screen_y));
               end;
           If arrow[multy_selected_arrow[count]].t='up' then
               begin
                   vremPic.Canvas.Line(Round(x+Screen_x),Round(y+Screen_y),
                                       Round(x+Screen_x),Round(y+30+Screen_y));
                   vremPic.Canvas.Line(Round(x+Screen_x),Round(y+Screen_y),
                                       Round(x+5+Screen_x),Round(y+10+Screen_y));
                   vremPic.Canvas.Line(Round(x+Screen_x),Round(y+Screen_y),
                                       Round(x-5+Screen_x),Round(y+10+Screen_y));
               end;
           If arrow[multy_selected_arrow[count]].t='down' then
               begin
                   vremPic.Canvas.Line(Round(x+Screen_x),Round(y+Screen_y),
                                       Round(x+Screen_x),Round(y-30+Screen_y));
                   vremPic.Canvas.Line(Round(x+Screen_x),Round(y+Screen_y),
                                       Round(x+5+Screen_x),Round(y-10+Screen_y));
                   vremPic.Canvas.Line(Round(x+Screen_x),Round(y+Screen_y),
                                       Round(x-5+Screen_x),Round(y-10+Screen_y));
               end;
       end;
   end;
   //Отрисовка дуг во фрагмент
   for count := 0 to High(multy_selected_arcs) do
   begin
       vremPic.Canvas.Pen.Color := arcs[multy_selected_arcs[count]].col;
       vremPic.Canvas.Pen.Style := arcs[multy_selected_arcs[count]].style;
       vremPic.Canvas.Pen.Width := arcs[multy_selected_arcs[count]].Width;
       With arcs[multy_selected_arcs[count]] do
            vremPic.canvas.Arc(Round(l_x+Screen_x),Round(t_y+Screen_y),Round(r_x+Screen_x),
                               Round(b_y+Screen_y),Round(x1+Screen_x),Round(y1+Screen_y),
                               Round(x2+Screen_x),Round(y2+Screen_y));
   end;
   //Отрисовка размеров во фрагмент
   for count := 0 to High(multy_selected_dim) do
   begin
       With DimAr[multy_selected_dim[count]] do
       begin
         vremPic.Canvas.Ellipse(Round(DimAr[count].x3-5), Round(DimAr[count].y3-5),
                                 Round(DimAr[count].x3+5), Round(DimAr[count].y3+5));
         vremPic.Canvas.Ellipse(Round(DimAr[count].x4-5), Round(DimAr[count].y4-5),
                                 Round(DimAr[count].x4+5), Round(DimAr[count].y4+5));
         //Размерная линия
         vremPic.Canvas.Line(Round(DimAr[count].x3),Round(DimAr[count].y3),
                          Round(DimAr[count].x4),Round(DimAr[count].y4));
         //Выносные линии
         vremPic.Canvas.Line(Round(DimAr[count].x1),Round(DimAr[count].y1),
                          Round(DimAr[count].x3),Round(DimAr[count].y3));
         vremPic.Canvas.Line(Round(DimAr[count].x2),Round(DimAr[count].y2),
                          Round(DimAr[count].x4),Round(DimAr[count].y4));

         //Вывод текста размера
         TextText := FloatToStr(SimpleRoundTo(DimAr[count].nadp,-DimAr[count].NumDigit));
         TextWidth := vremPic.Canvas.TextWidth(TextText);
         TextHeight := vremPic.Canvas.TextHeight(TextText);
         vremPic.Canvas.Brush.Style := bsSolid;
         vremPic.Canvas.Font.Color:= clBlue;
         vremPic.Canvas.TextOut(Round(DimAr[count].nadpCX - TextWidth/2), Round(DimAr[count].nadpCY-TextHeight / 2),
                             TextText);
       end;
   end;


   picX.Width := Round(RightBorder - LeftBorder);
   picX.Height := Round(BottomBorder - TopBorder);
   PicX.Canvas.CopyRect(Rect(0,0,PicX.Width,PicX.Height),
                            VremPic.Canvas,
                            Rect(Round(LeftBorder),Round(TopBorder),
                            Round(RightBorder),Round(BottomBorder)));

   Clipboard.Assign(PicX);

   vremPic.Free;
   PicX.Free;

   showmessage('Фрагмент скопирован в буфер');
end;

//Добавить к текущему чертежу объекты из указанного файла
procedure Add_vect(f_name:string);

var
  f:Text;
  s:string;

  ar5:array of string;
  i:integer;

  vrem : array of String;
  count : integer;
  lastBlockNum : integer;
  lastId : integer;

begin
     AssignFile(f,f_name);
     Reset(f);
         Readln(f,s);//Первый заголовок

         lastId := Id;

         i:=k;
         While (true) do //Загрузка линий
         begin
           Readln(f,s);
           if s= 'Circles' then break;

           ar5:=SplitString(s,'/');
           Lines[i].x1:=StrToFloat(ar5[0],my);
           Lines[i].y1:=StrToFloat(ar5[1],my);
           Lines[i].x2:=StrToFloat(ar5[2],my);
           Lines[i].y2:=StrToFloat(ar5[3],my);
           Lines[i].col:=StrToInt(ar5[4]);
           Lines[i].Width:=StrToInt(ar5[5]);
           case ar5[6] of
            'psSolid' : Lines[i].style := psSolid;
            'psDash' : Lines[i].style := psDash;
            'psDot' : Lines[i].style := psDot;
            'psDashDot' : Lines[i].style := psDashDot;
            'psDashDotDot' : Lines[i].style := psDashDotDot;
           end;
           Lines[i].id := LastId+StrToInt(ar5[7]);
           id += 1;

           i+=1;
         end;
         k:=i;

         i:=k_cir;
         While (true) do  //Загрузка окружностей
         begin
           Readln(f,s);
           if s = 'Arcs' then break;

           ar5:=SplitString(s,'/');
           Circles[i].x:=StrToFloat(ar5[0],my);
           Circles[i].y:=StrToFloat(ar5[1],my);
           Circles[i].r:=StrToFloat(ar5[2],my);
           Circles[i].col:=StrToInt(ar5[3]);
           Circles[i].Width:=StrToInt(ar5[4]);
           case ar5[5] of
            'psSolid' : Circles[i].style := psSolid;
            'psDash' : Circles[i].style := psDash;
            'psDot' : Circles[i].style := psDot;
            'psDashDot' : Circles[i].style := psDashDot;
            'psDashDotDot' : Circles[i].style := psDashDotDot;
           end;
           Circles[i].id := LastId+ + StrToInt(ar5[6]);
           id += 1;
           i+=1;
         end;
         k_cir:=i;

         i:=k_arc;
         While (true) do  //Загрузка дуг
         begin
           Readln(f,s);
           if s = 'Points' then break;

           ar5:=SplitString(s,'/');
           Arcs[i].x1:=StrToFloat(ar5[0],my);
           Arcs[i].y1:=StrToFloat(ar5[1],my);
           Arcs[i].x2:=StrToFloat(ar5[2],my);
           Arcs[i].y2:=StrToFloat(ar5[3],my);

           Arcs[i].l_x:=StrToFloat(ar5[4],my);
           Arcs[i].r_x:=StrToFloat(ar5[5],my);
           Arcs[i].t_y:=StrToFloat(ar5[6],my);
           Arcs[i].b_y:=StrToFloat(ar5[7],my);

           Arcs[i].col:=StrToInt(ar5[8]);
           Arcs[i].Width:=StrToInt(ar5[9]);

           case ar5[10] of
            'psSolid' : Arcs[i].style := psSolid;
            'psDash' : Arcs[i].style := psDash;
            'psDot' : Arcs[i].style := psDot;
            'psDashDot' : Arcs[i].style := psDashDot;
            'psDashDotDot' : Arcs[i].style := psDashDotDot;
           end;

           Arcs[i].CentrX := StrToFloat(ar5[11],my);
           Arcs[i].CentrY := StrToFloat(ar5[12],my);
           Arcs[i].rad := StrToFloat(ar5[13],my);
           Arcs[i].StartAngle := StrToFloat(ar5[14],my);
           Arcs[i].EndAngle := StrToFloat(ar5[15],my);

           Arcs[i].id := LastId+ + StrToInt(ar5[16]);
           id += 1;

           i+=1;
         end;
         k_arc:=i;

         i:=k_points;
         While (true) do  //Загрузка точек
         begin
           Readln(f,s);
           if s = 'Texts' then break;

           ar5:=SplitString(s,'/');
           points[i].x:=StrToFloat(ar5[0],my);
           points[i].y:=StrToFloat(ar5[1],my);
           points[i].id :=LastId+StrToInt(ar5[2]);
           id += 1;
           i+=1;
         end;
         k_points:=i;

         i:=k_nadp;
         While (true) do  //Загрузка текстов
         begin
           Readln(f,s);
           if s = 'Arrow' then break;

           ar5:=SplitString(s,'/');
           nadp[i].x:=StrToFloat(ar5[0],my);
           nadp[i].y:=StrToFloat(ar5[1],my);
           nadp[i].h:=StrToFloat(ar5[2],my);
           nadp[i].s:=ar5[3];
           nadp[i].col:=StrToInt(ar5[4]);
           nadp[i].id :=LastId+StrToInt(ar5[5]);
           id += 1;
           i+=1;
         end;
         k_nadp:=i;

         i:=k_arrow;
         While (true) do  //Загрузка стрелок
         begin
           Readln(f,s);
           if s = 'Dimension' then break;

           ar5:=SplitString(s,'/');
           arrow[i].x:=StrToFloat(ar5[0],my);
           arrow[i].y:=StrToFloat(ar5[1],my);
           arrow[i].t:=ar5[2];
           arrow[i].col:=StrToInt(ar5[3]);
           arrow[i].id :=LastId+StrToInt(ar5[4]);
           id += 1;
           i+=1;
         end;
         k_arrow:=i;

         While (true) do  //Загрузка размеров
         begin
            Readln(f,s);
            if s = 'Block' then break;

            SetLength(DimAr,Length(DimAr)+1);

            ar5:=SplitString(s,'/');
            DimAr[High(DimAr)].x1:=StrToFloat(ar5[0],my);
            DimAr[High(DimAr)].y1:=StrToFloat(ar5[1],my);
            DimAr[High(DimAr)].x2:=StrToFloat(ar5[2],my);
            DimAr[High(DimAr)].y2:=StrToFloat(ar5[3],my);
            DimAr[High(DimAr)].x3:=StrToFloat(ar5[4],my);
            DimAr[High(DimAr)].y3:=StrToFloat(ar5[5],my);
            DimAr[High(DimAr)].x4:=StrToFloat(ar5[6],my);
            DimAr[High(DimAr)].y4:=StrToFloat(ar5[7],my);
            DimAr[High(DimAr)].nadpCX:=StrToFloat(ar5[8],my);
            DimAr[High(DimAr)].nadpCY:=StrToFloat(ar5[9],my);
            DimAr[High(DimAr)].nadp:=StrToFloat(ar5[10],my);
            DimAr[High(DimAr)].NumDigit:=StrToInt(ar5[11]);
            DimAr[High(DimAr)].id:=StrToInt(ar5[12]);
            id += 1;
         end;

         if length(blocks)>0 then
            lastBlockNum := (blocks[High(blocks)].num)
         else
             lastBlockNum := 0;

         While (true) do //Загрузка блоков
         begin
           Readln(f,s);
              if s = 'Size' then break;
           SetLength(blocks,Length(blocks)+1);
           ar5:=SplitString(s,'/');
           blocks[High(blocks)].num := StrToInt(ar5[0])+lastBlockNum; //Номер блока

           for count := 1 to High(ar5) do
           begin
             SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
             vrem := SplitString(ar5[count],'_');
             blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0]:=StrToInt(vrem[0])+LastId;
             blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1]:=StrToInt(vrem[1]);
           end;
         end;

         Readln(f,loc);Readln(f,loc);//Пропуск размера чертежа

         Readln(f,loc); //Загрузка имени файла подложки

         CloseFile(f);

          If loc<>'' then //Загрузка подложки
           begin
               back_png:=TPortableNetworkGraphic.Create;
               back_png.LoadFromFile(ExtractFilePath(f_name)+loc);
               podl:=true;
               Form1.Clear_podl.Enabled:=true;
           end;
end;

procedure Load_vect(f_name_op:string); //Загрузить файл

var
   f:Text;
   s:string;

   ar5:array of string;
   i:integer;

   vrem : array of String;
   count : integer;

begin

     Clear_all_fig();

     AssignFile(f,f_name_op);
     Reset(f);
     Readln(f,s);//Первый заголовок

     i:=0;
     While (true) do //Загрузка линий
     begin
       Readln(f,s);
          if s= 'Circles' then break;
       ar5:=SplitString(s,'/');
       Lines[i].x1:=StrToFloat(ar5[0],my);
       Lines[i].y1:=StrToFloat(ar5[1],my);
       Lines[i].x2:=StrToFloat(ar5[2],my);
       Lines[i].y2:=StrToFloat(ar5[3],my);
       Lines[i].col:=StrToInt(ar5[4]);
       Lines[i].Width:=StrToInt(ar5[5]);
         case ar5[6] of
          'psSolid' : Lines[i].style := psSolid;
          'psDash' : Lines[i].style := psDash;
          'psDot' : Lines[i].style := psDot;
          'psDashDot' : Lines[i].style := psDashDot;
          'psDashDotDot' : Lines[i].style := psDashDotDot;
         end;
       Lines[i].id := StrToInt(ar5[7]);
       id += 1;

       i+=1;
     end;
     k:=i;

     i:=0;
     While (true) do  //Загрузка окружностей
     begin
       Readln(f,s);
          if s= 'Arcs' then break;
       ar5:=SplitString(s,'/');
       Circles[i].x:=StrToFloat(ar5[0],my);
       Circles[i].y:=StrToFloat(ar5[1],my);
       Circles[i].r:=StrToFloat(ar5[2],my);
       Circles[i].col:=StrToInt(ar5[3]);
       Circles[i].Width:=StrToInt(ar5[4]);
        case ar5[5] of
         'psSolid' : Circles[i].style := psSolid;
         'psDash' : Circles[i].style := psDash;
         'psDot' : Circles[i].style := psDot;
         'psDashDot' : Circles[i].style := psDashDot;
         'psDashDotDot' : Circles[i].style := psDashDotDot;
        end;
        Circles[i].id := StrToInt(ar5[6]);
        id+=1;

       i+=1;
     end;
     k_cir:=i;

     i:=0;
     While (true) do  //Загрузка дуг
     begin
       Readln(f,s);
       if s = 'Points' then break;

       ar5:=SplitString(s,'/');
       Arcs[i].x1:=StrToFloat(ar5[0],my);
       Arcs[i].y1:=StrToFloat(ar5[1],my);
       Arcs[i].x2:=StrToFloat(ar5[2],my);
       Arcs[i].y2:=StrToFloat(ar5[3],my);

       Arcs[i].l_x:=StrToFloat(ar5[4],my);
       Arcs[i].r_x:=StrToFloat(ar5[5],my);
       Arcs[i].t_y:=StrToFloat(ar5[6],my);
       Arcs[i].b_y:=StrToFloat(ar5[7],my);

       Arcs[i].col:=StrToInt(ar5[8]);
       Arcs[i].Width:=StrToInt(ar5[9]);

       case ar5[10] of
        'psSolid' : Arcs[i].style := psSolid;
        'psDash' : Arcs[i].style := psDash;
        'psDot' : Arcs[i].style := psDot;
        'psDashDot' : Arcs[i].style := psDashDot;
        'psDashDotDot' : Arcs[i].style := psDashDotDot;
       end;

       Arcs[i].CentrX := StrToFloat(ar5[11],my);
       Arcs[i].CentrY := StrToFloat(ar5[12],my);
       Arcs[i].rad := StrToFloat(ar5[13],my);
       Arcs[i].StartAngle := StrToFloat(ar5[14],my);
       Arcs[i].EndAngle := StrToFloat(ar5[15],my);

       Arcs[i].id := StrToInt(ar5[16]);
       id += 1;

       i+=1;
     end;
     k_arc:=i;

     i:=0;
     While (true) do  //Загрузка точек
     begin
       Readln(f,s);
          if s = 'Texts' then break;
       ar5:=SplitString(s,'/');
       points[i].x:=StrToFloat(ar5[0],my);
       points[i].y:=StrToFloat(ar5[1],my);
       points[i].id := StrToInt(ar5[2]);
       id += 1;

       i+=1;
     end;
     k_points:=i;

     i:=0;
     While (true) do  //Загрузка текстов
     begin
       Readln(f,s);
          if s = 'Arrow' then break;
       ar5:=SplitString(s,'/');
       nadp[i].x:=StrToFloat(ar5[0],my);
       nadp[i].y:=StrToFloat(ar5[1],my);
       nadp[i].h:=StrToFloat(ar5[2],my);
       nadp[i].s:=ar5[3];
       nadp[i].col:=StrToInt(ar5[4]);
       nadp[i].id := StrToInt(ar5[5]);
       id+=1;

       i+=1;
     end;
     k_nadp:=i;

     i:=0;
     While (true) do  //Загрузка стрелок
     begin
       Readln(f,s);
          if s = 'Dimension' then break;
       ar5:=SplitString(s,'/');
       arrow[i].x:=StrToFloat(ar5[0],my);
       arrow[i].y:=StrToFloat(ar5[1],my);
       arrow[i].t:=ar5[2];
       arrow[i].col:=StrToInt(ar5[3]);
       arrow[i].id := StrToInt(ar5[4]);
       id += 1;

       i+=1;
     end;
     k_arrow:=i;

     While (true) do  //Загрузка размеров
     begin
       Readln(f,s);
          if s = 'Block' then break;

       SetLength(DimAr,Length(DimAr)+1);

       ar5:=SplitString(s,'/');
       DimAr[High(DimAr)].x1:=StrToFloat(ar5[0],my);
       DimAr[High(DimAr)].y1:=StrToFloat(ar5[1],my);
       DimAr[High(DimAr)].x2:=StrToFloat(ar5[2],my);
       DimAr[High(DimAr)].y2:=StrToFloat(ar5[3],my);
       DimAr[High(DimAr)].x3:=StrToFloat(ar5[4],my);
       DimAr[High(DimAr)].y3:=StrToFloat(ar5[5],my);
       DimAr[High(DimAr)].x4:=StrToFloat(ar5[6],my);
       DimAr[High(DimAr)].y4:=StrToFloat(ar5[7],my);
       DimAr[High(DimAr)].nadpCX:=StrToFloat(ar5[8],my);
       DimAr[High(DimAr)].nadpCY:=StrToFloat(ar5[9],my);
       DimAr[High(DimAr)].nadp:=StrToFloat(ar5[10],my);
       DimAr[High(DimAr)].NumDigit:=StrToInt(ar5[11]);
       DimAr[High(DimAr)].id:=StrToInt(ar5[12]);
       id += 1;
     end;


     While (true) do //Загрузка блоков
     begin
       Readln(f,s);
          if s = 'Size' then break;
       SetLength(blocks,Length(blocks)+1);
       ar5:=SplitString(s,'/');
       blocks[High(blocks)].num := StrToInt(ar5[0]); //Номер блока

       for count := 1 to High(ar5) do
       begin
         SetLength(blocks[High(blocks)].id_num, Length(blocks[High(blocks)].id_num)+1,2);
         vrem := SplitString(ar5[count],'_');
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),0]:=StrToInt(vrem[0]);
         blocks[High(blocks)].id_num[High(blocks[High(blocks)].id_num),1]:=StrToInt(vrem[1]);
       end;
     end;

     ///////////////////////////////////////////////////
     Readln(f,s); //Загрузка размера рабочего поля
     ar5:=SplitString(s,'/');
     Form1.WS.Width := StrToInt(ar5[0]);
     Form1.WS.Height := StrToInt(ar5[1]);
     World_size_h := StrToInt(ar5[0]);
     World_size_v := StrToInt(ar5[1]);
     Form1.Scroll_h.Position := 0;
     Form1.Scroll_v.Position := 0;


     if Form1.WS.Width-Form1.Panel1.Width > 0 then
         begin
              Form1.Scroll_h.Max := Form1.WS.Width-Form1.Panel1.Width;
              Form1.Scroll_h.Enabled := true;
         end
      else
        Form1.Scroll_h.Enabled:=false;
      if Form1.WS.Height-Form1.Panel1.Height > 0 then
         begin
              Form1.Scroll_v.Max := Form1.WS.Height-Form1.Panel1.Height;
              Form1.Scroll_v.Enabled := true;
         end
      else
        Form1.Scroll_v.Enabled := false;

     ////////////////////////////////////////////////////

     Readln(f,loc); //Загрузка подложки
     Readln(f,loc);

     CloseFile(f);
     f_name:=f_name_op;

    If loc<>'' then //Загрузка подложки
     begin
         back_png:=TPortableNetworkGraphic.Create;
         back_png.LoadFromFile(ExtractFilePath(f_name)+loc);
         podl:=true;
     end;

    Form1.Start_timer_delay.Enabled:=true;
end;

procedure Save_vect(f_name:string);//Сохранить вектор

var
  f:Text;
  count, count2:integer;

begin
     AssignFile(f,f_name);
     Rewrite(f);

     Writeln(f,'Lines');
     for count:=0 to k-1 do
     Writeln(f,Lines[count].x1,'/',Lines[count].y1,'/',
               Lines[count].x2,'/',Lines[count].y2,'/',
               Lines[count].col,'/',Lines[count].Width,'/',
               Lines[count].style,'/',Lines[count].id);

     Writeln(f,'Circles');
     for count:=0 to k_cir-1 do
     Writeln(f,Circles[count].x,'/',Circles[count].y,'/',
               Circles[count].r,'/',Circles[count].col,'/',
               Circles[count].Width,'/',Circles[count].style,
               '/',Circles[count].id);

     Writeln(f,'Arcs');
     for count:=0 to k_arc-1 do
     Writeln(f,Arcs[count].x1,'/',Arcs[count].y1,'/',
               Arcs[count].x2,'/',Arcs[count].y2,'/',
               Arcs[count].l_x,'/',Arcs[count].r_x,'/',
               Arcs[count].t_y,'/',Arcs[count].b_y,'/',
               Arcs[count].col,'/',
               Arcs[count].Width,'/',Arcs[count].style,'/',
               Arcs[count].CentrX,'/',Arcs[count].CentrY,'/',
               Arcs[count].rad,'/',
               Arcs[count].StartAngle,'/',Arcs[count].EndAngle,'/',
               Arcs[count].id);

     Writeln(f,'Points');
     for count:=0 to k_points-1 do
     Writeln(f,points[count].x,'/',points[count].y, '/',
               points[count].id);

     Writeln(f,'Texts');
     for count:=0 to k_nadp-1 do
     Writeln(f,nadp[count].x,'/',nadp[count].y,'/',
               nadp[count].h,'/',nadp[count].s,'/',nadp[count].col,
               '/', nadp[count].id);

     Writeln(f,'Arrow');
     for count:=0 to k_arrow-1 do
     Writeln(f,arrow[count].x,'/',arrow[count].y,'/',
               arrow[count].t,'/',arrow[count].col,
               '/', arrow[count].id);

     Writeln(f,'Dimension');
     for count:=0 to High(DimAr) do
         begin
           Write(f,    DimAr[count].x1, '/', DimAr[count].y1, '/',
                       DimAr[count].x2, '/', DimAr[count].y2, '/',
                       DimAr[count].x3, '/', DimAr[count].y3, '/',
                       DimAr[count].x4, '/', DimAr[count].y4, '/',
                       DimAr[count].nadpCX, '/', DimAr[count].nadpCY, '/',
                       DimAr[count].nadp, '/', DimAr[count].NumDigit, '/',
                       DimAr[count].id);
           Writeln(f);
         end;

     Writeln(f,'Block');
     for count:=0 to High(blocks) do
         begin
           Write(f,blocks[count].num);
           for count2 := 0 to High(blocks[count].id_num) do
               Write(f,'/',blocks[count].id_num[count2,0], '_', blocks[count].id_num[count2,1]);
           Writeln(f);
         end;

     Writeln(f,'Size');
     Writeln(f,IntToStr(Form1.WS.Width),'/',IntToStr(Form1.WS.Height));

     Writeln(f,'Back');

     If podl = true then
        Writeln(f,ExtractFileName(F_name+'.png')); //Название файла подложки

     CloseFile(f);

     Form1.add_back.Enabled:=true;
end;


procedure SaveScript(); //Сохранить скрипт VB-Nanocad

var
  fName : string = '';
  f:Text;
  count:integer;
  AngleDim : double = 0;

begin
   Form1.SaveScriptDialog.FileName := '';
   if Form1.SaveScriptDialog.Execute then
      fName := Form1.SaveScriptDialog.FileName;
   if fName = '' then
   begin
       Showmessage('Файл не выбран');
       exit;
   end;

   AssignFile(f,fName);
   Rewrite(f);

   //В координатах (Form1.WS.Height - координата), т.к.
   //в Автокаде ось Y снизу ввверх

   //Отрезки
   Writeln(f,'Dim lineObj');
   for count:=0 to k-1 do
   Writeln(f,'Set lineObj = ThisDrawing.ModelSpace.AddLine("',
   Lines[count].x1,',',Form1.WS.Height-Lines[count].y1,',','0", "',
   Lines[count].x2,',',Form1.WS.Height-Lines[count].y2,',','0")');

   //Окружности
   Writeln(f,'Dim circObj');
   for count:=0 to k_cir-1 do
   Writeln(f,'Set circObj = ThisDrawing.ModelSpace.AddCircle("',
   Circles[count].x,',',Form1.WS.Height-Circles[count].y,',0",',Circles[count].r,')');

   //Дуги
   Writeln(f,'Dim ArcObj');
   for count:=0 to k_arc-1 do
   begin
       Writeln(f,'Set ArcObj = ThisDrawing.ModelSpace.AddArc("',
                     arcs[count].CentrX,',',
                     Form1.WS.Height-arcs[count].CentrY,',0",',
                     arcs[count].rad,',',arcs[count].StartAngle,',',arcs[count].EndAngle,')');
   end;

   //Текст
   Writeln(f,'Dim textObj');
   for count:=0 to k_nadp-1 do
   Writeln(f,'Set textObj = ThisDrawing.ModelSpace.AddText("',Nadp[count].s,'","',
   Nadp[count].x,',',Form1.WS.Height-Nadp[count].y-Nadp[count].h,',','0"',',',Nadp[count].h,')');

   //Стрелки
   Writeln(f,'Dim ArrowObj');
   for count:=0 to k_arrow-1 do
   begin
        case Arrow[count].t of
          'down':
              begin
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y+30,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x+5,',',Form1.WS.Height-Arrow[count].y+5,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x-5,',',Form1.WS.Height-Arrow[count].y+5,',','0")');
              end;
          ///
          'up':
              begin
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y-30,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x+5,',',Form1.WS.Height-Arrow[count].y-5,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x-5,',',Form1.WS.Height-Arrow[count].y-5,',','0")');
              end;
          'right':
              begin
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x-30,',',Form1.WS.Height-Arrow[count].y,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x-5,',',Form1.WS.Height-Arrow[count].y-5,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x-5,',',Form1.WS.Height-Arrow[count].y+5,',','0")');
              end;
          'left':
              begin
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x+30,',',Form1.WS.Height-Arrow[count].y,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x+5,',',Form1.WS.Height-Arrow[count].y+5,',','0")');
                  //
                  Writeln(f,'Set ArrowObj = ThisDrawing.ModelSpace.AddLine("',
                  Arrow[count].x,',',Form1.WS.Height-Arrow[count].y,',','0", "',
                  Arrow[count].x+5,',',Form1.WS.Height-Arrow[count].y-5,',','0")');
              end;
          end;
   end;

   //Размеры
   Writeln(f,'Dim DimObj');
   for count:=0 to High(DimAr) do
   begin
        if (DimAr[count].x3 <> DimAr[count].x4) and
           (DimAr[count].y3 <> DimAr[count].y4) then
            Writeln(f,'Set DimObj = ThisDrawing.ModelSpace.AddDimAligned("',
                         DimAr[count].x1,',',Form1.WS.Height-DimAr[count].y1,',0", "',
                         DimAr[count].x2,',',Form1.WS.Height-DimAr[count].y2,',0", "',
                         DimAr[count].nadpCX,',',Form1.WS.Height-DimAr[count].nadpCY,',0")')
        else
           begin
                if (DimAr[count].x3 = DimAr[count].x4) then
                 if (DimAr[count].y1 > DimAr[count].y3) then
                  AngleDim := DegToRad(90)
                  else AngleDim := DegToRad(270);
                 if (DimAr[count].y3 = DimAr[count].y4) then
                 if (DimAr[count].x1 > DimAr[count].x3) then
                  AngleDim := DegToRad(180)
                  else AngleDim := DegToRad(0);

                Writeln(f,'Set DimObj = ThisDrawing.ModelSpace.AddDimRotated("',
                             DimAr[count].x1,',',Form1.WS.Height-DimAr[count].y1,',0", "',
                             DimAr[count].x2,',',Form1.WS.Height-DimAr[count].y2,',0", "',
                             DimAr[count].nadpCX,',',Form1.WS.Height-DimAr[count].nadpCY,',0", '+
                             FloatToStr(AngleDim,my)+')');
                //AddDimRotated(XLine1Point, XLine2Point, DimLineLocation, RotationAngle)
           end;
   end;

   Writeln(f,'Application.ZOOMALL');

   Close(f);
   ShowMessage('Файл скрипта сохранен');
end;

end.

