unit my_modify;

{
    Vector graphical editor.
    Version: 10.
    Written on FreePascal (https://freepascal.org/).
    Copyright (C) 2024  Warfalomay

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
}

//Процедуры и функции для редактирования объектов

{$mode ObjFPC}{$H+}

interface
//Сдвинуть один или несколько объектов по X или по Y
procedure Move_vect(side_x, side_y : integer);
//Перемещение объектов указанием мыши
Procedure MoveObj(x_vrem, y_vrem : double);
//Копирование объектов указанием мыши
Procedure CopyObj(x_vrem, y_vrem : double);
//Сохранить предыдущий шаг
procedure save_prev();
//Удалить объект/объекты
procedure Delete_Ob();
//Удалить всё
procedure Clear_all_fig();
//Выбор единичного объекта
function Select(x,y:double):integer;
//Объектная привязка
procedure near_points(var x,y : double; dist : integer);
//Объектная привязка с игнорированием указанного объекта,
//т.е. привязываться будет к любому объекту, кроме указанного при вызове функции
//Ипсользуется, чтобы не привязываться к самому редактируемому объекту
procedure near_points(var x,y : double; dist, ob_type, ob_num : integer);
//Поворот выбранных объектов
procedure Rotate(xr,yr : double);
//Перетаскивание за точку
procedure Drag_by_point(x,y : double);
//Выбор объектов рамкой
procedure SelectByFrame(x1,y1,x2,y2 : double; action : byte {0 - искл, 1 - выделить, 2 - добавить});
//Удлинить или укоротить линию по другой линии
procedure Trim();
//Изменить длину отрезка
procedure LetChangeLength;
//Зеркальное отражение
procedure MoveMirror(x1,y1, x2,y2 : double);
//Зеркальное отражение без удаления исходных объектов
procedure CopyMirror(x1,y1, x2,y2 : double);

var
  block_num : integer = -1; //Переменная для выделения блока

implementation
uses Graphics,Unit1,Classes, SysUtils, View, math, Dialogs, Controls, MyFunc,
     block, primitives, Intersection_unit, Drawing_unit, SpecFunction, Calc;


//Зеркальное отражение
procedure MoveMirror(x1,y1, x2,y2 : double);

var
   count : integer;
   vremPoint : MyTPoint;
   arcP1, arcP2 : MyTPoint;
   Cx,Cy, vremRad : double;
   id_vrem : integer;
   VremColor : TColor;
   VremWidth : integer;
   VremStyle : TPenStyle;

begin
  TranslateSingleToMulti;

  for count := 0 to High(multy_selected_line) do
  begin
     vremPoint := MirrorCalc(x1,y1,x2,y2,lines[multy_selected_line[count]].x1, lines[multy_selected_line[count]].y1);
     lines[multy_selected_line[count]].x1 := vremPoint.x;
     lines[multy_selected_line[count]].y1 := vremPoint.y;
     vremPoint := MirrorCalc(x1,y1,x2,y2,lines[multy_selected_line[count]].x2, lines[multy_selected_line[count]].y2);
     lines[multy_selected_line[count]].x2 := vremPoint.x;
     lines[multy_selected_line[count]].y2 := vremPoint.y;
  end;

  for count := 0 to High(multy_selected_circle) do
  begin
     vremPoint := MirrorCalc(x1,y1,x2,y2,circles[multy_selected_circle[count]].x, circles[multy_selected_circle[count]].y);
     circles[multy_selected_circle[count]].x := vremPoint.x;
     circles[multy_selected_circle[count]].y := vremPoint.y;
  end;

  for count := 0 to High(multy_selected_point) do
  begin
     vremPoint := MirrorCalc(x1,y1,x2,y2,points[multy_selected_point[count]].x, points[multy_selected_point[count]].y);
     points[multy_selected_point[count]].x := vremPoint.x;
     points[multy_selected_point[count]].y := vremPoint.y;
  end;

  for count := 0 to High(multy_selected_arcs) do
  begin
     vremPoint := MirrorCalc(x1,y1,x2,y2,arcs[multy_selected_arcs[count]].x1, arcs[multy_selected_arcs[count]].y1);
     arcP1.x := vremPoint.x;
     arcP1.y := vremPoint.y;

     vremPoint := MirrorCalc(x1,y1,x2,y2,arcs[multy_selected_arcs[count]].x2, arcs[multy_selected_arcs[count]].y2);
     arcP2.x := vremPoint.x;
     arcP2.y := vremPoint.y;

     vremRad := arcs[multy_selected_arcs[count]].rad;
     Cx := arcs[multy_selected_arcs[count]].CentrX;
     Cy := arcs[multy_selected_arcs[count]].CentrY;
     VremColor := arcs[multy_selected_arcs[count]].col;
     VremWidth := arcs[multy_selected_arcs[count]].Width;
     VremStyle := arcs[multy_selected_arcs[count]].style;
     id_vrem := arcs[multy_selected_arcs[count]].id;

     Delete(arcs,multy_selected_arcs[count],1);

     if (x1 < Cx) and
        (x2 < Cx) then
        Draw_arc(arcP2,arcP1,vremRad)
     else
        Draw_arc(arcP2,arcP1,vremRad);

     arcs[High(arcs)].col := VremColor;
     arcs[High(arcs)].Width := VremWidth;
     arcs[High(arcs)].style := VremStyle;
     arcs[High(arcs)].id := id_vrem;
  end;

  for count := 0 to High(multy_selected_text) do
  begin
     vremPoint := MirrorCalc(x1,y1,x2,y2,nadp[multy_selected_text[count]].x, nadp[multy_selected_text[count]].y);
     nadp[multy_selected_text[count]].x := vremPoint.x;
     nadp[multy_selected_text[count]].y := vremPoint.y;
  end;

  for count := 0 to High(multy_selected_arrow) do
  begin
     vremPoint := MirrorCalc(x1,y1,x2,y2,arrow[multy_selected_arrow[count]].x, arrow[multy_selected_arrow[count]].y);
     arrow[multy_selected_arrow[count]].x := vremPoint.x;
     arrow[multy_selected_arrow[count]].y := vremPoint.y;

     if ((x1 > arrow[multy_selected_arrow[count]].x) and
        (x2 > arrow[multy_selected_arrow[count]].x)) or
        ((x1 < arrow[multy_selected_arrow[count]].x) and
        (x2 < arrow[multy_selected_arrow[count]].x))
        then
     if (arrow[multy_selected_arrow[count]].t = 'left') then
        arrow[multy_selected_arrow[count]].t := 'right'
     else
     if (arrow[multy_selected_arrow[count]].t = 'right') then
        arrow[multy_selected_arrow[count]].t := 'left';

     if ((y1 > arrow[multy_selected_arrow[count]].y) and
        (y2 > arrow[multy_selected_arrow[count]].y)) or
        ((y1 < arrow[multy_selected_arrow[count]].y) and
        (y2 < arrow[multy_selected_arrow[count]].y))
        then
     if (arrow[multy_selected_arrow[count]].t = 'down') then
        arrow[multy_selected_arrow[count]].t := 'up'
     else
     if (arrow[multy_selected_arrow[count]].t = 'up') then
        arrow[multy_selected_arrow[count]].t := 'down';

  end;
end;

//Зеркальное отражение без удаления исходных объектов
procedure CopyMirror(x1,y1, x2,y2 : double);

var
   count : integer;
   vremPoint : MyTPoint;
   arcP1, arcP2 : MyTPoint;
   BlockSum : array of integer; //массив для накопления элементов для нового блока

begin
  TranslateSingleToMulti;

  for count := 0 to High(multy_selected_line) do
  begin
     vremPoint := MirrorCalc(x1,y1,x2,y2,lines[multy_selected_line[count]].x1, lines[multy_selected_line[count]].y1);
     lines[k].x1 := vremPoint.x;
     lines[k].y1 := vremPoint.y;
     vremPoint := MirrorCalc(x1,y1,x2,y2,lines[multy_selected_line[count]].x2, lines[multy_selected_line[count]].y2);
     lines[k].x2 := vremPoint.x;
     lines[k].y2 := vremPoint.y;
     lines[k].col := lines[multy_selected_line[count]].col;
     lines[k].style := lines[multy_selected_line[count]].style;
     lines[k].Width := lines[multy_selected_line[count]].Width;
     lines[k].id := id;

     //Если копируемый отрезок входит в блок
     if (Prov_Sel_Block(lines[multy_selected_line[count]].id) <> -1) then
     begin
         SetLength(BlockSum, Length(BlockSum)+1);
         BlockSum[High(BlockSum)] := id;
     end;

     id += 1;
     k += 1;
  end;

  for count := 0 to High(multy_selected_circle) do
  begin
     vremPoint := MirrorCalc(x1,y1,x2,y2,circles[multy_selected_circle[count]].x, circles[multy_selected_circle[count]].y);
     circles[k_cir].x := vremPoint.x;
     circles[k_cir].y := vremPoint.y;
     circles[k_cir].r := circles[multy_selected_circle[count]].r;
     circles[k_cir].col := circles[multy_selected_circle[count]].col;
     circles[k_cir].style := circles[multy_selected_circle[count]].style;
     circles[k_cir].Width := circles[multy_selected_circle[count]].Width;
     circles[k_cir].id := id;

     //Если копируемая окружность входит в блок
     if (Prov_Sel_Block(circles[multy_selected_circle[count]].id) <> -1) then
     begin
         SetLength(BlockSum, Length(BlockSum)+1);
         BlockSum[High(BlockSum)] := id;
     end;

     id += 1;
     k_cir += 1;
  end;

  for count := 0 to High(multy_selected_point) do
  begin
     vremPoint := MirrorCalc(x1,y1,x2,y2,points[multy_selected_point[count]].x, points[multy_selected_point[count]].y);
     points[k_points].x := vremPoint.x;
     points[k_points].y := vremPoint.y;
     points[k_points].id := id;

     //Если копируемая точка входит в блок
     if (Prov_Sel_Block(points[multy_selected_point[count]].id) <> -1) then
     begin
         SetLength(BlockSum, Length(BlockSum)+1);
         BlockSum[High(BlockSum)] := id;
     end;

     id += 1;
     k_points += 1;
  end;

  for count := 0 to High(multy_selected_arcs) do
  begin
     vremPoint := MirrorCalc(x1,y1,x2,y2,arcs[multy_selected_arcs[count]].x1, arcs[multy_selected_arcs[count]].y1);
     arcP1.x := vremPoint.x;
     arcP1.y := vremPoint.y;

     vremPoint := MirrorCalc(x1,y1,x2,y2,arcs[multy_selected_arcs[count]].x2, arcs[multy_selected_arcs[count]].y2);
     arcP2.x := vremPoint.x;
     arcP2.y := vremPoint.y;

     if (x1 < arcs[multy_selected_arcs[count]].CentrX) and
        (x2 < arcs[multy_selected_arcs[count]].CentrX) then
        Draw_arc(arcP2,arcP1,arcs[multy_selected_arcs[count]].rad)
     else
        Draw_arc(arcP2,arcP1,arcs[multy_selected_arcs[count]].rad);

     id -= 1; //Т.к. в drawarc добавляется id
     k_arc -= 1; // Т.к. в drawarc добавляется k_arc

     arcs[k_arc].id := id;

     //Если копируемая дуга входит в блок
     if (Prov_Sel_Block(arcs[multy_selected_arcs[count]].id) <> -1) then
     begin
         SetLength(BlockSum, Length(BlockSum)+1);
         BlockSum[High(BlockSum)] := id;
     end;

     id += 1;
     k_arc += 1;
  end;

  for count := 0 to High(multy_selected_text) do
  begin
     vremPoint := MirrorCalc(x1,y1,x2,y2,nadp[multy_selected_text[count]].x, nadp[multy_selected_text[count]].y);
     nadp[k_nadp].x := vremPoint.x;
     nadp[k_nadp].y := vremPoint.y;
     nadp[k_nadp].id := id;
     nadp[k_nadp].h := nadp[count].h;
     nadp[k_nadp].col := nadp[count].col;
     nadp[k_nadp].s := nadp[count].s;

     //Если копируемая надпись входит в блок
     if (Prov_Sel_Block(nadp[multy_selected_text[count]].id) <> -1) then
     begin
         SetLength(BlockSum, Length(BlockSum)+1);
         BlockSum[High(BlockSum)] := id;
     end;

     id += 1;
     k_nadp += 1;
  end;

  for count := 0 to High(multy_selected_arrow) do
  begin
     vremPoint := MirrorCalc(x1,y1,x2,y2,arrow[multy_selected_arrow[count]].x, arrow[multy_selected_arrow[count]].y);
     arrow[k_arrow].x := vremPoint.x;
     arrow[k_arrow].y := vremPoint.y;
     arrow[k_arrow].col := arrow[multy_selected_arrow[count]].col;
     arrow[k_arrow].id := id;

     arrow[k_arrow].t := arrow[multy_selected_arrow[count]].t;

     if ((x1 > arrow[k_arrow].x) and
        (x2 > arrow[k_arrow].x)) or
        ((x1 < arrow[k_arrow].x) and
        (x2 < arrow[k_arrow].x))
        then
     if (arrow[multy_selected_arrow[count]].t = 'left') then
        arrow[k_arrow].t := 'right'
     else
     if (arrow[multy_selected_arrow[count]].t = 'right') then
        arrow[k_arrow].t := 'left';

     if ((y1 > arrow[k_arrow].y) and
        (y2 > arrow[k_arrow].y)) or
        ((y1 < arrow[k_arrow].y) and
        (y2 < arrow[k_arrow].y))
        then
     if (arrow[multy_selected_arrow[count]].t = 'down') then
        arrow[k_arrow].t := 'up'
     else
     if (arrow[multy_selected_arrow[count]].t = 'up') then
        arrow[k_arrow].t := 'down';

     //Если копируемая стрелка входит в блок
     if (Prov_Sel_Block(arrow[multy_selected_arrow[count]].id) <> -1) then
     begin
         SetLength(BlockSum, Length(BlockSum)+1);
         BlockSum[High(BlockSum)] := id;
     end;

     id += 1;
     k_arrow += 1;
  end;

  //Если среди выбр.элементов был блок, то создать новый блок
   if (length(BlockSum) > 0) then
      MakeBlock(BlockSum);
end;

procedure Drag_by_point(x,y : double);//Перетаскивание за точку

var
   prover : boolean = false;

begin
    If select_type = line then //Если выбрана линия
    begin
       If ((x = lines[Selected].x1) and //Если указал не конечную точку
          (y = lines[Selected].y1)) or
          ((x = lines[Selected].x2) and
          (y = lines[Selected].y2)) then
          prover := true;

       if not prover then exit;

       Form1.WS.Cursor := crHandPoint;
       drag_point_mode := true;
       Drag_Point.X := x;
       Drag_Point.Y := y;
       Drag_selected := Selected;
    end;
    If select_type = circle then //Если выбрана окружность
    begin
       If ((x <> circles[Selected].x) and //Если указал не конечную точку
          (y <> circles[Selected].y)) then
          exit;
       Form1.WS.Cursor := crHandPoint;
       drag_point_mode := true;
       Drag_Point.X := x;
       Drag_Point.Y := y;
       Drag_selected := Selected;
    end;
    If select_type = dim then //Если выбран размер
    begin
       If ((x = DimAr[Selected].x1) and //Если указал конечную точку
           (y = DimAr[Selected].y1)) then
           begin
              Drag_Point.X := DimAr[Selected].x1;
              Drag_Point.Y := DimAr[Selected].y1;
              Form1.WS.Cursor := crHandPoint;
              drag_point_mode := true;
              Drag_selected := Selected;
           end;
       If ((x = DimAr[Selected].x2) and //Если указал конечную точку
           (y = DimAr[Selected].y2)) then
           begin
              Drag_Point.X := DimAr[Selected].x2;
              Drag_Point.Y := DimAr[Selected].y2;
              Form1.WS.Cursor := crHandPoint;
              drag_point_mode := true;
              Drag_selected := Selected;
           end;
       If ((x = DimAr[Selected].x3) and //Если указал конечную точку
           (y = DimAr[Selected].y3)) then
           begin
              Drag_Point.X := DimAr[Selected].x3;
              Drag_Point.Y := DimAr[Selected].y3;
              Form1.WS.Cursor := crHandPoint;
              drag_point_mode := true;
              Drag_selected := Selected;
           end;
       If ((x = DimAr[Selected].x4) and //Если указал конечную точку
           (y = DimAr[Selected].y4)) then
           begin
              Drag_Point.X := DimAr[Selected].x4;
              Drag_Point.Y := DimAr[Selected].y4;
              Form1.WS.Cursor := crHandPoint;
              drag_point_mode := true;
              Drag_selected := Selected;
           end;
    end;
    //Если выбран блок

end;

//Вспомогательная функция для проверки выбора рамкой (только для отрезков)
function CrossLine(num : integer; x1,x2,y1,y2 : double) : boolean;

var
  v1,v2,v3,v4:double;

begin
  Result:=false;

  v1:=(Lines[num].x2-Lines[num].x1)*(y1-Lines[num].y1)-(Lines[num].y2-Lines[num].y1)*(x1-Lines[num].x1);
  v2:=(Lines[num].x2-Lines[num].x1)*(y2-Lines[num].y1)-(Lines[num].y2-Lines[num].y1)*(x2-Lines[num].x1);
  v3:=(x2-x1)*(Lines[num].y1-y1)-(y2-y1)*(Lines[num].x1-x1);
  v4:=(x2-x1)*(Lines[num].y2-y1)-(y2-y1)*(Lines[num].x2-x1);
  If (v1*v2<0) and (v3*v4<0) then result:=true;
end;

//Вспомогательная функция для проверки выбора рамкой (только для размеров)
function CrossDim(num : integer; x1,x2,y1,y2 : double) : boolean;

var
  v1,v2,v3,v4:double;

begin
  Result:=false;

  v1:=(DimAr[num].x3-DimAr[num].x1)*(y1-DimAr[num].y1)-(DimAr[num].y3-DimAr[num].y1)*(x1-DimAr[num].x1);
  v2:=(DimAr[num].x3-DimAr[num].x1)*(y2-DimAr[num].y1)-(DimAr[num].y3-DimAr[num].y1)*(x2-DimAr[num].x1);
  v3:=(x2-x1)*(DimAr[num].y1-y1)-(y2-y1)*(DimAr[num].x1-x1);
  v4:=(x2-x1)*(DimAr[num].y3-y1)-(y2-y1)*(DimAr[num].x3-x1);
  If (v1*v2<0) and (v3*v4<0) then
  begin
     result:=true;
     exit;
  end;

  v1:=(DimAr[num].x4-DimAr[num].x2)*(y1-DimAr[num].y2)-(DimAr[num].y4-DimAr[num].y2)*(x1-DimAr[num].x2);
  v2:=(DimAr[num].x4-DimAr[num].x2)*(y2-DimAr[num].y2)-(DimAr[num].y4-DimAr[num].y2)*(x2-DimAr[num].x2);
  v3:=(x2-x1)*(DimAr[num].y2-y1)-(y2-y1)*(DimAr[num].x2-x1);
  v4:=(x2-x1)*(DimAr[num].y4-y2)-(y2-y1)*(DimAr[num].x4-x1);
  If (v1*v2<0) and (v3*v4<0) then
  begin
     result:=true;
     exit;
  end;

  v1:=(DimAr[num].x4-DimAr[num].x3)*(y1-DimAr[num].y3)-(DimAr[num].y4-DimAr[num].y3)*(x1-DimAr[num].x3);
  v2:=(DimAr[num].x4-DimAr[num].x3)*(y2-DimAr[num].y3)-(DimAr[num].y4-DimAr[num].y3)*(x2-DimAr[num].x3);
  v3:=(x2-x1)*(DimAr[num].y3-y1)-(y2-y1)*(DimAr[num].x3-x1);
  v4:=(x2-x1)*(DimAr[num].y4-y2)-(y2-y1)*(DimAr[num].x4-x1);
  If (v1*v2<0) and (v3*v4<0) then
  begin
     result:=true;
     exit;
  end;
end;


//Функции преобразования номера в общем массиве объектов в номер
//в массиве выделения
//Поиск линии в массиве выделенных по номеру в общем массиве
function findMultiSelectLine(num : integer) : integer;

var
  count : integer;

begin
  result := -1;

  for count := 0 to High(multy_selected_line) do
  if (multy_selected_line[count] = num) then
  begin
   result := count;
   exit;
  end;
end;
//Поиск окружности в массиве выделенных по номеру в общем массиве
function findMultiSelectCircle(num : integer) : integer;

var
  count : integer;

begin
  result := -1;

  for count := 0 to High(multy_selected_circle) do
  if (multy_selected_circle[count] = num) then
  begin
   result := count;
   exit;
  end;
end;
//Поиск точки в массиве выделенных по номеру в общем массиве
function findMultiSelectPoint(num : integer) : integer;

var
  count : integer;

begin
  result := -1;

  for count := 0 to High(multy_selected_point) do
  if (multy_selected_point[count] = num) then
  begin
   result := count;
   exit;
  end;
end;
//Поиск текста в массиве выделенных по номеру в общем массиве
function findMultiSelectText(num : integer) : integer;

var
  count : integer;

begin
  result := -1;

  for count := 0 to High(multy_selected_text) do
  if (multy_selected_text[count] = num) then
  begin
   result := count;
   exit;
  end;
end;
//Поиск стрелки в массиве выделенных по номеру в общем массиве
function findMultiSelectArrow(num : integer) : integer;

var
  count : integer;

begin
  result := -1;

  for count := 0 to High(multy_selected_arrow) do
  if (multy_selected_arrow[count] = num) then
  begin
   result := count;
   exit;
  end;
end;
//Поиск дуги в массиве выделенных по номеру в общем массиве
function findMultiSelectArcs(num : integer) : integer;

var
  count : integer;

begin
  result := -1;

  for count := 0 to High(multy_selected_arcs) do
  if (multy_selected_arcs[count] = num) then
  begin
   result := count;
   exit;
  end;
end;
//Поиск размера в массиве выделенных по номеру в общем массиве
function findMultiSelectDim(num : integer) : integer;

var
  count : integer;

begin
  result := -1;

  for count := 0 to High(multy_selected_dim) do
  if (multy_selected_dim[count] = num) then
  begin
   result := count;
   exit;
  end;
end;

//Выбор объектов рамкой
procedure SelectByFrame(x1,y1,x2,y2 : double; action : byte {0 - искл, 1 - выделить});

var
  count : integer;
  count2 : integer;
  resFind : boolean = false;

  Xmin, Ymin : double;
  Xmax, Ymax : double;
  lin1, lin2, lin3, lin4 : TLineType;
  cir : TCirType;

begin
    Clear();

    Xmin := min(x1,x2);
    Ymin := min(y1,y2);
    Xmax := max(x1,x2);
    Ymax := max(y1,y2);

    lin1.x1 := Xmin; lin1.x2 := Xmax; //верх_гор
    lin1.y1 := Ymin; lin1.y2 := Ymin;

    lin2.x1 := Xmin; lin2.x2 := Xmax; //ниж_гор
    lin2.y1 := Ymax; lin2.y2 := Ymax;

    lin3.x1 := Xmin; lin3.x2 := Xmin; //лев_верт
    lin3.y1 := Ymin; lin3.y2 := Ymax;

    lin4.x1 := Xmax; lin4.x2 := Xmax; //пра_верт
    lin4.y1 := Ymin; lin4.y2 := Ymax;

    //Проверка выбора линий
    If Form1.SelectChoiceMenu.Checked[0] then
    for count := 0 to High(lines) do
    begin
        if ((lines[count].x1 >= Xmin) and
           (lines[count].x1 <= Xmax) and
           (lines[count].y1 >= Ymin) and
           (lines[count].y1 <= Ymax)) or
           ((lines[count].x2 >= Xmin) and
           (lines[count].x2 <= Xmax) and
           (lines[count].y2 >= Ymin) and
           (lines[count].y2 <= Ymax) or
           CrossLine(count,x1,x2,y1,y2))then
           begin
              resFind := false;
              if (action = 1) or
                (action = 2)
                then  //выбрать объекты
                begin
                   //Чтобы нельзя было дважды выбрать элемент
                   for count2 := 0 to High(multy_selected_line) do
                   begin
                        if count = multy_selected_line[count2] then resFind := true;
                   end;
                   if not (resFind) then
                   begin
                       setLength(multy_selected_line,Length(multy_selected_line)+1);
                       multy_selected_line[High(multy_selected_line)] := count;
                   end;
                end
             else     //исключить из выбранных
                Delete(multy_selected_line,findMultiSelectLine(count),1);
           end;
    end;

    //Проверка выбора окружностей
    If Form1.SelectChoiceMenu.Checked[1] then
    for count := 0 to High(circles) do
    begin
        cir.x := circles[count].x;
        cir.y := circles[count].y;
        cir.r := circles[count].r;

        if ((circles[count].x-circles[count].r >= Xmin) and
           (circles[count].x+circles[count].r <= Xmax) and
           (circles[count].y-circles[count].r >= Ymin) and
           (circles[count].y+circles[count].r <= Ymax)) or
           LineCrosCirc(lin1,cir) or
           LineCrosCirc(lin2,cir) or
           LineCrosCirc(lin3,cir) or
           LineCrosCirc(lin4,cir)
        then
           begin
             resFind := false;
             if (action = 1) or
                (action = 2)
                then  //выбрать объекты
                begin
                  //Чтобы нельзя было дважды выбрать элемент
                   for count2 := 0 to High(multy_selected_circle) do
                   begin
                        if count = multy_selected_circle[count2] then resFind := true;
                   end;

                   if not (resFind) then
                   begin
                       setLength(multy_selected_circle,Length(multy_selected_circle)+1);
                       multy_selected_circle[High(multy_selected_circle)] := count;
                   end;
                end
             else     //исключить из выбранных
                Delete(multy_selected_circle,findMultiSelectCircle(count),1);
           end;
    end;

    //Проверка выбора точек
    If Form1.SelectChoiceMenu.Checked[2] then
    If P_Show then
    for count := 0 to High(points) do
    begin
        if ((points[count].x >= Xmin) and
           (points[count].x <= Xmax) and
           (points[count].y >= Ymin) and
           (points[count].y <= Ymax)) then
           begin
             resFind := false;
             if (action = 1) or
                (action = 2)
                then  //выбрать объекты
                begin
                  //Чтобы нельзя было дважды выбрать элемент
                   for count2 := 0 to High(multy_selected_point) do
                   begin
                        if count = multy_selected_point[count2] then resFind := true;
                   end;

                   if not (resFind) then
                   begin
                       setLength(multy_selected_point,Length(multy_selected_point)+1);
                       multy_selected_point[High(multy_selected_point)] := count;
                   end;
                end
             else     //исключить из выбранных
                Delete(multy_selected_point,findMultiSelectPoint(count),1);
           end;
    end;

    //Проверка выбора текста
    If Form1.SelectChoiceMenu.Checked[4] then
    for count := 0 to High(nadp) do
    begin
        if ((nadp[count].x >= Xmin) and
           (nadp[count].x <= Xmax) and
           (nadp[count].y >= Ymin) and
           (nadp[count].y <= Ymax)) then
           begin
             resFind := false;
             if (action = 1) or
                (action = 2) then  //выбрать объекты
                begin
                  //Чтобы нельзя было дважды выбрать элемент
                   for count2 := 0 to High(multy_selected_text) do
                   begin
                        if count = multy_selected_text[count2] then resFind := true;
                   end;

                   if not (resFind) then
                   begin
                       setLength(multy_selected_text,Length(multy_selected_text)+1);
                       multy_selected_text[High(multy_selected_text)] := count;
                   end;
                end
             else                //исключить из выбранных
                Delete(multy_selected_text,findMultiSelectText(count),1);
           end;
    end;

    //Проверка выбора стрелок
    If Form1.SelectChoiceMenu.Checked[4] then
    for count := 0 to High(arrow) do
    begin
        if ((arrow[count].x >= Xmin) and
           (arrow[count].x <= Xmax) and
           (arrow[count].y >= Ymin) and
           (arrow[count].y <= Ymax)) then
           begin
             resFind := false;
             if (action = 1) or
                (action = 2)
                then  //выбрать объекты
                begin
                  //Чтобы нельзя было дважды выбрать элемент
                   for count2 := 0 to High(multy_selected_arrow) do
                   begin
                        if count = multy_selected_arrow[count2] then resFind := true;
                   end;

                   if not (resFind) then
                   begin
                       setLength(multy_selected_arrow,Length(multy_selected_arrow)+1);
                       multy_selected_arrow[High(multy_selected_arrow)] := count;
                   end;
                end
             else     //исключить из выбранных
                Delete(multy_selected_arrow,findMultiSelectArrow(count),1);
           end;
    end;

    //Проверка выбора дуги
    If Form1.SelectChoiceMenu.Checked[3] then
    for count := 0 to High(arcs) do
    begin
        if ((arcs[count].x1 >= Xmin) and
           (arcs[count].x2 <= Xmax) and
           (arcs[count].y1 >= Ymin) and
           (arcs[count].y2 <= Ymax)) then
           begin
             resFind := false;
             if (action = 1) or
                (action = 2)
                then  //выбрать объекты
                begin
                  //Чтобы нельзя было дважды выбрать элемент
                   for count2 := 0 to High(multy_selected_arcs) do
                   begin
                        if count = multy_selected_arcs[count2] then resFind := true;
                   end;

                   if not (resFind) then
                   begin
                       setLength(multy_selected_arcs,Length(multy_selected_arcs)+1);
                       multy_selected_arcs[High(multy_selected_arcs)] := count;
                   end;
                end
             else     //исключить из выбранных
                Delete(multy_selected_arcs,findMultiSelectArcs(count),1);
           end;
    end;

    //Проверка выбора размеров
    If Form1.SelectChoiceMenu.Checked[4] then
    for count := 0 to High(DimAr) do
    begin
        if ((DimAr[count].x1 >= Xmin) and
           (DimAr[count].x1 <= Xmax) and
           (DimAr[count].y1 >= Ymin) and
           (DimAr[count].y1 <= Ymax)) or
           ((DimAr[count].x2 >= Xmin) and
           (DimAr[count].x2 <= Xmax) and
           (DimAr[count].y2 >= Ymin) and
           (DimAr[count].y2 <= Ymax)) or
           ((DimAr[count].x3 >= Xmin) and
           (DimAr[count].x3 <= Xmax) and
           (DimAr[count].y3 >= Ymin) and
           (DimAr[count].y3 <= Ymax)) or
           ((DimAr[count].x4 >= Xmin) and
           (DimAr[count].x4 <= Xmax) and
           (DimAr[count].y4 >= Ymin) and
           (DimAr[count].y4 <= Ymax))
           or (CrossDim(count,x1,x2,y1,y2))
           then
           begin
             resFind := false;
             if (action = 1) or
                (action = 2)
                then  //выбрать объекты
                begin
                   //Чтобы нельзя было дважды выбрать элемент
                   for count2 := 0 to High(multy_selected_dim) do
                   begin
                        if count = multy_selected_dim[count2] then resFind := true;
                   end;

                   if not (resFind) then
                   begin
                       setLength(multy_selected_dim,Length(multy_selected_dim)+1);
                       multy_selected_dim[High(multy_selected_dim)] := count;
                   end;
                end
             else     //исключить из выбранных
                Delete(multy_selected_dim,findMultiSelectDim(count),1);
           end;
    end;

    /////////////////////////////////////////////////////////////////////////////////////////

    //окрашивание выбранных объектов
    begin
          Form1.WS.Canvas.Pen.Color := clRed;

          for count := 0 to High(multy_selected_line) do
          begin
              Form1.WS.Canvas.Line(Round(lines[multy_selected_line[count]].x1) + Screen_x,
                                   Round(lines[multy_selected_line[count]].y1) + Screen_y,
                                   Round(lines[multy_selected_line[count]].x2) + Screen_x,
                                   Round(lines[multy_selected_line[count]].y2) + Screen_y);
          end;
          for count := 0 to High(multy_selected_circle) do
          begin
              Form1.WS.Canvas.Ellipse(Round(circles[multy_selected_circle[count]].x + Screen_x -
                                            circles[multy_selected_circle[count]].r),
                                      Round(circles[multy_selected_circle[count]].y + Screen_y -
                                            circles[multy_selected_circle[count]].r),
                                      Round(circles[multy_selected_circle[count]].x + Screen_x +
                                            circles[multy_selected_circle[count]].r),
                                      Round(circles[multy_selected_circle[count]].y + Screen_y +
                                            circles[multy_selected_circle[count]].r));
          end;
          for count := 0 to High(multy_selected_point) do
          begin
              Form1.WS.Canvas.Ellipse(Round(points[multy_selected_point[count]].x) + Screen_x - 5,
                                           Round(points[multy_selected_point[count]].y) + Screen_y - 5,
                                           Round(points[multy_selected_point[count]].x) + Screen_x + 5,
                                           Round(points[multy_selected_point[count]].y) + Screen_y + 5);
          end;
          for count := 0 to High(multy_selected_text) do
          begin
              Form1.WS.Canvas.Font.Size := Round(nadp[multy_selected_text[count]].h);
              Form1.WS.Canvas.Font.Color := clRed;
              Form1.WS.Canvas.TextOut(Round(nadp[multy_selected_text[count]].x + Screen_x),
                                      Round(nadp[multy_selected_text[count]].y + Screen_y),
                                      nadp[multy_selected_text[count]].s);
          end;
          for count := 0 to High(multy_selected_arrow) do
          begin
              Form1.WS.Canvas.Ellipse(Round(arrow[multy_selected_arrow[count]].x + Screen_x -5),
                                      Round(arrow[multy_selected_arrow[count]].y + Screen_y +5),
                                      Round(arrow[multy_selected_arrow[count]].x + Screen_x +5),
                                      Round(arrow[multy_selected_arrow[count]].y + Screen_y -5));
                   if arrow[multy_selected_arrow[count]].t = 'left' then
                        Form1.WS.Canvas.Ellipse(Round(arrow[multy_selected_arrow[count]].x+30-5+Screen_x),
                                                Round(arrow[multy_selected_arrow[count]].y+5+Screen_y),
                                                Round(arrow[multy_selected_arrow[count]].x+30+5+Screen_x),
                                                Round(arrow[multy_selected_arrow[count]].y-5+Screen_y));
                   if arrow[multy_selected_arrow[count]].t = 'right' then
                        Form1.WS.Canvas.Ellipse(Round(arrow[multy_selected_arrow[count]].x-30-5+Screen_x),
                                                Round(arrow[multy_selected_arrow[count]].y+5+Screen_y),
                                                Round(arrow[multy_selected_arrow[count]].x-30+5+Screen_x),
                                                Round(arrow[multy_selected_arrow[count]].y-5+Screen_y));
                   if arrow[multy_selected_arrow[count]].t = 'up' then
                        Form1.WS.Canvas.Ellipse(Round(arrow[multy_selected_arrow[count]].x-5+Screen_x),
                                                Round(arrow[multy_selected_arrow[count]].y+30+5+Screen_y),
                                                Round(arrow[multy_selected_arrow[count]].x+5+Screen_x),
                                                Round(arrow[multy_selected_arrow[count]].y+30-5+Screen_y));
                   if arrow[multy_selected_arrow[count]].t = 'down' then
                        Form1.WS.Canvas.Ellipse(Round(arrow[multy_selected_arrow[count]].x-5+Screen_x),
                                                Round(arrow[multy_selected_arrow[count]].y-30+5+Screen_y),
                                                Round(arrow[multy_selected_arrow[count]].x+5+Screen_x),
                                                Round(arrow[multy_selected_arrow[count]].y-30-5+Screen_y));
          end;
          for count := 0 to High(multy_selected_arcs) do
          begin
                Form1.WS.canvas.Arc(Round(Arcs[multy_selected_arcs[count]].l_x+Screen_x),
                                    Round(Arcs[multy_selected_arcs[count]].t_y+Screen_y),
                                    Round(Arcs[multy_selected_arcs[count]].r_x+Screen_x),
                                    Round(Arcs[multy_selected_arcs[count]].b_y+Screen_y),
                                    Round(Arcs[multy_selected_arcs[count]].x1+Screen_x),
                                    Round(Arcs[multy_selected_arcs[count]].y1+Screen_y),
                                    Round(Arcs[multy_selected_arcs[count]].x2+Screen_x),
                                    Round(Arcs[multy_selected_arcs[count]].y2+Screen_y));
          end;
          for count := 0 to High(multy_selected_dim) do
          begin
            Form1.WS.Canvas.Pen.Color := clRed;
            Form1.WS.Canvas.Ellipse(Round(DimAr[multy_selected_dim[count]].x3-5+Screen_X), Round(DimAr[multy_selected_dim[count]].y3-5+Screen_Y),
                                    Round(DimAr[multy_selected_dim[count]].x3+5+Screen_X), Round(DimAr[multy_selected_dim[count]].y3+5+Screen_Y));
            Form1.WS.Canvas.Ellipse(Round(DimAr[multy_selected_dim[count]].x4-5+Screen_X), Round(DimAr[multy_selected_dim[count]].y4-5+Screen_Y),
                                    Round(DimAr[multy_selected_dim[count]].x4+5+Screen_X), Round(DimAr[multy_selected_dim[count]].y4+5+Screen_Y));

            Form1.WS.Canvas.Ellipse(Round(DimAr[multy_selected_dim[count]].x1-5+Screen_X), Round(DimAr[multy_selected_dim[count]].y1-5+Screen_Y),
                                    Round(DimAr[multy_selected_dim[count]].x1+5+Screen_X), Round(DimAr[multy_selected_dim[count]].y1+5+Screen_Y));
            Form1.WS.Canvas.Ellipse(Round(DimAr[multy_selected_dim[count]].x2-5+Screen_X), Round(DimAr[multy_selected_dim[count]].y2-5+Screen_Y),
                                    Round(DimAr[multy_selected_dim[count]].x2+5+Screen_X), Round(DimAr[multy_selected_dim[count]].y2+5+Screen_Y));
          end;
    end;


    if MultiSelect then
      Form1.StatusBar1.Panels.Items[0].Text:='Выбраны элементы'
    else
      Form1.StatusBar1.Panels.Items[0].Text:='Ничего не выбрано';

    Point1.X := -1;
    Point1.Y := -1;
end;

//Объектная привязка
procedure near_points(var x,y : double; dist:integer);

var
  count:integer;
  midX, midY : double;

begin
  if not Form1.SnapCheck.Checked then exit;

  Form1.WS.Cursor := crDefault;

  for count:=0 to Arr_length-1 do
  begin
    //Проверка по массиву линий
    //Вычисление середины отрезка
    midX := min(Lines[count].x1,Lines[count].x2)+
            ((max(Lines[count].x1,Lines[count].x2)-
            min(Lines[count].x1,Lines[count].x2)) / 2);
    midY := min(Lines[count].y1,Lines[count].y2)+
            ((max(Lines[count].y1,Lines[count].y2)-
            min(Lines[count].y1,Lines[count].y2)) / 2);

    if (sqrt(sqr(abs(x-Lines[count].x1))+sqr(abs(y-Lines[count].y1)))<dist) then
     begin
       //if count = Drag_selected then continue;
       x:=Lines[count].x1;
       y:=Lines[count].y1;
       //Form1.WS.Cursor := crCross;
     end;
    if (sqrt(sqr(abs(x-Lines[count].x2))+sqr(abs(y-Lines[count].y2)))<dist) then
     begin
       //if count = Drag_selected then continue;
       x:=Lines[count].x2;
       y:=Lines[count].y2;
       //Form1.WS.Cursor := crCross;
     end;
     if (sqrt(sqr(abs(x-midX))+sqr(abs(y-midY)))<dist) then
     begin
       //if count = Drag_selected then continue;
       x:=midX;
       y:=midY;
       //Form1.WS.Cursor := crCross;
     end;


    //Проверка по массиву кругов////////////////////////////////////////////////////////
    //Центр
    if (sqrt(sqr(abs(x-Circles[count].x))+sqr(abs(y-Circles[count].y)))<dist) then
     begin
       x:=Circles[count].x;
       y:=Circles[count].y;
       //Form1.WS.Cursor := crCross;
     end;
    //Лев.орд.
    if (sqrt(sqr((x-(Circles[count].x-Circles[count].r)))+sqr(abs(y-Circles[count].y)))<dist) then
     begin
       x:=Circles[count].x-Circles[count].r;
       y:=Circles[count].y;
       //Form1.WS.Cursor := crCross;
     end;
    //Прав.орд.
    if (sqrt(sqr((x-(Circles[count].x+Circles[count].r)))+sqr(abs(y-Circles[count].y)))<dist) then
     begin
       x:=Circles[count].x+Circles[count].r;
       y:=Circles[count].y;
       //Form1.WS.Cursor := crCross;
     end;
    //Верх.орд.
    if (sqrt(sqr(abs(x-Circles[count].x))+sqr((y-Circles[count].y-Circles[count].r)))<dist) then
     begin
       x:=Circles[count].x;
       y:=Circles[count].y+Circles[count].r;
       Form1.WS.Cursor := crCross;
     end;
    //Нижн.орд.
    if (sqrt(sqr(abs(x-Circles[count].x))+sqr((y-Circles[count].y+Circles[count].r)))<dist) then
     begin
       x:=Circles[count].x;
       y:=Circles[count].y-Circles[count].r;
       //Form1.WS.Cursor := crCross;
     end;


    //Проверка по массиву дуг////////////////////////////////////////////////////////////////
    if (sqrt(sqr(abs(x-Arcs[count].x1))+sqr(abs(y-Arcs[count].y1)))<dist) then
     begin
       x:=Arcs[count].x1;
       y:=Arcs[count].y1;
       //Form1.WS.Cursor := crCross;
     end;
    if (sqrt(sqr(abs(x-Arcs[count].x2))+sqr(abs(y-Arcs[count].y2)))<dist) then
     begin
       x:=Arcs[count].x2;
       y:=Arcs[count].y2;
       //Form1.WS.Cursor := crCross;
     end;

    //Проверка по массиву точек
    if (sqrt(sqr(abs(x-points[count].x))+sqr(abs(y-points[count].y)))<dist) then
     begin
       x:=points[count].x;
       y:=points[count].y;
       //Form1.WS.Cursor := crCross;
     end;
  end;

  //Проверка по массиву размеров//////////////////////////////////////////
  for count := 0 to High(DimAr) do
  begin
      if (sqrt(sqr(x-DimAr[count].x1)+sqr(y-DimAr[count].y1))<dist) then
       begin
          x:=DimAr[count].x1;
          y:=DimAr[count].y1;
          //Form1.WS.Cursor := crCross;
       end;
      if (sqrt(sqr(x-DimAr[count].x2)+sqr(y-DimAr[count].y2))<dist) then
       begin
          x:=DimAr[count].x2;
          y:=DimAr[count].y2;
          //Form1.WS.Cursor := crCross;
       end;
      if (sqrt(sqr(x-DimAr[count].x3)+sqr(y-DimAr[count].y3))<dist) then
       begin
          x:=DimAr[count].x3;
          y:=DimAr[count].y3;
          //Form1.WS.Cursor := crCross;
       end;
      if (sqrt(sqr(x-DimAr[count].x4)+sqr(y-DimAr[count].y4))<dist) then
       begin
          x:=DimAr[count].x4;
          y:=DimAr[count].y4;
          //Form1.WS.Cursor := crCross;
       end;
  end;
   /////////////////////////////////////////////////////////////////////////
end;

//Объектная привязка с игнорированием указанного объекта,
//т.е. привязываться будет к любому объекту, кроме указанного при вызове функции
//Используется, чтобы не привязываться к самому редактируемому объекту
procedure near_points(var x,y : double; dist, ob_type, ob_num:integer);

var
  count:integer;
  midX, midY : double;

begin
  if not Form1.SnapCheck.Checked then exit;

  Form1.WS.Cursor := crDefault;

  for count:=0 to Arr_length-1 do
  begin
    //Проверка по массиву линий

    //Вычисление середины отрезка
    midX := min(Lines[count].x1,Lines[count].x2)+
            ((max(Lines[count].x1,Lines[count].x2)-
            min(Lines[count].x1,Lines[count].x2)) / 2);
    midY := min(Lines[count].y1,Lines[count].y2)+
            ((max(Lines[count].y1,Lines[count].y2)-
            min(Lines[count].y1,Lines[count].y2)) / 2);

    if (ob_type = Line) and
       (ob_num = count) then continue;

    if (sqrt(sqr(abs(x-Lines[count].x1))+sqr(abs(y-Lines[count].y1)))<dist) then
     begin
       x:=Lines[count].x1;
       y:=Lines[count].y1;
       Form1.WS.Cursor := crCross;
       exit;
     end;
    if (sqrt(sqr(abs(x-Lines[count].x2))+sqr(abs(y-Lines[count].y2)))<dist) then
     begin
       x:=Lines[count].x2;
       y:=Lines[count].y2;
       Form1.WS.Cursor := crCross;
       exit;
     end;

    if (sqrt(sqr(abs(x-midX))+sqr(abs(y-midY)))<dist) then
     begin
       x:=midX;
       y:=midY;
       Form1.WS.Cursor := crCross;
       exit;
     end;

    //Проверка по массиву кругов////////////////////////////////////////////////////////
    if (ob_type = circle) and
       (ob_num = count) then continue;
    //Центр
    if (sqrt(sqr(abs(x-Circles[count].x))+sqr(abs(y-Circles[count].y)))<dist) then
     begin
       x:=Circles[count].x;
       y:=Circles[count].y;
       Form1.WS.Cursor := crCross;
       exit;
     end;
    //Лев.орд.
    if (sqrt(sqr((x-(Circles[count].x-Circles[count].r)))+sqr(abs(y-Circles[count].y)))<dist) then
     begin
       x:=Circles[count].x-Circles[count].r;
       y:=Circles[count].y;
       Form1.WS.Cursor := crCross;
       exit;
     end;
    //Прав.орд.
    if (sqrt(sqr((x-(Circles[count].x+Circles[count].r)))+sqr(abs(y-Circles[count].y)))<dist) then
     begin
       x:=Circles[count].x+Circles[count].r;
       y:=Circles[count].y;
       Form1.WS.Cursor := crCross;
       exit;
     end;
    //Верх.орд.
    if (sqrt(sqr(abs(x-Circles[count].x))+sqr((y-Circles[count].y-Circles[count].r)))<dist) then
     begin
       x:=Circles[count].x;
       y:=Circles[count].y+Circles[count].r;
       Form1.WS.Cursor := crCross;
       exit;
     end;
    //Нижн.орд.
    if (sqrt(sqr(abs(x-Circles[count].x))+sqr((y-Circles[count].y+Circles[count].r)))<dist) then
     begin
       x:=Circles[count].x;
       y:=Circles[count].y-Circles[count].r;
       Form1.WS.Cursor := crCross;
       exit;
     end;


    //Проверка по массиву дуг////////////////////////////////////////////////////////////////
    if (ob_type = arc) and
       (ob_num = count) then continue;

    if (sqrt(sqr(abs(x-Arcs[count].x1))+sqr(abs(y-Arcs[count].y1)))<dist) then
     begin
       x:=Arcs[count].x1;
       y:=Arcs[count].y1;
       Form1.WS.Cursor := crCross;
       exit;
     end;
    if (sqrt(sqr(abs(x-Arcs[count].x2))+sqr(abs(y-Arcs[count].y2)))<dist) then
     begin
       x:=Arcs[count].x2;
       y:=Arcs[count].y2;
       Form1.WS.Cursor := crCross;
       exit;
     end;

    //Проверка по массиву точек
    if (ob_type = dot) and
       (ob_num = count) then continue;

    if (sqrt(sqr(abs(x-points[count].x))+sqr(abs(y-points[count].y)))<dist) then
     begin
       x:=points[count].x;
       y:=points[count].y;
       Form1.WS.Cursor := crCross;
       exit;
     end;
  end;

  //Проверка по массиву размеров
  for count := 0 to High(DimAr) do
  begin
     if (ob_type = dim) and
        (ob_num = count) then continue;

     if (sqrt(sqr(abs(x-DimAr[count].x1))+sqr(abs(y-DimAr[count].y1)))<dist) then
     begin
       x:=points[count].x;
       y:=points[count].y;
       Form1.WS.Cursor := crCross;
       exit;
     end;

     if (sqrt(sqr(abs(x-DimAr[count].x2))+sqr(abs(y-DimAr[count].y2)))<dist) then
     begin
       x:=points[count].x;
       y:=points[count].y;
       Form1.WS.Cursor := crCross;
       exit;
     end;

     if (sqrt(sqr(abs(x-DimAr[count].x3))+sqr(abs(y-DimAr[count].y3)))<dist) then
     begin
       x:=points[count].x;
       y:=points[count].y;
       Form1.WS.Cursor := crCross;
       exit;
     end;

     if (sqrt(sqr(abs(x-DimAr[count].x4))+sqr(abs(y-DimAr[count].y4)))<dist) then
     begin
       x:=points[count].x;
       y:=points[count].y;
       Form1.WS.Cursor := crCross;
       exit;
     end;
  end;
end;

//Сохранить предыдущий шаг
procedure save_prev();
begin
   stad:=false;

   lines_vrem:=copy(lines);
   circles_vrem:=copy(circles);
   arcs_vrem:=copy(arcs);
   points_vrem:=copy(points);
   nadp_vrem:=copy(nadp);
   arrow_vrem:=copy(arrow);
   DimAr_vrem := copy(DimAr);

   k_vrem:=k;
   k_cir_vrem:=k_cir;
   k_arc_vrem:=k_arc;
   k_points_vrem:=k_points;
   k_nadp_vrem:=k_nadp;
   k_arrow_vrem:=k_arrow;

   NeedSave := true;
end;

//Сдвинуть один или несколько объектов по X или по Y
procedure Move_vect(side_x, side_y : integer);

var
 i:integer;
 count : integer;

begin
 TranslateSingleToMulti();

  //Если выбрано несколько объектов ///////////////////////////////////////////
 if (MultiSelect) then
 begin
    save_prev(); //Сохранить предыдущее состояние

    for count := 0 to High(multy_selected_line) do
    begin
       Lines[multy_selected_line[count]].x1+=side_x;
       Lines[multy_selected_line[count]].x2+=side_x;
       Lines[multy_selected_line[count]].y1+=side_y;
       Lines[multy_selected_line[count]].y2+=side_y;
    end;
    for count := 0 to High(multy_selected_circle) do
    begin
       circles[multy_selected_circle[count]].x+=side_x;
       circles[multy_selected_circle[count]].y+=side_y;
    end;
    for count := 0 to High(multy_selected_arcs) do
    begin
       arcs[multy_selected_arcs[count]].x1+=side_x;
       arcs[multy_selected_arcs[count]].y1+=side_y;
       arcs[multy_selected_arcs[count]].x2+=side_x;
       arcs[multy_selected_arcs[count]].y2+=side_y;
       arcs[multy_selected_arcs[count]].l_x+=side_x;
       arcs[multy_selected_arcs[count]].r_x+=side_x;
       arcs[multy_selected_arcs[count]].t_y+=side_y;
       arcs[multy_selected_arcs[count]].b_y+=side_y;
       arcs[multy_selected_arcs[count]].CentrX+=side_x;
       arcs[multy_selected_arcs[count]].CentrY+=side_y;
    end;
    for count := 0 to High(multy_selected_point) do
    begin
       points[multy_selected_point[count]].x+=side_x;
       points[multy_selected_point[count]].y+=side_y;
    end;
    for count := 0 to High(multy_selected_text) do
    begin
       nadp[multy_selected_text[count]].x+=side_x;
       nadp[multy_selected_text[count]].y+=side_y;
    end;
    for count := 0 to High(multy_selected_arrow) do
    begin
       arrow[multy_selected_arrow[count]].x+=side_x;
       arrow[multy_selected_arrow[count]].y+=side_y;
    end;
    for count := 0 to High(multy_selected_dim) do
    begin
       DimAr[multy_selected_dim[count]].x1+=side_x;
       DimAr[multy_selected_dim[count]].y1+=side_y;
       DimAr[multy_selected_dim[count]].x2+=side_x;
       DimAr[multy_selected_dim[count]].y2+=side_y;
       DimAr[multy_selected_dim[count]].x3+=side_x;
       DimAr[multy_selected_dim[count]].y3+=side_y;
       DimAr[multy_selected_dim[count]].x4+=side_x;
       DimAr[multy_selected_dim[count]].y4+=side_y;
       DimAr[multy_selected_dim[count]].nadpCX+=side_x;
       DimAr[multy_selected_dim[count]].nadpCY+=side_y;
    end;

    Form1.Start_timer_delay.Enabled:=true;
    exit;
 end;

    Form1.Start_timer_delay.Enabled:=true;
    Exit;


  //Сдвинуть все выбраные объекты
 If (select_type >= 1000000) and (select_type <= 1111111) then
 begin
       save_prev(); //Сохранить предыдущее состояние
       If IntToStr(select_type)[7] ='1' then  //Сдвинуть все линии
       begin
        for i:=k downto 0 do
         begin
           lines[i].x1+=side_x;
           lines[i].x2+=side_x;
           lines[i].y1+=side_y;
           lines[i].y2+=side_y;
          end;
       end;

       If IntToStr(select_type)[6] ='1' then   //Сдвинуть все окружности
       begin
        for i:=k_cir downto 0 do
         begin
           Circles[i].x+=side_x;
           Circles[i].y+=side_y;
          end;
       end;

       If IntToStr(select_type)[5] ='1' then   //Сдвинуть все точки
       begin
        for i:=k_points downto 0 do
         begin
           If (points[i].x<>0) and (points[i].y<>0) then
           begin
               points[i].x+=side_x;
               points[i].y+=side_y;
           end;
          end;
       end;

       If IntToStr(select_type)[4] ='1' then   //Сдвинуть все тексты
       begin
        for i:=k_nadp downto 0 do
         begin
           nadp[i].x+=side_x;
           nadp[i].y+=side_y;
          end;
       end;

       If IntToStr(select_type)[3] ='1' then   //Сдвинуть все стрелки
       begin
        for i:=k_arrow downto 0 do
         begin
           arrow[i].x+=side_x;
           arrow[i].y+=side_y;
          end;
       end;

       If IntToStr(select_type)[2] ='1' then   //Сдвинуть все дуги
       begin
        for i:=k_arc downto 0 do
         begin
           arcs[i].x1+=side_x;
           arcs[i].x2+=side_x;
           arcs[i].y1+=side_y;
           arcs[i].y2+=side_y;
           arcs[i].r_x+=side_x;
           arcs[i].l_x+=side_x;
           arcs[i].t_y+=side_y;
           arcs[i].b_y+=side_y;
          end;
       end;

 end;
 Form1.Start_timer_delay.Enabled:=true;
end;

//Удалить объект
procedure Delete_Ob();

var
 count : integer;
 vrem : integer = -1;

begin
   If (((select_type <= 1000000) or (select_type >= 1111111))) and
      (selected = 10000) and
      not (MultiSelect) then
      begin
        ShowMessage('Ни один объект не был выбран');
        exit;
      end;

   save_prev(); //Сохранить предыдущее состояние

 If selected <> 10000 then
    TranslateSingleToMulti;

    MyFunc.sort(multy_selected_line);
    MyFunc.sort(multy_selected_circle);
    MyFunc.sort(multy_selected_arcs);
    MyFunc.sort(multy_selected_point);
    MyFunc.sort(multy_selected_text);
    MyFunc.sort(multy_selected_arrow);

    for count := High(multy_selected_line) downto 0 do
    begin
        vrem := Prov_Sel_Block(FindLineNumber(lines[multy_selected_line[count]].id));
        if vrem <> -1 then
            DeleteBlock(vrem);

        Delete(lines,multy_selected_line[count],1);
        SetLength(lines,arr_length);
        k-=1;
    end;
    for count := High(multy_selected_circle) downto 0 do
    begin
        vrem := Prov_Sel_Block(FindCircleNumber(circles[multy_selected_circle[count]].id));
        if vrem <> -1 then
            DeleteBlock(vrem);

        Delete(circles,multy_selected_circle[count],1);
        SetLength(circles,arr_length);
        k_cir-=1;
    end;
    for count := High(multy_selected_arcs) downto 0 do
    begin
        vrem := Prov_Sel_Block(FindArcNumber(Arcs[multy_selected_arcs[count]].id));
        if vrem <> -1 then
            DeleteBlock(vrem);

        Delete(arcs,multy_selected_arcs[count],1);
        SetLength(arcs,arr_length);
        k_arc-=1;
    end;
    for count := High(multy_selected_point) downto 0 do
    begin
        vrem := Prov_Sel_Block(FindPointNumber(Points[multy_selected_point[count]].id));
        if vrem <> -1 then
            DeleteBlock(vrem);

        Delete(points,multy_selected_point[count],1);
        SetLength(points,arr_length);
        k_points-=1;
    end;
    for count := High(multy_selected_text) downto 0 do
    begin
        vrem := Prov_Sel_Block(FindTextNumber(Nadp[multy_selected_text[count]].id));
        if vrem <> -1 then
            DeleteBlock(vrem);

        Delete(nadp,multy_selected_text[count],1);
        SetLength(nadp,arr_length);
        k_nadp-=1;
    end;
    for count := High(multy_selected_arrow) downto 0 do
    begin
        vrem := Prov_Sel_Block(FindArrowNumber(arrow[multy_selected_arrow[count]].id));
        if vrem <> -1 then
            DeleteBlock(vrem);

        Delete(arrow,multy_selected_arrow[count],1);
        SetLength(arrow,arr_length);
        k_arrow-=1;
    end;
    for count := High(multy_selected_dim) downto 0 do
    begin
        //vrem := Prov_Sel_Block(FindArrowNumber(arrow[multy_selected_arrow[count]].id));
        //if vrem <> -1 then
        //    DeleteBlock(vrem);

        Delete(DimAr,multy_selected_dim[count],1);
    end;

    SetLength(multy_selected_line,0);
    SetLength(multy_selected_circle,0);
    SetLength(multy_selected_arcs,0);
    SetLength(multy_selected_point,0);
    SetLength(multy_selected_text,0);
    SetLength(multy_selected_arrow,0);
    SetLength(multy_selected_dim,0);

    if k<0 then k := 0;
    if k_cir<0 then k_cir := 0;
    if k_arc<0 then k_arc := 0;
    if k_points<0 then k_points := 0;
    if k_nadp<0 then k_nadp := 0;
    if k_arrow<0 then k_arrow := 0;

    selected := 10000;
    select_type := 0;

    clear;

    If (NeedHelp) then
       ShowMessage('Объекты удалены');

    exit;

  If (NeedHelp) then
  ShowMessage('Объекты удалены');

  Form1.StatusBar1.Panels.Items[0].Text:='Ничего не выбрано';
end;

//Удалить всё
procedure Clear_all_fig();

var
  i:integer;

begin
  save_prev(); //Сохранить предыдущее состояние

  for i:=k downto 0 do
   begin
     Delete(lines,i,1);
    end;
  for i:=k_cir downto 0 do
   begin
     Delete(Circles,i,1);
   end;
  for i:=k_arc downto 0 do
   begin
     Delete(Arcs,i,1);
   end;
  for i:=k_points downto 0 do
   begin
     Delete(points,i,1);
   end;
  for i:=k_nadp downto 0 do
   begin
     Delete(nadp,i,1);
   end;
  for i:=k_arrow downto 0 do
   begin
     Delete(arrow,i,1);
   end;

  SetLength(lines,arr_length);
  SetLength(Circles,arr_length);
  SetLength(points,arr_length);
  SetLength(nadp,arr_length);
  SetLength(arrow,arr_length);
  SetLength(blocks,0);
  SetLength(DimAr,0);

  k:=0; k_cir:=0; k_points:=0; k_nadp:=0; k_arrow:=0; k_arc:=0;
  id := 0;

  loc:='';
  f_name:='';
  podl:=false;

  Form1.add_back.Enabled:=false;
  Form1.Clear_podl.Enabled:=False;

 selected:=10000;
 Clear();

 Form1.StatusBar1.Panels.Items[0].Text:='Ничего не выбрано';
end;

//Выбор единичного объекта
function select(x,y:double):integer;

var
  count:integer;
  count2:integer;
  a,b,c,
  p : double;
  h : double = 0;
  //alpha,beta,gamma,
  angle_vrem : double;
  angle_vrem2 : double;
  angle_vrem3 : double;
  angle_vrem23 : double;

  LArcVrem : double = 0;

  //Расстояние от точки до отрезка (для выбора размеров)
  vremA, vremB, vremC : double;
  vremDistR, vremDistV1, vremDistV2 : double;

begin
  Form1.WS.Canvas.Pen.Color:=clRed;

  result:=10000;

  Form1.StatusBar1.Panels.Items[0].Text:='Ничего не выбрано';

  //Проверка выбора линии
  If Form1.SelectChoiceMenu.Checked[0] then
  for count:=0 to k-1 do
  begin
     if PointInLine(x,y) then exit; //Чтобы нельзя было выбрать линию за точку

     a:=sqrt(sqr(abs(Lines[count].x2-Lines[count].x1))+
                          sqr(abs(Lines[count].y2-Lines[count].y1)));

     b:=sqrt(sqr(abs(x-Lines[count].x1))+
                          sqr(abs(y-Lines[count].y1)));

     c:=sqrt(sqr(abs(x-Lines[count].x2))+
                          sqr(abs(y-Lines[count].y2)));

     p:=(a+b+c)/2;

     if (a<>0) and (b<>0) and (c<>0) then
           h:=2*sqrt(p*abs(p-a)*abs(p-b)*abs(p-c))/a;

     if (a=0) or (b=0) or (c=0) then continue;

     //alpha:=RadToDeg(ArcCos((Power(a,2)+Power(b,2)-Power(c,2))/(2*a*b)));
     //beta:=RadToDeg(ArcCos((Power(a,2)+Power(c,2)-Power(b,2))/(2*a*c)));
     //gamma:=RadToDeg(ArcCos((Power(b,2)+Power(c,2)-Power(a,2))/(2*b*c)));

     if (h<5)then
      begin
          if ((x+5) < min(round(lines[count].x1),round(lines[count].x2))) or
             ((x-5) > max(round(lines[count].x1),round(lines[count].x2))) or
             ((y+5) < min(round(lines[count].y1),round(lines[count].y2))) or
             ((y-5) > max(round(lines[count].y1),round(lines[count].y2))) then
             continue;

          if (lines[count].x1<>lines[count].x2) then
            angle_vrem :=  ArcTan(abs(lines[count].y1-lines[count].y2)/abs(lines[count].x1-lines[count].x2))
          else
            angle_vrem:=DegToRad(90);
          angle_vrem := RadToDeg(angle_vrem);

          if ((lines[count].x2>lines[count].x1) and
             (lines[count].y2>lines[count].y1)) or
             ((lines[count].x1>lines[count].x2) and
             (lines[count].y1>lines[count].y2))
             then angle_vrem := -angle_vrem;

          angle_vrem := SimpleRoundTo(angle_vrem,-3);
          Form1.StatusBar1.Panels.Items[0].Text:='L = '+FloatToStr(SimpleRoundTo(a,-3)) +
                                                 ' : Угол = '+ FloatToStr(angle_vrem);
          //////////////////////////////////////////////////////////////////////
          //Проверка выделения блока по примитиву Линия
          block_num := Prov_Sel_Block(lines[count].id);
          if block_num<>-1 then  //Линия входит в блок
             begin
                 //Reset_mode;

                 //чтобы нельзя было выбрать повторно один и тот же блок дважды
                 for count2 := 0 to High(multy_selected_line) do
                 if count = multy_selected_line[count2] then exit;

                 Select_block(block_num);
                 Form1.StatusBar1.Panels.Items[0].Text:='Блок №' + block_num.ToString;
                 exit;
             end
          //////////////////////////////////////////////////////////////////////
          else
             begin
                form1.WS.Canvas.Ellipse(Round(Lines[count].x1)-5+Screen_x,
                                        Round(Lines[count].y1)-5+screen_y,
                                        Round(Lines[count].x1)+5+Screen_x,
                                        Round(Lines[count].y1)+5+Screen_y);
                form1.WS.Canvas.Ellipse(Round(Lines[count].x2)-5+Screen_x,
                                        Round(Lines[count].y2)-5+Screen_y,
                                        Round(Lines[count].x2)+5+Screen_x,
                                        Round(Lines[count].y2)+5+Screen_y);

                //чтобы нельзя было выбрать повторно один и ту же линию дважды
                 for count2 := 0 to High(multy_selected_line) do
                 if count = multy_selected_line[count2] then exit;

                result:=count;
                select_type:=line;
                exit;
             end;
      end;
  end;

  //Проверка выбора круга
  If Form1.SelectChoiceMenu.Checked[1] then
  for count:=0 to k_cir-1 do
  begin
      if selected <> 10000 then continue;
      if (sqrt(sqr(x-Circles[count].x)+sqr(y-Circles[count].y))<Circles[count].r+5) and
         (sqrt(sqr(x-Circles[count].x)+sqr(y-Circles[count].y))>Circles[count].r-5) then
       begin
          //////////////////////////////////////////////////////////////////////
          //Проверка выделения блока по примитиву Круг
          block_num := Prov_Sel_Block(circles[count].id);
          if block_num<>-1 then  //Круг входит в блок
             begin
                 //Reset_mode;

                 //чтобы нельзя было выбрать повторно один и тот же блок дважды
                 for count2 := 0 to High(multy_selected_circle) do
                 if count = multy_selected_circle[count2] then exit;

                 Select_block(block_num);
                 Form1.StatusBar1.Panels.Items[0].Text:='Блок №' + block_num.ToString;
                 exit;
             end
          //////////////////////////////////////////////////////////////////////
          else
             begin
                  Form1.StatusBar1.Panels.Items[0].Text:='R = '+FloatToStr(Circles[count].r);

                  Form1.WS.Canvas.Ellipse(Round(Circles[count].x-Circles[count].r)-5+Screen_x,
                                          Round(Circles[count].y+Circles[count].r)+5+Screen_y,
                                          Round(Circles[count].x-Circles[count].r)+5+Screen_x,
                                          Round(Circles[count].y+Circles[count].r)-5+Screen_y);
                  Form1.WS.Canvas.Ellipse(Round(Circles[count].x-Circles[count].r)-5+Screen_x,
                                          Round(Circles[count].y-Circles[count].r)+5+Screen_y,
                                          Round(Circles[count].x-Circles[count].r)+5+Screen_x,
                                          Round(Circles[count].y-Circles[count].r)-5+Screen_y);
                  Form1.WS.Canvas.Ellipse(Round(Circles[count].x+Circles[count].r)-5+Screen_x,
                                          Round(Circles[count].y+Circles[count].r)+5+Screen_y,
                                          Round(Circles[count].x+Circles[count].r)+5+Screen_x,
                                          Round(Circles[count].y+Circles[count].r)-5+Screen_y);
                  Form1.WS.Canvas.Ellipse(Round(Circles[count].x+Circles[count].r)-5+Screen_x,
                                          Round(Circles[count].y-Circles[count].r)+5+Screen_y,
                                          Round(Circles[count].x+Circles[count].r)+5+Screen_x,
                                          Round(Circles[count].y-Circles[count].r)-5+Screen_y);
                  Form1.WS.Canvas.Ellipse(Round(Circles[count].x-3+Screen_x),
                                          Round(Circles[count].y-3+Screen_y),
                                          Round(Circles[count].x+3+Screen_x),
                                          Round(Circles[count].y+3+Screen_y));
                  //чтобы нельзя было выбрать повторно одну и ту же окружность дважды
                  for count2 := 0 to High(multy_selected_circle) do
                  if count = multy_selected_circle[count2] then exit;

                  result:=count;
                  select_type:=circle;

                  exit;
            end;
       end;
  end;

  //Проверка выбора дуги
  If Form1.SelectChoiceMenu.Checked[3] then
  for count:=0 to k_arc-1 do
  begin
      if selected <> 10000 then continue;
      //////////////////////////////////////////////////////////
      //Рассчитывает угол наклона луча из центра дуги через выбранную точку
      angle_vrem := CalcAngle(x+screen_X,y+screen_Y,
                              Arcs[count].CentrX,Arcs[count].Centry)+DegToRad(180);

      //Расчет, попадает ли луч между стартовым и конечным углами дуги
      angle_vrem2 := Arcs[count].StartAngle;
      angle_vrem3 := Arcs[count].EndAngle;

      while (angle_vrem > DegToRad(360)) do angle_vrem -=DegToRad(360);
      while (angle_vrem < DegToRad(0)) do angle_vrem +=DegToRad(360);

      while (angle_vrem2 > DegToRad(360)) do angle_vrem2 -=DegToRad(360);
      while (angle_vrem2 < DegToRad(0)) do angle_vrem2 +=DegToRad(360);

      while (angle_vrem3 > DegToRad(360)) do angle_vrem3 -=DegToRad(360);
      while (angle_vrem3 < DegToRad(0)) do angle_vrem3 +=DegToRad(360);

      angle_vrem23 := min(angle_vrem2, angle_vrem3);
      angle_vrem3 :=  max(angle_vrem2, angle_vrem3);
      angle_vrem2 :=  angle_vrem23;
      ////////////////////////////////////////////////////////////

      if ((sqrt(sqr(x+screen_X-Arcs[count].CentrX)+
              sqr(y+screen_Y-Arcs[count].Centry)) > Arcs[count].rad - 7) and
         (sqrt(sqr(x+screen_X-Arcs[count].CentrX)+
              sqr(y+screen_Y-Arcs[count].Centry)) < Arcs[count].rad + 7)) and
         (angle_vrem > angle_vrem2) and
         (angle_vrem < angle_vrem3)
         then
       begin
            //////////////////////////////////////////////////////////////////////
            //Проверка выделения блока по примитиву Дуга
            block_num := Prov_Sel_Block(arcs[count].id);
            if block_num<>-1 then  //Дуга входит в блок
             begin
                 //Reset_mode;

                 //чтобы нельзя было выбрать повторно один и тот же блок дважды
                 for count2 := 0 to High(multy_selected_arcs) do
                 if count = multy_selected_arcs[count2] then exit;

                 Select_block(block_num);
                 Form1.StatusBar1.Panels.Items[0].Text:='Блок №' + block_num.ToString;
                 exit;
             end
          //////////////////////////////////////////////////////////////////////
            else
            begin
                LArcVrem := Arcs[count].rad*2*Pi *
                            abs(Arcs[count].StartAngle-
                                Arcs[count].EndAngle)/
                            DegToRad(360);
                Form1.StatusBar1.Panels.Items[0].Text:='Дуга. Длина = '+
                                                       floatToSTr(LArcVrem);

                Form1.WS.Canvas.Ellipse(Round(Arcs[count].x1-5+Screen_x),
                                        Round(Arcs[count].y1-5+Screen_y),
                                        Round(Arcs[count].x1+5+Screen_x),
                                        Round(Arcs[count].y1+5+Screen_y));
                Form1.WS.Canvas.Ellipse(Round(Arcs[count].x2-5+Screen_x),
                                        Round(Arcs[count].y2-5+Screen_y),
                                        Round(Arcs[count].x2+5+Screen_x),
                                        Round(Arcs[count].y2+5+Screen_y));

                //чтобы нельзя было выбрать повторно одну и ту же дугу дважды
                for count2 := 0 to High(multy_selected_arcs) do
                if count = multy_selected_arcs[count2] then exit;

                result:=count;
                select_type:=arc;

                exit;
            end;
       end;
  end;

   //Проверка выбора точки
  If Form1.SelectChoiceMenu.Checked[2] then
  for count:=0 to k_points do
  begin
      if (sqrt(sqr(abs(x-points[count].x))+sqr(abs(y-points[count].y)))<5) then
       begin
            if selected <> 10000 then continue;
            //////////////////////////////////////////////////////////////////////
            //Проверка выделения блока по примитиву Точка
            block_num := Prov_Sel_Block(points[count].id);
            if block_num<>-1 then  //Точка входит в блок
             begin
                 //Reset_mode;

                 //чтобы нельзя было выбрать повторно один и тот же блок дважды
                 for count2 := 0 to High(multy_selected_point) do
                 if count = multy_selected_point[count2] then exit;

                 Select_block(block_num);
                 Form1.StatusBar1.Panels.Items[0].Text:='Блок №' + block_num.ToString;
                 exit;
             end
          //////////////////////////////////////////////////////////////////////
            else
            begin
                Form1.WS.Canvas.Ellipse(Round(points[count].x)-15+Screen_x,
                                        Round(points[count].y)+15+Screen_y,
                                        Round(points[count].x)+15+Screen_x,
                                        Round(points[count].y)-15+Screen_y);
                Form1.StatusBar1.Panels.Items[0].Text:='Точка';

                //чтобы нельзя было выбрать повторно одну и ту же точку дважды
                for count2 := 0 to High(multy_selected_point) do
                if count = multy_selected_point[count2] then exit;
                result:=count;
                select_type:=dot;

                exit;
            end;
       end;
  end;

   //Проверка выбора текста
  If Form1.SelectChoiceMenu.Checked[4] then
  for count:=0 to k_nadp do
  begin
      if (nadp[count].x<x) and
             ((nadp[count].x+Length(nadp[count].s)*nadp[count].h)>x) and
             (nadp[count].y<y) and
             ((nadp[count].y+nadp[count].h)>y) then
       begin
            //////////////////////////////////////////////////////////////////////
            //Проверка выделения блока по примитиву Надпись
            block_num := Prov_Sel_Block(nadp[count].id);
            if block_num<>-1 then  //Надпись входит в блок
             begin
                 //Reset_mode;

                 //чтобы нельзя было выбрать повторно один и тот же блок дважды
                 for count2 := 0 to High(multy_selected_text) do
                 if count = multy_selected_text[count2] then exit;

                 Select_block(block_num);
                 Form1.StatusBar1.Panels.Items[0].Text:='Блок №' + block_num.ToString;
                 exit;
             end
          //////////////////////////////////////////////////////////////////////
            else
            begin
                Form1.WS.Canvas.Font.Size:=Round(nadp[count].h);
                Form1.WS.Canvas.Font.Color:= clRed;
                Form1.WS.Canvas.TextOut(Round(nadp[count].x+Screen_x),
                                        Round(nadp[count].y+Screen_y),
                                        nadp[count].s);
                Form1.StatusBar1.Panels.Items[0].Text:='Текст';

                //чтобы нельзя было выбрать повторно один и тот же текст дважды
                for count2 := 0 to High(multy_selected_text) do
                if count = multy_selected_text[count2] then exit;
                result:=count;
                select_type:=txt;

                exit;
            end;
       end;
  end;

  //Проверка выбора стрелки
  If Form1.SelectChoiceMenu.Checked[4] then
  for count:=0 to k_arrow do
  begin
      if (sqrt(sqr(abs(x-arrow[count].x))+sqr(abs(y-arrow[count].y)))<5) then
       begin
            //////////////////////////////////////////////////////////////////////
            //Проверка выделения блока по примитиву Стрелка
            block_num := Prov_Sel_Block(arrow[count].id);
            if block_num<>-1 then  //Стрелка входит в блок
             begin
                 //Reset_mode;

                 //чтобы нельзя было выбрать повторно один и тот же блок дважды
                 for count2 := 0 to High(multy_selected_arrow) do
                 if count = multy_selected_arrow[count2] then exit;

                 Select_block(block_num);
                 Form1.StatusBar1.Panels.Items[0].Text:='Блок №' + block_num.ToString;
                 exit;
             end
          //////////////////////////////////////////////////////////////////////
            else
            begin
                Form1.WS.Canvas.Ellipse(Round(arrow[count].x-5),Round(arrow[count].y+5),
                                        Round(arrow[count].x+5),Round(arrow[count].y-5));
                if arrow[count].t = 'left' then
                      Form1.WS.Canvas.Ellipse(Round(arrow[count].x+30-5+Screen_x),
                                              Round(arrow[count].y+5+Screen_y),
                                              Round(arrow[count].x+30+5+Screen_x),
                                              Round(arrow[count].y-5+Screen_y));
                if arrow[count].t = 'right' then
                      Form1.WS.Canvas.Ellipse(Round(arrow[count].x-30-5+Screen_x),
                                              Round(arrow[count].y+5+Screen_y),
                                              Round(arrow[count].x-30+5+Screen_x),
                                              Round(arrow[count].y-5+Screen_y));
                if arrow[count].t = 'up' then
                      Form1.WS.Canvas.Ellipse(Round(arrow[count].x-5+Screen_x),
                                              Round(arrow[count].y+30+5+Screen_y),
                                              Round(arrow[count].x+5+Screen_x),
                                              Round(arrow[count].y+30-5+Screen_y));
                if arrow[count].t = 'down' then
                      Form1.WS.Canvas.Ellipse(Round(arrow[count].x-5+Screen_x),
                                              Round(arrow[count].y-30+5+Screen_y),
                                              Round(arrow[count].x+5+Screen_x),
                                              Round(arrow[count].y-30-5+Screen_y));
                Form1.StatusBar1.Panels.Items[0].Text:='Стрелка';

                //чтобы нельзя было выбрать повторно одну и ту же стрелку дважды
                for count2 := 0 to High(multy_selected_arrow) do
                if count = multy_selected_arrow[count2] then exit;
                result:=count;
                select_type:=arr;

                exit;
            end;
       end;
  end;

  //Проверка выбора размера
  If Form1.SelectChoiceMenu.Checked[4] then
  for count := 0 to High(DimAr) do
  begin
      if PointInDim(x,y) then exit;

      //длина перпендикуляра до размерной линии
      vremA := sqrt(sqr(x-DimAr[count].x3)+
                    sqr(y-DimAr[count].y3));
      vremB := sqrt(sqr(x-DimAr[count].x4)+
                    sqr(y-DimAr[count].y4));
      vremC := sqrt(sqr(DimAr[count].x3-DimAr[count].x4)+
                    sqr(DimAr[count].y3-DimAr[count].y4));
      vremDistR := 2*Area3(vremA,vremB,vremC)/vremC;

      //длина перпендикуляра до 1 выносной линии
      vremA := sqrt(sqr(x-DimAr[count].x3)+
                    sqr(y-DimAr[count].y3));
      vremB := sqrt(sqr(x-DimAr[count].x1)+
                    sqr(y-DimAr[count].y1));
      vremC := sqrt(sqr(DimAr[count].x3-DimAr[count].x1)+
                    sqr(DimAr[count].y3-DimAr[count].y1));

      If vremC = 0 then exit;

      vremDistV1 := 2*Area3(vremA,vremB,vremC)/vremC;

      //длина перпендикуляра до 2 выносной линии
      vremA := sqrt(sqr(x-DimAr[count].x4)+
                    sqr(y-DimAr[count].y4));
      vremB := sqrt(sqr(x-DimAr[count].x2)+
                    sqr(y-DimAr[count].y2));
      vremC := sqrt(sqr(DimAr[count].x4-DimAr[count].x2)+
                    sqr(DimAr[count].y4-DimAr[count].y2));
      vremDistV2 := 2*Area3(vremA,vremB,vremC)/vremC;

      if (sqrt(sqr(abs(x-DimAr[count].nadpCX))+sqr(abs(y-DimAr[count].nadpCY))) < 15) //выбран размер по надписи
         or
        (vremDistR<5)  //до размерной линии
         or
        (vremDistV1<5)  //до 1 выносной линии
         or
        (vremDistV2<5)  //до 2 выносной линии
        then
        begin
          //////////////////////////////////////////////////////////////////////
          //Проверка выделения блока по примитиву размер
          block_num := Prov_Sel_Block(DimAr[count].id);
          if block_num<>-1 then  //Размер входит в блок
           begin
               //Reset_mode;

               //чтобы нельзя было выбрать повторно один и тот же блок дважды
               for count2 := 0 to High(multy_selected_dim) do
               if count = multy_selected_dim[count2] then exit;

               Select_block(block_num);
               Form1.StatusBar1.Panels.Items[0].Text:='Блок №' + block_num.ToString;
               exit;
           end
          //////////////////////////////////////////////////////////////////////
          else
          begin
              Form1.WS.Canvas.Pen.Color := clRed;
              Form1.WS.Canvas.Ellipse(Round(DimAr[count].x3-5+Screen_X), Round(DimAr[count].y3-5+Screen_Y),
                                      Round(DimAr[count].x3+5+Screen_X), Round(DimAr[count].y3+5+Screen_Y));
              Form1.WS.Canvas.Ellipse(Round(DimAr[count].x4-5+Screen_X), Round(DimAr[count].y4-5+Screen_Y),
                                      Round(DimAr[count].x4+5+Screen_X), Round(DimAr[count].y4+5+Screen_Y));

              Form1.WS.Canvas.Ellipse(Round(DimAr[count].x1-5+Screen_X), Round(DimAr[count].y1-5+Screen_Y),
                                      Round(DimAr[count].x1+5+Screen_X), Round(DimAr[count].y1+5+Screen_Y));
              Form1.WS.Canvas.Ellipse(Round(DimAr[count].x2-5+Screen_X), Round(DimAr[count].y2-5+Screen_Y),
                                      Round(DimAr[count].x2+5+Screen_X), Round(DimAr[count].y2+5+Screen_Y));

              Form1.StatusBar1.Panels.Items[0].Text:='Линейный размер';

              //чтобы нельзя было выбрать повторно один и тот же размер дважды
              for count2 := 0 to High(multy_selected_dim) do
              if count = multy_selected_dim[count2] then exit;
              result:=count;
              select_type:=dim;

              exit;
          end;
        end;
  end;

  Form1.WS.Canvas.Pen.Color:=clBlack;
end;

//Поворот размеров (вынес поворот размеров в отдельную проведуру, т.к. много параметров)
procedure RotDimension(xr,yr : double; count : integer; angle : double);

var
  rad : double;
  angle_start : double;

begin
  rad := sqrt(sqr(DimAr[multy_selected_dim[count]].x1-xr)+
                        sqr(DimAr[multy_selected_dim[count]].y1-yr));

  if (xr<DimAr[multy_selected_dim[count]].x1) then  //кон.точка справа от точки вращения
  begin
      angle_start := math.DegToRad(90)- ArcTan((DimAr[multy_selected_dim[count]].y1-yr)/
                            (DimAr[multy_selected_dim[count]].x1-xr));
      DimAr[multy_selected_dim[count]].x1 := xr + (rad * sin(angle + angle_start));
      DimAr[multy_selected_dim[count]].y1 := yr + (rad * cos(angle + angle_start));
  end
  else                                               //кон.точка слева от точки вращения
  begin
       angle_start := -math.DegToRad(90)-ArcTan((DimAr[multy_selected_dim[count]].y1-yr)/
                        (DimAr[multy_selected_dim[count]].x1-xr));
       DimAr[multy_selected_dim[count]].x1 := xr - (rad * sin(math.DegToRad(180) + angle + angle_start));
       DimAr[multy_selected_dim[count]].y1 := yr + (rad * cos(angle + angle_start));
  end;

  rad := sqrt(sqr(DimAr[multy_selected_dim[count]].x2-xr)+
              sqr(DimAr[multy_selected_dim[count]].y2-yr));

  if (xr<DimAr[multy_selected_dim[count]].x2) then  //кон.точка справа от точки вращения
  begin
      angle_start := math.DegToRad(90)- ArcTan((DimAr[multy_selected_dim[count]].y2-yr)/
                            (DimAr[multy_selected_dim[count]].x2-xr));
      DimAr[multy_selected_dim[count]].x2 := xr + (rad * sin(angle + angle_start));
      DimAr[multy_selected_dim[count]].y2 := yr + (rad * cos(angle + angle_start));
  end
  else                                               //кон.точка слева от точки вращения
  begin
       angle_start := -math.DegToRad(90)-ArcTan((DimAr[multy_selected_dim[count]].y2-yr)/
                        (DimAr[multy_selected_dim[count]].x2-xr));
       DimAr[multy_selected_dim[count]].x2 := xr - (rad * sin(math.DegToRad(180) + angle + angle_start));
       DimAr[multy_selected_dim[count]].y2 := yr + (rad * cos(angle + angle_start));
  end;


  rad := sqrt(sqr(DimAr[multy_selected_dim[count]].x3-xr)+
              sqr(DimAr[multy_selected_dim[count]].y3-yr));

  if (xr<DimAr[multy_selected_dim[count]].x3) then  //кон.точка справа от точки вращения
  begin
      angle_start := math.DegToRad(90)- ArcTan((DimAr[multy_selected_dim[count]].y3-yr)/
                            (DimAr[multy_selected_dim[count]].x3-xr));
      DimAr[multy_selected_dim[count]].x3 := xr + (rad * sin(angle + angle_start));
      DimAr[multy_selected_dim[count]].y3 := yr + (rad * cos(angle + angle_start));
  end
  else                                               //кон.точка слева от точки вращения
  begin
       angle_start := -math.DegToRad(90)-ArcTan((DimAr[multy_selected_dim[count]].y3-yr)/
                        (DimAr[multy_selected_dim[count]].x3-xr));
       DimAr[multy_selected_dim[count]].x3 := xr - (rad * sin(math.DegToRad(180) + angle + angle_start));
       DimAr[multy_selected_dim[count]].y3 := yr + (rad * cos(angle + angle_start));
  end;

  rad := sqrt(sqr(DimAr[multy_selected_dim[count]].x4-xr)+
              sqr(DimAr[multy_selected_dim[count]].y4-yr));

  if (xr<DimAr[multy_selected_dim[count]].x4) then  //кон.точка справа от точки вращения
  begin
      angle_start := math.DegToRad(90)- ArcTan((DimAr[multy_selected_dim[count]].y4-yr)/
                            (DimAr[multy_selected_dim[count]].x4-xr));
      DimAr[multy_selected_dim[count]].x4 := xr + (rad * sin(angle + angle_start));
      DimAr[multy_selected_dim[count]].y4 := yr + (rad * cos(angle + angle_start));
  end
  else                                               //кон.точка слева от точки вращения
  begin
       angle_start := -math.DegToRad(90)-ArcTan((DimAr[multy_selected_dim[count]].y4-yr)/
                        (DimAr[multy_selected_dim[count]].x4-xr));
       DimAr[multy_selected_dim[count]].x4 := xr - (rad * sin(math.DegToRad(180) + angle + angle_start));
       DimAr[multy_selected_dim[count]].y4 := yr + (rad * cos(angle + angle_start));
  end;

  //Середина размерной линии
  DimAr[multy_selected_dim[count]].nadpCX := min(DimAr[multy_selected_dim[count]].x3,DimAr[multy_selected_dim[count]].x4) +
                                             abs(DimAr[multy_selected_dim[count]].x3 - DimAr[multy_selected_dim[count]].x4)/2;
  DimAr[multy_selected_dim[count]].nadpCY := min(DimAr[multy_selected_dim[count]].y3,DimAr[multy_selected_dim[count]].y4) +
                                             abs(DimAr[multy_selected_dim[count]].y3 - DimAr[multy_selected_dim[count]].y4)/2;

end;

//Поворот фигуры
procedure Rotate(xr,yr : double);

var
  Angle, angle_start:double;

  count : integer;
  rad : double;

  VremPoint1, VremPoint2 : MyTPoint;
  VremRad : double;

  //Текущие значения
  VremLineWeight : byte;
  VremLineStyle: TPenStyle;
  VremColor : TColor;

begin
   if (Selected<> 10000) then
      TranslateSingleToMulti;

      save_prev(); //Сохранить предыдущее состояние

      try
         Angle:=StrToFloat(InputBox('Поворот фигуры','Введи угол против час.стр.','0'));
         Angle:=Angle*System.Pi/180;
      except
         Angle:=0;
      end;

      //Поворот ОТРЕЗКОВ
      for count := 0 to High(multy_selected_line) do
      begin
          //1 точка
          rad := sqrt(sqr(lines[multy_selected_line[count]].y1-yr)+
                      sqr(lines[multy_selected_line[count]].x1-xr));

          if (lines[multy_selected_line[count]].x1<>xr) then
             angle_start := math.DegToRad(90)- ArcTan((lines[multy_selected_line[count]].y1-yr)/
                                (lines[multy_selected_line[count]].x1-xr))
          else
             angle_start := 0;

          if (xr<lines[multy_selected_line[count]].x1) then  //кон.точка справа от точки вращения
          begin
               lines[multy_selected_line[count]].x1 := xr + (rad * sin(angle + angle_start));
               lines[multy_selected_line[count]].y1 := yr + (rad * cos(angle + angle_start));
          end
          else if (xr>lines[multy_selected_line[count]].x1)  then//кон.точка слева от точки вращения
          begin
               If (lines[multy_selected_line[count]].x1<>xr) then
                   angle_start := -math.DegToRad(90)-ArcTan((lines[multy_selected_line[count]].y1-yr)/
                                (lines[multy_selected_line[count]].x1-xr))
               else
                   angle_start := 0;
               lines[multy_selected_line[count]].x1 := xr - (rad * sin(math.DegToRad(180) + angle + angle_start));
               lines[multy_selected_line[count]].y1 := yr + (rad * cos(angle + angle_start));
          end
          else  //Точка лежит на верт.линии, проходящей через центр вращения
          begin
               if lines[multy_selected_line[count]].y1 < yr then
               begin
                 lines[multy_selected_line[count]].x1 := xr - (rad * sin(math.DegToRad(180) - angle));
                 lines[multy_selected_line[count]].y1 := yr - (rad * cos(angle));
               end
               else
               begin
                 lines[multy_selected_line[count]].x1 := xr + (rad * sin(math.DegToRad(180) - angle));
                 lines[multy_selected_line[count]].y1 := yr + (rad * cos(angle));
               end;
          end;

          //2 точка
          rad := sqrt(sqr(lines[multy_selected_line[count]].y2-yr)+
                      sqr(lines[multy_selected_line[count]].x2-xr));

          if (lines[multy_selected_line[count]].x2<>xr) then
             angle_start := math.DegToRad(90)-ArcTan((lines[multy_selected_line[count]].y2-yr)/
                                (lines[multy_selected_line[count]].x2-xr))
          else
             angle_start := 0;

          if (xr<lines[multy_selected_line[count]].x2) then  //кон.точка справа от точки вращения
          begin
               lines[multy_selected_line[count]].x2 := xr + (rad * sin(angle + angle_start));
               lines[multy_selected_line[count]].y2 := yr + (rad * cos(angle + angle_start));
          end
          else if (xr>lines[multy_selected_line[count]].x2) then //кон.точка слева от точки вращения
          begin
               if (lines[multy_selected_line[count]].x2<>xr) then
                   angle_start := -math.DegToRad(90)-ArcTan((lines[multy_selected_line[count]].y2-yr)/
                                (lines[multy_selected_line[count]].x2-xr))
               else
                   angle_start := 0;
               lines[multy_selected_line[count]].x2 := xr - (rad * sin(math.DegToRad(180) + angle + angle_start));
               lines[multy_selected_line[count]].y2 := yr + (rad * cos(angle + angle_start));
          end
          else  //Точка лежит на верт.линии, проходящей через центр вращения
          begin
               if lines[multy_selected_line[count]].y2 < yr then
               begin
                 lines[multy_selected_line[count]].x2 := xr - (rad * sin(math.DegToRad(180) - angle));
                 lines[multy_selected_line[count]].y2 := yr - (rad * cos(angle));
               end
               else
               begin
                 lines[multy_selected_line[count]].x2 := xr + (rad * sin(math.DegToRad(180) - angle));
                 lines[multy_selected_line[count]].y2 := yr + (rad * cos(angle));
               end;
          end;
      end;

      //Поворот ОКРУЖНОСТЕЙ
      for count := 0 to High(multy_selected_circle) do
      begin
          rad := sqrt(sqr(circles[multy_selected_circle[count]].y-yr)+
                      sqr(circles[multy_selected_circle[count]].x-xr));

          if (xr<circles[multy_selected_circle[count]].x) then  //кон.точка справа от точки вращения
          begin
              if (circles[multy_selected_circle[count]].x<>xr) then
                  angle_start := math.DegToRad(90)- ArcTan((circles[multy_selected_circle[count]].y-yr)/
                                    (circles[multy_selected_circle[count]].x-xr))
              else
                  angle_start := 0;
              circles[multy_selected_circle[count]].x := xr + (rad * sin(angle + angle_start));
              circles[multy_selected_circle[count]].y := yr + (rad * cos(angle + angle_start));
          end
          else                                                  //кон.точка слева от точки вращения
          begin
               if (circles[multy_selected_circle[count]].x<>xr) then
                  angle_start := -math.DegToRad(90)-ArcTan((circles[multy_selected_circle[count]].y-yr)/
                                (circles[multy_selected_circle[count]].x-xr))
               else
                  angle_start := 0;

               circles[multy_selected_circle[count]].x := xr - (rad * sin(math.DegToRad(180) + angle + angle_start));
               circles[multy_selected_circle[count]].y := yr + (rad * cos(angle + angle_start));
          end;
      end;

      //Поворот ТОЧЕК
      for count := 0 to High(multy_selected_point) do
      begin
          rad := sqrt(sqr(points[multy_selected_point[count]].y-yr)+
                      sqr(points[multy_selected_point[count]].x-xr));

          if (xr<points[multy_selected_point[count]].x) then  //кон.точка справа от точки вращения
          begin
              if (points[multy_selected_point[count]].x<>xr) then
                 angle_start := math.DegToRad(90)- ArcTan((points[multy_selected_point[count]].y-yr)/
                                    (points[multy_selected_point[count]].x-xr))
              else
                 angle_start := 0;
              points[multy_selected_point[count]].x := xr + (rad * sin(angle + angle_start));
              points[multy_selected_point[count]].y := yr + (rad * cos(angle + angle_start));
          end
          else                                                //кон.точка слева от точки вращения
          begin
               if (points[multy_selected_point[count]].x<>xr) then
                  angle_start := -math.DegToRad(90)-ArcTan((points[multy_selected_point[count]].y-yr)/
                                (points[multy_selected_point[count]].x-xr))
               else
                  angle_start := 0;
               points[multy_selected_point[count]].x := xr - (rad * sin(math.DegToRad(180) + angle + angle_start));
               points[multy_selected_point[count]].y := yr + (rad * cos(angle + angle_start));
          end;
      end;

      //Поворот ТЕКСТОВ
      for count := 0 to High(multy_selected_text) do
      begin
          rad := sqrt(sqr(nadp[multy_selected_text[count]].y-yr)+
                      sqr(nadp[multy_selected_text[count]].x-xr));

          if (xr<nadp[multy_selected_text[count]].x) then  //кон.точка справа от точки вращения
          begin
              if (nadp[multy_selected_text[count]].x<>xr) then
                 angle_start := math.DegToRad(90)- ArcTan((nadp[multy_selected_text[count]].y-yr)/
                                    (nadp[multy_selected_text[count]].x-xr))
              else
                 angle_start := 0;
              nadp[multy_selected_text[count]].x := xr + (rad * sin(angle + angle_start));
              nadp[multy_selected_text[count]].y := yr + (rad * cos(angle + angle_start));
          end
          else                                             //кон.точка слева от точки вращения
          begin
               if (nadp[multy_selected_text[count]].x<>xr) then
                  angle_start := -math.DegToRad(90)-ArcTan((nadp[multy_selected_text[count]].y-yr)/
                                (nadp[multy_selected_text[count]].x-xr))
               else
                  angle_start := 0;
               nadp[multy_selected_text[count]].x := xr - (rad * sin(math.DegToRad(180) + angle + angle_start));
               nadp[multy_selected_text[count]].y := yr + (rad * cos(angle + angle_start));
          end;
      end;

      //Поворот СТРЕЛОК
      for count := 0 to High(multy_selected_arrow) do
      begin
          rad := sqrt(sqr(arrow[multy_selected_arrow[count]].y-yr)+
                      sqr(arrow[multy_selected_arrow[count]].x-xr));

          if (xr<arrow[multy_selected_arrow[count]].x) then  //кон.точка справа от точки вращения
          begin
              angle_start := math.DegToRad(90)- ArcTan((arrow[multy_selected_arrow[count]].y-yr)/
                                    (arrow[multy_selected_arrow[count]].x-xr));
              arrow[multy_selected_arrow[count]].x := xr + (rad * sin(angle + angle_start));
              arrow[multy_selected_arrow[count]].y := yr + (rad * cos(angle + angle_start));
          end
          else                                               //кон.точка слева от точки вращения
          begin
               angle_start := -math.DegToRad(90)-ArcTan((arrow[multy_selected_arrow[count]].y-yr)/
                                (arrow[multy_selected_arrow[count]].x-xr));
               arrow[multy_selected_arrow[count]].x := xr - (rad * sin(math.DegToRad(180) + angle + angle_start));
               arrow[multy_selected_arrow[count]].y := yr + (rad * cos(angle + angle_start));
          end;
      end;


      //Поворот ДУГ
      for count := High(multy_selected_arcs) downto 0 do
      begin
          //Старая дуга должна удаляться, а строиться новая по пересчитанным координатам.
          rad := sqrt(sqr(arcs[multy_selected_arcs[count]].y1-yr)+
                      sqr(arcs[multy_selected_arcs[count]].x1-xr));

          if (xr<arcs[multy_selected_arcs[count]].x1) then  //кон.точка справа от точки вращения
          begin
              if (arcs[multy_selected_arcs[count]].x1<>xr) then
                 angle_start := math.DegToRad(90)- ArcTan((arcs[multy_selected_arcs[count]].y1-yr)/
                                (arcs[multy_selected_arcs[count]].x1-xr))
              else
                 angle_start := 0;
              VremPoint1.x := xr + (rad * sin(angle + angle_start));
              VremPoint1.y := yr + (rad * cos(angle + angle_start));
          end
          else                                             //кон.точка слева от точки вращения
          begin
               if (arcs[multy_selected_arcs[count]].x1<>xr) then
                  angle_start := -math.DegToRad(90)-ArcTan((arcs[multy_selected_arcs[count]].y1-yr)/
                                (arcs[multy_selected_arcs[count]].x1-xr))
               else
                  angle_start := 0;
               VremPoint1.x := xr - (rad * sin(math.DegToRad(180) + angle + angle_start));
               VremPoint1.y := yr + (rad * cos(angle + angle_start));
          end;

          rad := sqrt(sqr(arcs[multy_selected_arcs[count]].y2-yr)+
                      sqr(arcs[multy_selected_arcs[count]].x2-xr));

          if (xr<arcs[multy_selected_arcs[count]].x2) then  //кон.точка справа от точки вращения
          begin
              if (arcs[multy_selected_arcs[count]].x2<>xr) then
                 angle_start := math.DegToRad(90)- ArcTan((arcs[multy_selected_arcs[count]].y2-yr)/
                                (arcs[multy_selected_arcs[count]].x2-xr))
              else
                 angle_start := 0;
              VremPoint2.x := xr + (rad * sin(angle + angle_start));
              VremPoint2.y := yr + (rad * cos(angle + angle_start));
          end
          else                                             //кон.точка слева от точки вращения
          begin
               if (arcs[multy_selected_arcs[count]].x2<>xr) then
                  angle_start := -math.DegToRad(90)-ArcTan((arcs[multy_selected_arcs[count]].y2-yr)/
                                (arcs[multy_selected_arcs[count]].x2-xr))
               else
                  angle_start := 0;
               VremPoint2.x := xr - (rad * sin(math.DegToRad(180) + angle + angle_start));
               VremPoint2.y := yr + (rad * cos(angle + angle_start));
          end;

          //Текущие значения  параметров линии
          VremLineWeight := LineWeight;
          VremLineStyle := LineStyle;
          VremColor := Cvet;

          VremRad := arcs[multy_selected_arcs[count]].rad;

          //Скопировать значения  параметров линии со старой дуги
          Cvet := arcs[multy_selected_arcs[count]].col;
          LineStyle := arcs[multy_selected_arcs[count]].style ;
          LineWeight := arcs[multy_selected_arcs[count]].Width;

          Delete(arcs,multy_selected_arcs[count],1);
          SetLength(arcs,Length(arcs)+1);
          k_arc -= 1;
          Draw_arc(VremPoint1,VremPoint2,VremRad);

          //Установить предыдущие значения параметров линии
          Cvet := VremColor;
          LineStyle := VremLineStyle;
          LineWeight := VremLineWeight;
      end;

      //Поворот РАЗМЕРОВ
      for count := High(multy_selected_dim) downto 0 do
      begin
          RotDimension(xr,yr,count, angle);
      end;

      clear;
end;

//Перемещение объектов указанием мыши
Procedure MoveObj(x_vrem, y_vrem : double);

var
  count : integer;

begin
   near_points(x_vrem,y_vrem,4);

   if (stad_copy = false) then
   begin
        x_copy:=x_vrem; y_copy:=y_vrem;
        stad_copy:=true;
        Form1.StatusBar1.Panels.Items[0].Text:='Куда переместить?';
   end
   else
   begin
        save_prev();

        delta_x:=x_vrem-x_copy;
        delta_y:=y_vrem-y_copy;

       //Перевод из ед.выбора и мульти
       if selected <> 10000 then TranslateSingleToMulti;

       //Мультивыбор////////////////////////////////////////////////
       if (MultiSelect) then
       begin
          for count := 0 to High(multy_selected_line) do
          begin
             Lines[multy_selected_line[count]].x1+=delta_x;
             Lines[multy_selected_line[count]].x2+=delta_x;
             Lines[multy_selected_line[count]].y1+=delta_y;
             Lines[multy_selected_line[count]].y2+=delta_y;
          end;
          for count := 0 to High(multy_selected_circle) do
          begin
             circles[multy_selected_circle[count]].x+=delta_x;
             circles[multy_selected_circle[count]].y+=delta_y;
          end;
          for count := 0 to High(multy_selected_arcs) do
          begin
             arcs[multy_selected_arcs[count]].x1+=delta_x;
             arcs[multy_selected_arcs[count]].y1+=delta_y;
             arcs[multy_selected_arcs[count]].x2+=delta_x;
             arcs[multy_selected_arcs[count]].y2+=delta_y;
             arcs[multy_selected_arcs[count]].l_x+=delta_x;
             arcs[multy_selected_arcs[count]].r_x+=delta_x;
             arcs[multy_selected_arcs[count]].t_y+=delta_y;
             arcs[multy_selected_arcs[count]].b_y+=delta_y;
             arcs[multy_selected_arcs[count]].CentrX += delta_x;
             arcs[multy_selected_arcs[count]].CentrY += delta_y;
          end;
          for count := 0 to High(multy_selected_point) do
          begin
             points[multy_selected_point[count]].x+=delta_x;
             points[multy_selected_point[count]].y+=delta_y;
          end;
          for count := 0 to High(multy_selected_text) do
          begin
             nadp[multy_selected_text[count]].x+=delta_x;
             nadp[multy_selected_text[count]].y+=delta_y;
          end;
          for count := 0 to High(multy_selected_arrow) do
          begin
             arrow[multy_selected_arrow[count]].x+=delta_x;
             arrow[multy_selected_arrow[count]].y+=delta_y;
          end;
          for count := 0 to High(multy_selected_dim) do
          begin
             DimAr[multy_selected_dim[count]].x1+=delta_x;
             DimAr[multy_selected_dim[count]].y1+=delta_y;
             DimAr[multy_selected_dim[count]].x2+=delta_x;
             DimAr[multy_selected_dim[count]].y2+=delta_y;
             DimAr[multy_selected_dim[count]].x3+=delta_x;
             DimAr[multy_selected_dim[count]].y3+=delta_y;
             DimAr[multy_selected_dim[count]].x4+=delta_x;
             DimAr[multy_selected_dim[count]].y4+=delta_y;
             DimAr[multy_selected_dim[count]].nadpCX+=delta_x;
             DimAr[multy_selected_dim[count]].nadpCY+=delta_y;
          end;

          SetLength(multy_selected_line,0);
          SetLength(multy_selected_circle,0);
          SetLength(multy_selected_arcs,0);
          SetLength(multy_selected_point,0);
          SetLength(multy_selected_text,0);
          SetLength(multy_selected_arrow,0);
          SetLength(multy_selected_dim,0);

          stad_copy:=false;
          select_type:=-1;
          selected:=10000;
          cut_mode:=false;

          clear;
          exit;
       end;
   end;
   Form1.StatusBar1.Panels.Items[0].Text:='Не выбран ни один объект';
end;

//Копирование объектов указанием мыши
Procedure CopyObj(x_vrem, y_vrem : double);

var
  count : integer;
  BlockSum : array of integer; //массив для накопления элементов для нового блока

begin
   SetLength(BlockSum, 0);

   near_points(x_vrem,y_vrem,4);

   if (stad_copy = false) then
   begin
        x_copy:=x_vrem; y_copy:=y_vrem;
        stad_copy:=true;
        Form1.StatusBar1.Panels.Items[0].Text:='Куда скопировать?';
   end
   else
   begin
        delta_x:=x_vrem-x_copy;
        delta_y:=y_vrem-y_copy;

     //Перевод из ед.выбора и мульти
     if selected <> 10000 then TranslateSingleToMulti;
       ///Копирование
       //Мультивыбор////////////////////////////////////////////////
     if (MultiSelect) then
     begin
        for count := 0 to High(multy_selected_line) do
        begin
           delta_x_copy:=Lines[multy_selected_line[count]].x2-Lines[multy_selected_line[count]].x1;
           delta_y_copy:=Lines[multy_selected_line[count]].y2-Lines[multy_selected_line[count]].y1;

           Lines[k].x1:=Lines[multy_selected_line[count]].x1+delta_x;
           Lines[k].x2:=Lines[multy_selected_line[count]].x2+delta_x;
           Lines[k].y1:=Lines[multy_selected_line[count]].y1+delta_y;
           Lines[k].y2:=Lines[multy_selected_line[count]].y2+delta_y;
           Lines[k].col:=Lines[multy_selected_line[count]].col;
           Lines[k].Width:=Lines[multy_selected_line[count]].width;
           Lines[k].style:=Lines[multy_selected_line[count]].style;
           Lines[k].id := id;

           //Если копируемый отрезок входит в блок
           if (Prov_Sel_Block(lines[multy_selected_line[count]].id) <> -1) then
           begin
               SetLength(BlockSum, Length(BlockSum)+1);
               BlockSum[High(BlockSum)] := id;
           end;

           id+=1;
           k+=1;
        end;

        for count := 0 to High(multy_selected_circle) do
        begin
           Circles[k_cir].x:=Circles[multy_selected_circle[count]].x+delta_x;
           Circles[k_cir].y:=Circles[multy_selected_circle[count]].y+delta_y;
           Circles[k_cir].r:=Circles[multy_selected_circle[count]].r;
           Circles[k_cir].col:=Circles[multy_selected_circle[count]].col;
           Circles[k_cir].width:=Circles[multy_selected_circle[count]].width;
           Circles[k_cir].style:=Circles[multy_selected_circle[count]].style;
           Circles[k_cir].id := id;

           //Если копируемая окружность входит в блок
           if (Prov_Sel_Block(circles[multy_selected_circle[count]].id) <> -1) then
           begin
               SetLength(BlockSum, Length(BlockSum)+1);
               BlockSum[High(BlockSum)] := id;
           end;

           id+=1;
           k_cir+=1;
        end;

        for count := 0 to High(multy_selected_arcs) do
        begin
           Arcs[k_arc].x1:=Arcs[multy_selected_arcs[count]].x1+delta_x;
           Arcs[k_arc].y1:=Arcs[multy_selected_arcs[count]].y1+delta_y;
           Arcs[k_arc].x2:=Arcs[multy_selected_arcs[count]].x2+delta_x;
           Arcs[k_arc].y2:=Arcs[multy_selected_arcs[count]].y2+delta_y;
           Arcs[k_arc].r_x:=Arcs[multy_selected_arcs[count]].r_x+delta_x;
           Arcs[k_arc].t_y:=Arcs[multy_selected_arcs[count]].t_y+delta_y;
           Arcs[k_arc].l_x:=Arcs[multy_selected_arcs[count]].l_x+delta_x;
           Arcs[k_arc].b_y:=Arcs[multy_selected_arcs[count]].b_y+delta_y;

           Arcs[k_arc].CentrX:=arcs[multy_selected_arcs[count]].CentrX + delta_x;
           Arcs[k_arc].CentrY:=arcs[multy_selected_arcs[count]].CentrY + delta_y;
           Arcs[k_arc].rad := arcs[multy_selected_arcs[count]].rad;
           Arcs[k_arc].StartAngle := arcs[multy_selected_arcs[count]].StartAngle;
           Arcs[k_arc].EndAngle := arcs[multy_selected_arcs[count]].EndAngle;

           Arcs[k_arc].col:=Arcs[multy_selected_arcs[count]].col;
           Arcs[k_arc].width:=Arcs[multy_selected_arcs[count]].width;
           Arcs[k_arc].style:=Arcs[multy_selected_arcs[count]].style;
           Arcs[k_arc].id := id;

           //Если копируемая дуга входит в блок
           if (Prov_Sel_Block(arcs[multy_selected_arcs[count]].id) <> -1) then
           begin
               SetLength(BlockSum, Length(BlockSum)+1);
               BlockSum[High(BlockSum)] := id;
           end;

           id+=1;
           k_arc+=1;
        end;

        for count := 0 to High(multy_selected_point) do
        begin
           Points[k_points].x:=Points[multy_selected_point[count]].x+delta_x;
           Points[k_points].y:=Points[multy_selected_point[count]].y+delta_y;
           Points[k_points].id := id;

           //Если копируемая точка входит в блок
           if (Prov_Sel_Block(points[multy_selected_point[count]].id) <> -1) then
           begin
               SetLength(BlockSum, Length(BlockSum)+1);
               BlockSum[High(BlockSum)] := id;
           end;

           id += 1;
           k_points+=1;
        end;

        for count := 0 to High(multy_selected_text) do
        begin
           Nadp[k_nadp].x:=Nadp[multy_selected_text[count]].x+delta_x;
           Nadp[k_nadp].y:=Nadp[multy_selected_text[count]].y+delta_y;
           Nadp[k_nadp].h:=Nadp[multy_selected_text[count]].h;
           Nadp[k_nadp].col:=Nadp[multy_selected_text[count]].col;
           Nadp[k_nadp].s:=Nadp[multy_selected_text[count]].s;
           Nadp[k_nadp].id := id;

           //Если копируемая надпись входит в блок
           if (Prov_Sel_Block(nadp[multy_selected_text[count]].id) <> -1) then
           begin
               SetLength(BlockSum, Length(BlockSum)+1);
               BlockSum[High(BlockSum)] := id;
           end;

           id +=1;
           k_nadp+=1;
        end;

        for count := 0 to High(multy_selected_arrow) do
        begin
           Arrow[k_arrow].x:=Arrow[multy_selected_arrow[count]].x+delta_x;
           Arrow[k_arrow].y:=Arrow[multy_selected_arrow[count]].y+delta_y;
           Arrow[k_arrow].t:=Arrow[multy_selected_arrow[count]].t;
           Arrow[k_arrow].col:=Arrow[multy_selected_arrow[count]].col;
           Arrow[k_arrow].id := id;

           //Если копируемая стрелка входит в блок
           if (Prov_Sel_Block(arrow[multy_selected_arrow[count]].id) <> -1) then
           begin
               SetLength(BlockSum, Length(BlockSum)+1);
               BlockSum[High(BlockSum)] := id;
           end;

           id += 1;
           k_arrow+=1;
        end;

        for count := 0 to High(multy_selected_dim) do
        begin
           SetLength(DimAr,Length(DimAr)+1);
           DimAr[High(DimAr)].x1:=DimAr[multy_selected_dim[count]].x1+delta_x;
           DimAr[High(DimAr)].y1:=DimAr[multy_selected_dim[count]].y1+delta_y;
           DimAr[High(DimAr)].x2:=DimAr[multy_selected_dim[count]].x2+delta_x;
           DimAr[High(DimAr)].y2:=DimAr[multy_selected_dim[count]].y2+delta_y;
           DimAr[High(DimAr)].x3:=DimAr[multy_selected_dim[count]].x3+delta_x;
           DimAr[High(DimAr)].y3:=DimAr[multy_selected_dim[count]].y3+delta_y;
           DimAr[High(DimAr)].x4:=DimAr[multy_selected_dim[count]].x4+delta_x;
           DimAr[High(DimAr)].y4:=DimAr[multy_selected_dim[count]].y4+delta_y;
           DimAr[High(DimAr)].nadpCX:=DimAr[multy_selected_dim[count]].nadpCX+delta_x;
           DimAr[High(DimAr)].nadpCY:=DimAr[multy_selected_dim[count]].nadpCY+delta_y;
           DimAr[High(DimAr)].NumDigit:=DimAr[multy_selected_dim[count]].NumDigit;
           DimAr[High(DimAr)].nadp:=DimAr[multy_selected_dim[count]].nadp;
           DimAr[High(DimAr)].id := id;

           //Если копируемый размер входит в блок
           if (Prov_Sel_Block(DimAr[multy_selected_dim[count]].id) <> -1) then
           begin
               SetLength(BlockSum, Length(BlockSum)+1);
               BlockSum[High(BlockSum)] := id;
           end;

           id += 1;
        end;

        clear;
     end;
   end;

   //Если среди выбр.элементов был блок, то создать новый блок
   if (length(BlockSum) > 0) then
      MakeBlock(BlockSum);
end;

//Удлинить или укоротить линию по другой линии
procedure Trim();

var
  CrossPoint : MyTPoint;
  dist1, dist2 : double; //расстояние от конца линии до точки пересечения

begin
   CrossPoint := PointCross2Line(multy_selected_line[0],
                                 multy_selected_line[1], true);

   if (CrossPoint.x >= min(lines[multy_selected_line[1]].x1,lines[multy_selected_line[1]].x2)) and
      (CrossPoint.x <= max(lines[multy_selected_line[1]].x1,lines[multy_selected_line[1]].x2)) and
      (CrossPoint.y >= min(lines[multy_selected_line[1]].y1,lines[multy_selected_line[1]].y2)) and
      (CrossPoint.y <= max(lines[multy_selected_line[1]].y1,lines[multy_selected_line[1]].y2)) then
   begin
      save_prev(); //Сохранить предыдущее состояние

      dist1 := sqrt(sqr(Lines[multy_selected_line[0]].x1-CrossPoint.x)+
                    sqr(Lines[multy_selected_line[0]].y1-CrossPoint.y));
      dist2 := sqrt(sqr(Lines[multy_selected_line[0]].x2-CrossPoint.x)+
                    sqr(Lines[multy_selected_line[0]].y2-CrossPoint.y));

      if dist1 <= dist2 then
      begin
          Lines[multy_selected_line[0]].x1 := CrossPoint.x;
          Lines[multy_selected_line[0]].y1 := CrossPoint.y;
      end
      else
      begin
          Lines[multy_selected_line[0]].x2 := CrossPoint.x;
          Lines[multy_selected_line[0]].y2 := CrossPoint.y;
      end
   end
   else
   ShowMessage('Точка пересечения находится за пределами отрезка');
   Reset_mode;
   Clear();
end;

//Изменить длину отрезка
procedure LetChangeLength;

var
  VremAngle : double;
  oldLength : double;
  newLength : double;

begin
  VremAngle := CalcAngle(lines[Selected].x1,lines[Selected].y1,
                         lines[Selected].x2,lines[Selected].y2)+DegToRad(90);
  oldLength := sqrt(sqr(lines[Selected].x1-lines[Selected].x2)+
                    sqr(lines[Selected].y1-lines[Selected].y2));

  newLength := StrToFloat(InputBox('Отрезок','Укажи новую длину отрезка', FloatToStr(oldLength)));

  if (newLength <> 0) and
     (newLength <> oldLength) then
     begin
         save_prev(); //Сохранить предыдущее состояние

         lines[Selected].x2 := lines[Selected].x1 + newLength * sin(vremAngle);
         lines[Selected].y2 := lines[Selected].y1 + newLength * cos(vremAngle);
     end;
  clear;
end;


end.
