unit SpecFunction;

{
    Vector graphical editor.
    Version: 10.
    Written on FreePascal (https://freepascal.org/).
    Copyright (C) 2024  Warfalomay

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
}

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, primitives, Drawing_unit;

//Отключить все режимы
Procedure Reset_mode;
//Прервать текущую операцию
procedure BreakOperation;
//Занесение единичного объекта в массив мультивыделения
procedure TranslateSingleToMulti;
//Проверка принадлежности точки какому-либо размеру за искл.указанного
function PointInDim(x,y : double) : boolean;
//Проверка принадлежности точки какому-либо отрезку (чтобы не выбирать отрезок за точку)
function PointInLine(x,y : double) : boolean;
//Преобразование координат (double) в точку
function myPoint(x,y : double) : MyTPoint;
//Проверка содержимого массивов мультивыбора
function MultiSelect : boolean;


implementation

uses
  Unit1, View, my_modify;

//Проверка содержимого массивов мультивыбора
function MultiSelect : boolean;
begin
    result := true;
    if (length(multy_selected_line) = 0) and
       (length(multy_selected_circle) = 0) and
       (length(multy_selected_point) = 0) and
       (length(multy_selected_arcs) = 0) and
       (length(multy_selected_arrow) = 0) and
       (length(multy_selected_text) = 0) and
       (length(multy_selected_dim) = 0) then
       result := false;
end;

//Преобразование координат (double) в точку
function myPoint(x,y : double) : MyTPoint;
begin
    result.x := x;
    result.y := y;
end;

//Отключить все режимы
procedure Reset_mode;
begin
     selected := 10000;

     select_mode := false; //Режим выделения по умолчанию
     multy_select_mode := false; //Режим множеств. выделения
     FrameSelect_mode := false; //Режим выбора рамкой
     SelAllMode := false; //Режим выделить все

     stad := false;
     stad_cir := false;
     stad_copy := false;
     stad_dim := false;

     if (draw_line_mode) and
        (stad) then
        begin
             lines[k].x1:=0;
             lines[k].y1:=0;
        end;

     draw_line_mode := false;
     draw_line_coord_mode := false;
     draw_rectangle_mode := false;
     Draw_MultiAngleMode := false;//Описанный многоугольник
     Draw_MultiAngleOutMode := false;//Описанный многоугольник

     draw_circle_mode := false;
     draw_circle2point_mode := false;
     draw_circle3points_mode := false; //Окружность, вписанная в треугольник
     draw_arc_mode := false;
     draw_points_mode := false;
     draw_arrow_mode := false; //Режим рисования стрелок
     drawDimensionMode := false;//Режим добавления размеров

     text_input_mode := false;

     get_dim_mode := false;//Режим измерения
     get_area_mode := false;
     get_angle_mode := false; //Режим измерения углов
     get_centr_mass_mode := false; //Режим нахождения центра тяжести
     get_perimetr_mode := false;   //Режим нахождения периметра

     RotMode := false;//Режим поворота объектов

     MirMode := false;//Режим зеркального отражения

     RoundMode := false;//Режим скругления 2 отрезков

     Hor_napr_mode := false;   //Горизонтальный луч
     Vert_napr_mode := false; //Вертикальный луч
     Angle_napr_mode := false; //Луч под углом
     Two_point_napr_mode := false; //Прямая через 2 точки
     Par_shift_mode := false;//Параллельная прямая

     drag_point_mode := false;//Режим перемещения за ключевые точки

     OrtoDim := false; //Режим ортогональных размеров

     last_point.X := -1;   //Точка для непрерывного ввода линии
     last_point.Y := -1;

     point1.X := -1;   //Точка для рисования дуги
     point1.Y := -1;

     Form1.StatusBar1.Panels.Items[0].Text:='Ничего не выбрано';
     Form1.StatusBar1.Panels.Items[3].Text:='S=';

     setLength(multy_selected_line,0);
     setLength(multy_selected_circle,0);
     setLength(multy_selected_arcs,0);
     setLength(multy_selected_point,0);
     setLength(multy_selected_text,0);
     setLength(multy_selected_arrow,0);
     setLength(multy_selected_dim,0);

     SetLength(Two_point_ar,0);
     SetLength(Free_point_ar,0);
end;

//Прервать текущую операцию
procedure BreakOperation;
begin
   if (draw_line_mode) and
         (last_point.X <> -1) and
         (last_point.Y <> -1) then
         begin
            last_point.X := -1;
            last_point.Y := -1;
            Form1.StatusBar1.Panels.Items[0].Text:='Укажи первую точку';
            Clear;
            exit;
         end;

      if (draw_line_mode) and
        (stad) then
        begin
             lines[k].x1:=0;
             lines[k].y1:=0;
        end;

      if (draw_circle_mode) and
         (stad_cir) then
        begin
            circles[k_cir].x := -1;
            circles[k_cir].y := -1;
        end;

     copy_mode := false;
     cut_mode := false;
     drag_point_mode := false;

      selected := 10000;
      block_num := -1;
      setLength(multy_selected_line,0);
      setLength(multy_selected_circle,0);
      setLength(multy_selected_arcs,0);
      setLength(multy_selected_point,0);
      setLength(multy_selected_text,0);
      setLength(multy_selected_arrow,0);

      Form1.StatusBar1.Panels.Items[0].Text:='Ничего не выбрано';
      Form1.StatusBar1.Panels.Items[3].Text:='S=';

      if get_centr_mass_mode then
        begin
          Form1.Show_pset.Checked := vremShowPset;
          P_show := vremShowPset;
        end;

      Clear;

      if not (select_mode) or (selected <> 10000) then
      Reset_mode;
end;

//Занесение единичного объекта в массив мультивыделения
procedure TranslateSingleToMulti;
begin
    if selected = 10000 then exit;

    SetLength(multy_selected_line,0);
    SetLength(multy_selected_circle,0);
    SetLength(multy_selected_point,0);
    SetLength(multy_selected_text,0);
    SetLength(multy_selected_arrow,0);
    SetLength(multy_selected_arcs,0);
    SetLength(multy_selected_dim,0);

    Case select_type of
    line:   begin
              SetLength(multy_selected_line,1);
              multy_selected_line[0] := Selected;
            end;
    circle: begin
              SetLength(multy_selected_circle,1);
              multy_selected_circle[0] := Selected;
            end;
    dot:    begin
              SetLength(multy_selected_point,1);
              multy_selected_point[0] := Selected;
            end;
    txt:    begin
              SetLength(multy_selected_text,1);
              multy_selected_text[0] := Selected;
            end;
    arr:    begin
              SetLength(multy_selected_arrow,1);
              multy_selected_arrow[0] := Selected;
            end;
    arc:    begin
              SetLength(multy_selected_arcs,1);
              multy_selected_arcs[0] := Selected;
            end;
    dim:    begin
              SetLength(multy_selected_dim,1);
              multy_selected_dim[0] := Selected;
            end;
    end;
    Selected := 10000;
end;

//Проверка принадлежности точки какому-либо размеру за искл.указанного
function PointInDim(x,y : double) : boolean;

var
  count : integer;

begin
    result := false;

    for count := 0 to High(DimAr) do
    begin
        if (((x = DimAr[count].x1) and (y = DimAr[count].y1)) or
           ((x = DimAr[count].x2) and (y = DimAr[count].y2)) or
           ((x = DimAr[count].x3) and (y = DimAr[count].y3)) or
           ((x = DimAr[count].x4) and (y = DimAr[count].y4))) and
           (count <> Drag_selected) then
           begin
               result := true;
               exit;
           end;
    end;
end;

//Проверка принадлежности точки какому-либо отрезку
function PointInLine(x,y : double) : boolean;

var
  count : integer;

begin
    result := false;

    for count := 0 to High(Lines) do
    begin
        if ((x = Lines[count].x1) and (y = Lines[count].y1)) or
           ((x = Lines[count].x2) and (y = Lines[count].y2)) then
           begin
               result := true;
               exit;
           end;
    end;
end;

end.

