unit View;

{
    Vector graphical editor.
    Version: 10.
    Written on FreePascal (https://freepascal.org/).
    Copyright (C) 2024  Warfalomay

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
}

//Функции построения изображения

{$mode ObjFPC}{$H+}

interface
uses Graphics;

//Перерисовать экран
procedure Clear();
//Подготовка изображения для сохранения в растр
procedure prepare_pic (pic : TPortableNetworkGraphic);
//Отрисовка нового места расположения объектов при копировании
//или перетаскивании
procedure DrawMoved(ShiftX,ShiftY : double);
//Отрисовка рамки при выборе рамкой
procedure DrawFrame(x,y : integer);
//Отрисовка заготовки прямоугольника при указании размеров мышью
procedure PrepareDrawRectangle(x_rect, y_rect, x_vrem, y_vrem : double;
                               x,y : integer);
//Отрисовка заготовки отрезка при указании КОНЕЧНОЙ точки мышью
Procedure PrepareDrawLine(x_vrem, y_vrem : double; x,y : integer);
//Отрисовка заготовки отрезка при указании СЛЕДУЮЩЕЙ точки мышью
Procedure PrepareDrawNextLine(x_vrem, y_vrem : double; x,y : integer);
//Отрисовка заготовки окружности при построении по 2 ординатам
Procedure PrepareDrawCirc2Ord(x_vrem, y_vrem : double; x,y : integer);
//Отрисовка заготовки окружности при построении по центру и радиусу
Procedure PrepareDrawCircCentrRad(x_vrem, y_vrem : double; x,y : integer);
//Отрисовка шаблона объектов при перетаскивании за точку
Procedure PrepareDrawDotMoved(x_vrem, y_vrem : double; x,y : integer);
//Отрисовка шаблона размера при перетаскивании за точку
Procedure PrepareDrawDotMoved(x_vrem, y_vrem : double; movedPoint : byte);
//Установка курсора мыши
Procedure SetMouseCursor(x_vrem, y_vrem : double);
//Отрисовка линии симметрии для зеркального отражения
Procedure DrawMirLine(x_vrem, y_vrem : double);

implementation
uses
  Unit1,Classes, SysUtils, dialogs, primitives, math, Drawing_unit,
  Controls, my_modify, calc;

//Отрисовка линии симметрии для зеркального отражения
Procedure DrawMirLine(x_vrem, y_vrem : double);

var
  VremColor : TColor;

begin
    clear;

    VremColor := Cvet;
    Form1.WS.Canvas.Pen.Color := clBlue;

    If orto_graph then
    begin
        if abs(round(Point1.x)-round(x_vrem)) >=
           abs(round(Point1.y)-round(y_vrem)) then
              Form1.WS.Canvas.Line(round(Point1.x),round(Point1.y),
                                   round(x_vrem), round(Point1.y))
        else
              Form1.WS.Canvas.Line(round(Point1.x),round(Point1.y),
                                   round(Point1.x), round(y_vrem));
    end
    else
        Form1.WS.Canvas.Line(round(Point1.x),round(Point1.y),
                             round(x_vrem), round(y_vrem));

    Form1.WS.Canvas.Pen.Color := VremColor;
end;

procedure Clear();

var
  i:integer;
  vrem : TBitmap;
  TextWidth : byte;
  TextHeight : byte;
  TextText : string;

begin
  Vrem := TBitmap.Create();
  Vrem.Width := World_size_h;
  Vrem.Height := World_size_v;

  Vrem.Canvas.Refresh;

  Vrem.Canvas.Brush.Color:=clWhite;
  Vrem.Canvas.FillRect(Rect(0,0,Vrem.Width,Vrem.Height));
  Vrem.Canvas.Brush.Style:=bsClear;

  if (podl=true) and (show_back = true)  //Показывать подложку
                     then Vrem.Canvas.draw(0,0,back_png);

  Form1.WS.Canvas.Pen.Color:=clRed;

  for i:=0 to Arr_length-1 do
  begin
      with Lines[i] do
      begin
         Vrem.Canvas.Pen.Style:=Lines[i].style;
         Vrem.Canvas.Pen.Width:=Lines[i].Width;
         Vrem.Canvas.Pen.Color:=Lines[i].col;
         Vrem.Canvas.Line(round(x1),round(y1),
                          round(x2),round(y2));
         //Отрисовка концов линии, чтобы контур был цельным
         //(ошибки рисования на канве)
         if (get_centr_mass_mode) then
         begin
             Vrem.Canvas.EllipseC(round(x1),round(y1),1,1);
             Vrem.Canvas.EllipseC(round(x2),round(y2),1,1);
         end;
      end;

      with Circles[i] do
      begin
          if (Circles[i].x > 0) and
             (Circles[i].y > 0) then
          begin
              Vrem.Canvas.Pen.Style:=Circles[i].style;
              Vrem.Canvas.Pen.Width:=Circles[i].Width;
              Vrem.Canvas.Pen.Color:=Circles[i].col;
              Vrem.Canvas.Ellipse(Round(Circles[i].x-Circles[i].r),
                                  Round(Circles[i].y-Circles[i].r),
                                  Round(Circles[i].x+Circles[i].r),
                                  Round(Circles[i].y+Circles[i].r));

              Vrem.Canvas.Pen.Style:=psSolid;
              Vrem.Canvas.Pen.Width:=1;
              Vrem.Canvas.Pen.Color:=clGray;
          end;

          If not (drag_point_mode) and
                 (Circles[i].r>0) then
          begin
              If P_show then
              begin
                  //Центр
                  Vrem.Canvas.Ellipse(Round(Circles[i].x)-2, Round(Circles[i].y)-2,
                                      Round(Circles[i].x)+2, Round(Circles[i].y)+2);
                  //Ординаты
                  Vrem.Canvas.Ellipse(Round(Circles[i].x-Circles[i].r)-2, Round(Circles[i].y)-2,
                                      Round(Circles[i].x-Circles[i].r)+2, Round(Circles[i].y)+2);
                  Vrem.Canvas.Ellipse(Round(Circles[i].x+Circles[i].r)-2, Round(Circles[i].y)-2,
                                      Round(Circles[i].x+Circles[i].r)+2, Round(Circles[i].y)+2);
                  Vrem.Canvas.Ellipse(Round(Circles[i].x)-2, Round(Circles[i].y-Circles[i].r)-2,
                                      Round(Circles[i].x)+2, Round(Circles[i].y-Circles[i].r)+2);
                  Vrem.Canvas.Ellipse(Round(Circles[i].x)-2, Round(Circles[i].y+Circles[i].r)-2,
                                      Round(Circles[i].x)+2, Round(Circles[i].y+Circles[i].r)+2);
              end;
          end;
      end;

      with arcs[i] do
      begin
          If (arcs[i].l_x <> 0) and
             (arcs[i].r_x <> 0) and
             (arcs[i].b_y <> 0) and
             (arcs[i].t_y <> 0) then
          begin
              Vrem.Canvas.Pen.Style:=arcs[i].style;
              Vrem.Canvas.Pen.Width:=arcs[i].Width;
              Vrem.Canvas.Pen.Color:=arcs[i].col;
              Vrem.canvas.Arc(Round(Arcs[i].l_x),Round(Arcs[i].t_y),
                              Round(Arcs[i].r_x),Round(Arcs[i].b_y),
                              Round(Arcs[i].x1),Round(Arcs[i].y1),
                              Round(Arcs[i].x2),Round(Arcs[i].y2));

              //Отрисовка концов линии, чтобы контур был цельным
              //(ошибки рисования на канве)
              if (get_centr_mass_mode) then
              begin
                  Vrem.Canvas.EllipseC(Round(Arcs[i].x1),Round(Arcs[i].y1),2,2);
                  Vrem.Canvas.EllipseC(Round(Arcs[i].x2),Round(Arcs[i].y2),2,2);
              end;
          end;
      end;

      If P_Show then    //Показывать точки (вкл.по умолчанию)
      begin
          with points[i] do
          begin
               If (points[i].x<>0) and (points[i].y<>0) then
               begin
                   Vrem.Canvas.Pen.Color:=clGreen;
                   Vrem.Canvas.Pen.Width:=1;
                   Vrem.Canvas.Ellipse(Round(points[i].x)-5,
                                       Round(points[i].y)+5,
                                       Round(points[i].x)+5,
                                       Round(points[i].y)-5);
               end;
          end;
      end;

      with nadp[i] do
      begin
           Vrem.Canvas.Font.Size:=Round(nadp[i].h);
           Vrem.Canvas.Font.Color:= nadp[i].col;
           Vrem.Canvas.Pen.Width:=1;
           Vrem.Canvas.TextOut(Round(nadp[i].x),Round(nadp[i].y),
                               nadp[i].s);
      end;

      with arrow[i] do
      begin
           Vrem.Canvas.Pen.Color:=arrow[i].col;
           Vrem.Canvas.Pen.Width:=1;
           If arrow[i].t='left' then
           begin
               Vrem.Canvas.Line(Round(x),Round(y),Round(x+30),Round(y));
               Vrem.Canvas.Line(Round(x),Round(y),Round(x+10),Round(y+5));
               Vrem.Canvas.Line(Round(x),Round(y),Round(x+10),Round(y-5));
           end;
           If arrow[i].t='right' then
           begin
               Vrem.Canvas.Line(Round(x),Round(y),Round(x-30),Round(y));
               Vrem.Canvas.Line(Round(x),Round(y),Round(x-10),Round(y+5));
               Vrem.Canvas.Line(Round(x),Round(y),Round(x-10),Round(y-5));
           end;
           If arrow[i].t='up' then
           begin
               Vrem.Canvas.Line(Round(x),Round(y),Round(x),Round(y+30));
               Vrem.Canvas.Line(Round(x),Round(y),Round(x+5),Round(y+10));
               Vrem.Canvas.Line(Round(x),Round(y),Round(x-5),Round(y+10));
           end;
           If arrow[i].t='down' then
           begin
               Vrem.Canvas.Line(Round(x),Round(y),Round(x),Round(y-30));
               Vrem.Canvas.Line(Round(x),Round(y),Round(x+5),Round(y-10));
               Vrem.Canvas.Line(Round(x),Round(y),Round(x-5),Round(y-10));
           end;
      end;
  end;

  Vrem.Canvas.Pen.Color := clBlue;

  //Отрисовка размеров
  for i := 0 to High(DimAr) do
  begin
      //Vrem.Canvas.Ellipse(Round(DimAr[i].x1-5), Round(DimAr[i].y1-5),
      //                        Round(DimAr[i].x1+5), Round(DimAr[i].y1+5));
      //Vrem.Canvas.Ellipse(Round(DimAr[i].x2-5), Round(DimAr[i].y2-5),
      //                        Round(DimAr[i].x2+5), Round(DimAr[i].y2+5));


      Vrem.Canvas.Ellipse(Round(DimAr[i].x3-5), Round(DimAr[i].y3-5),
                              Round(DimAr[i].x3+5), Round(DimAr[i].y3+5));
      Vrem.Canvas.Ellipse(Round(DimAr[i].x4-5), Round(DimAr[i].y4-5),
                              Round(DimAr[i].x4+5), Round(DimAr[i].y4+5));
      //Размерная линия
      Vrem.Canvas.Line(Round(DimAr[i].x3),Round(DimAr[i].y3),
                       Round(DimAr[i].x4),Round(DimAr[i].y4));
      //Выносные линии
      Vrem.Canvas.Line(Round(DimAr[i].x1),Round(DimAr[i].y1),
                       Round(DimAr[i].x3),Round(DimAr[i].y3));
      Vrem.Canvas.Line(Round(DimAr[i].x2),Round(DimAr[i].y2),
                       Round(DimAr[i].x4),Round(DimAr[i].y4));

      //Вывод текста размера
      TextText := FloatToStr(SimpleRoundTo(DimAr[i].nadp,-DimAr[i].NumDigit));
      TextWidth := Vrem.Canvas.TextWidth(TextText);
      TextHeight := Vrem.Canvas.TextHeight(TextText);
      Vrem.Canvas.Brush.Style := bsSolid;
      Vrem.Canvas.Font.Color:= clBlue;
      Vrem.Canvas.TextOut(Round(DimAr[i].nadpCX - TextWidth/2), Round(DimAr[i].nadpCY-TextHeight / 2),
                          TextText);
  end;

  if block_num = -1 then
     Form1.WS.Canvas.Pen.Color := Cvet;

  Form1.WS.Canvas.Draw(0+Screen_x,0+Screen_y,vrem);
  vrem.Free;

  with Form1 do
   imgTemp.Canvas.CopyRect(Rect(0,0,WS.Width, WS.Height), WS.Canvas, Rect(0,0,WS.Width, WS.Height));
end;

procedure prepare_pic (pic : TPortableNetworkGraphic);

var
  i : integer;

begin
    pic.Canvas.Brush.Color:=clWhite;
    pic.Canvas.FillRect(Rect(0,0,save_png.Width,save_png.Height));
    pic.Canvas.Brush.Style:=bsClear;

    if (podl=true) and (show_back = true)  //Показывать подложку
                           then pic.Canvas.draw(0,0,back_png);

    for i:=0 to Arr_length-1 do
        begin
          with Lines[i] do
          begin
               pic.Canvas.Pen.Style:=Lines[i].style;
               pic.Canvas.Pen.Width:=Lines[i].Width;
               pic.Canvas.Pen.Color:=Lines[i].col;
               pic.Canvas.Line(Round(x1),Round(y1),
                                    Round(x2),Round(y2));
          end;

          with Circles[i] do
          begin
              pic.Canvas.Pen.Style:=Circles[i].style;
              pic.Canvas.Pen.Width:=Circles[i].Width;
              pic.Canvas.Pen.Color:=Circles[i].col;
              pic.Canvas.Ellipse(Round(Circles[i].x-Circles[i].r),
                                 Round(Circles[i].y-Circles[i].r),
                                 Round(Circles[i].x+Circles[i].r),
                                 Round(Circles[i].y+Circles[i].r));
          end;

          with arcs[i] do
          begin
              pic.Canvas.Pen.Style:=arcs[i].style;
              pic.Canvas.Pen.Width:=arcs[i].Width;
              pic.Canvas.Pen.Color:=arcs[i].col;
              pic.canvas.Arc(Round(Arcs[i].l_x),Round(Arcs[i].t_y),
                             Round(Arcs[i].r_x),Round(Arcs[i].b_y),
                             Round(Arcs[i].x1),Round(Arcs[i].y1),
                             Round(Arcs[i].x2),Round(Arcs[i].y2));
          end;

          If P_Show then    //Показывать точки (вкл.по умолчанию)
          begin
              with points[i] do
              begin
                   If (points[i].x<>0) and (points[i].y<>0) then
                   begin
                       pic.Canvas.Pen.Color:=clGreen;
                       pic.Canvas.Pen.Width:=1;
                       pic.Canvas.Ellipse(Round(points[i].x)-5,
                                          Round(points[i].y)+5,
                                          Round(points[i].x)+5,
                                          Round(points[i].y)-5);
                   end;
              end;
          end;

          with nadp[i] do
          begin
               pic.Canvas.Font.Size:=Round(nadp[i].h);
               pic.Canvas.Font.Color:= nadp[i].col;
               pic.Canvas.Pen.Width:=1;
               pic.Canvas.TextOut(Round(nadp[i].x),Round(nadp[i].y),
                                       nadp[i].s);
          end;


          with arrow[i] do
          begin
               pic.Canvas.Pen.Color:=arrow[i].col;
               pic.Canvas.Pen.Width:=1;
               If arrow[i].t='left' then
               begin
                   pic.Canvas.Line(Round(x),Round(y),Round(x+30),Round(y));
                   pic.Canvas.Line(Round(x),Round(y),Round(x+10),Round(y+5));
                   pic.Canvas.Line(Round(x),Round(y),Round(x+10),Round(y-5));
               end;
               If arrow[i].t='right' then
               begin
                   pic.Canvas.Line(Round(x),Round(y),Round(x-30),Round(y));
                   pic.Canvas.Line(Round(x),Round(y),Round(x-10),Round(y+5));
                   pic.Canvas.Line(Round(x),Round(y),Round(x-10),Round(y-5));
               end;
               If arrow[i].t='up' then
               begin
                   pic.Canvas.Line(Round(x),Round(y),Round(x),Round(y+30));
                   pic.Canvas.Line(Round(x),Round(y),Round(x+5),Round(y+10));
                   pic.Canvas.Line(Round(x),Round(y),Round(x-5),Round(y+10));
               end;
               If arrow[i].t='down' then
               begin
                   pic.Canvas.Line(Round(x),Round(y),Round(x),Round(y-30));
                   pic.Canvas.Line(Round(x),Round(y),Round(x+5),Round(y-10));
                   pic.Canvas.Line(Round(x),Round(y),Round(x-5),Round(y-10));
               end;

          end;
        end;
end;

procedure DrawMoved(ShiftX,ShiftY : double);

var
  count : integer;

begin
    With Form1 do
    begin
        if selected<>10000 then
        begin
            case select_type of
            line :   begin
                         SetLength(multy_selected_line,1);
                         multy_selected_line[0] := selected;
                     end;
            circle : begin
                         SetLength(multy_selected_circle,1);
                         multy_selected_circle[0] := selected;
                   end;
            dot    : begin
                         SetLength(multy_selected_point,1);
                         multy_selected_point[0] := selected;
                     end;
            txt    : begin
                         SetLength(multy_selected_text,1);
                         multy_selected_text[0] := selected;
                     end;
            arr    : begin
                         SetLength(multy_selected_arrow,1);
                         multy_selected_arrow[0] := selected;
                     end;
            arc    : begin
                         SetLength(multy_selected_arcs,1);
                         multy_selected_arcs[0] := selected;
                     end;
            dim    : begin
                         SetLength(multy_selected_dim,1);
                         multy_selected_dim[0] := selected;
                     end;
            end;
        end;

        for count := 0 to High(multy_selected_line) do
        begin
           WS.Canvas.Line(Round(Lines[multy_selected_line[count]].x1 + ShiftX)+Screen_X,
                          Round(Lines[multy_selected_line[count]].y1 + ShiftY)+Screen_Y,
                          Round(Lines[multy_selected_line[count]].x2 + ShiftX)+Screen_X,
                          Round(Lines[multy_selected_line[count]].y2 + ShiftY)+Screen_Y);
        end;
        for count := 0 to High(multy_selected_circle) do
        begin
           WS.Canvas.Ellipse(Round(circles[multy_selected_circle[count]].x-
                             circles[multy_selected_circle[count]].r + ShiftX)+Screen_X,
                             Round(circles[multy_selected_circle[count]].y-
                             circles[multy_selected_circle[count]].r + ShiftY)+Screen_Y,
                             Round(circles[multy_selected_circle[count]].x+
                             circles[multy_selected_circle[count]].r + ShiftX)+Screen_X,
                             Round(circles[multy_selected_circle[count]].y+
                             circles[multy_selected_circle[count]].r + ShiftY)+Screen_Y);
        end;
        for count := 0 to High(multy_selected_arcs) do
        begin
           WS.Canvas.Arc(Round(Arcs[multy_selected_arcs[count]].l_x+ShiftX)+Screen_X,
                         Round(Arcs[multy_selected_arcs[count]].t_y+ShiftY)+Screen_Y,
                         Round(Arcs[multy_selected_arcs[count]].r_x+ShiftX)+Screen_X,
                         Round(Arcs[multy_selected_arcs[count]].b_y+ShiftY)+Screen_Y,
                         Round(Arcs[multy_selected_arcs[count]].x1+ShiftX)+Screen_X,
                         Round(Arcs[multy_selected_arcs[count]].y1+ShiftY)+Screen_Y,
                         Round(Arcs[multy_selected_arcs[count]].x2+ShiftX)+Screen_X,
                         Round(Arcs[multy_selected_arcs[count]].y2+ShiftY)+Screen_Y);
        end;
        for count := 0 to High(multy_selected_point) do
        begin
           WS.Canvas.Ellipse(Round(points[multy_selected_point[count]].x-5+ShiftX)+Screen_X,
                             Round(points[multy_selected_point[count]].y+5+ShiftY)+Screen_Y,
                             Round(points[multy_selected_point[count]].x+5+ShiftX)+Screen_X,
                             Round(points[multy_selected_point[count]].y-5+ShiftY)+Screen_Y);
        end;
        for count := 0 to High(multy_selected_text) do
        begin
           WS.Canvas.Font.Size:=Round(nadp[count].h);
           WS.Canvas.TextOut(Round(nadp[multy_selected_text[count]].x+ShiftX)+Screen_X,
                             Round(nadp[multy_selected_text[count]].y+ShiftY)+Screen_Y,
                             nadp[multy_selected_text[count]].s);
        end;
        for count := 0 to High(multy_selected_arrow) do
        begin
           If arrow[multy_selected_arrow[count]].t='left' then
           begin
               WS.Canvas.Line(Round(arrow[multy_selected_arrow[count]].x+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+ShiftY)+Screen_Y,
                              Round(arrow[multy_selected_arrow[count]].x+30+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+ShiftY)+Screen_Y);
               WS.Canvas.Line(Round(arrow[multy_selected_arrow[count]].x+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+ShiftY)+Screen_Y,
                              Round(arrow[multy_selected_arrow[count]].x+10+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+5+ShiftY)+Screen_Y);
               WS.Canvas.Line(Round(arrow[multy_selected_arrow[count]].x+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+ShiftY)+Screen_Y,
                              Round(arrow[multy_selected_arrow[count]].x+10+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y-5+ShiftY)+Screen_Y);
           end;
           If arrow[multy_selected_arrow[count]].t='right' then
           begin
               WS.Canvas.Line(Round(arrow[multy_selected_arrow[count]].x+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+ShiftY)+Screen_Y,
                              Round(arrow[multy_selected_arrow[count]].x+ShiftX-30)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+ShiftY)+Screen_Y);
               WS.Canvas.Line(Round(arrow[multy_selected_arrow[count]].x+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+ShiftY)+Screen_Y,
                              Round(arrow[multy_selected_arrow[count]].x+ShiftX-10)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+5+ShiftY)+Screen_Y);
               WS.Canvas.Line(Round(arrow[multy_selected_arrow[count]].x+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+ShiftY),
                              Round(arrow[multy_selected_arrow[count]].x+ShiftX-10)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y-5+ShiftY)+Screen_Y);
           end;
           If arrow[multy_selected_arrow[count]].t='up' then
           begin
               WS.Canvas.Line(Round(arrow[multy_selected_arrow[count]].x+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+ShiftY)+Screen_Y,
                              Round(arrow[multy_selected_arrow[count]].x+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+30+ShiftY)+Screen_Y);
               WS.Canvas.Line(Round(arrow[multy_selected_arrow[count]].x+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+ShiftY)+Screen_Y,
                              Round(arrow[multy_selected_arrow[count]].x+5+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+10+ShiftY)+Screen_Y);
               WS.Canvas.Line(Round(arrow[multy_selected_arrow[count]].x+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+ShiftY)+Screen_Y,
                              Round(arrow[multy_selected_arrow[count]].x-5+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+10+ShiftY)+Screen_Y);
           end;
           If arrow[multy_selected_arrow[count]].t='down' then
           begin
               WS.Canvas.Line(Round(arrow[multy_selected_arrow[count]].x+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+ShiftY)+Screen_Y,
                              Round(arrow[multy_selected_arrow[count]].x+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y-30+ShiftY)+Screen_Y);
               WS.Canvas.Line(Round(arrow[multy_selected_arrow[count]].x+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+ShiftY)+Screen_Y,
                              Round(arrow[multy_selected_arrow[count]].x+5+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y-10+ShiftY)+Screen_Y);
               WS.Canvas.Line(Round(arrow[multy_selected_arrow[count]].x+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y+ShiftY)+Screen_Y,
                              Round(arrow[multy_selected_arrow[count]].x-5+ShiftX)+Screen_X,
                              Round(arrow[multy_selected_arrow[count]].y-10+ShiftY)+Screen_Y);
           end;
        end;
        for count := 0 to High(multy_selected_dim) do
        begin
           WS.Canvas.Ellipse(Round(DimAr[multy_selected_dim[count]].x3-5 + ShiftX + Screen_X),
                             Round(DimAr[multy_selected_dim[count]].y3-5 + ShiftY + Screen_Y),
                             Round(DimAr[multy_selected_dim[count]].x3+5 + ShiftX + Screen_X),
                             Round(DimAr[multy_selected_dim[count]].y3+5 + ShiftY + Screen_Y));
           WS.Canvas.Ellipse(Round(DimAr[multy_selected_dim[count]].x4-5 + ShiftX + Screen_X),
                             Round(DimAr[multy_selected_dim[count]].y4-5 + ShiftY + Screen_Y),
                             Round(DimAr[multy_selected_dim[count]].x4+5 + ShiftX + Screen_X),
                             Round(DimAr[multy_selected_dim[count]].y4+5 + ShiftY + Screen_Y));
           //Размерная линия
           WS.Canvas.Line(Round(DimAr[multy_selected_dim[count]].x3 + ShiftX + Screen_X),
                          Round(DimAr[multy_selected_dim[count]].y3 + ShiftY + Screen_Y),
                          Round(DimAr[multy_selected_dim[count]].x4 + ShiftX + Screen_X),
                          Round(DimAr[multy_selected_dim[count]].y4 + ShiftY + Screen_Y));
           //Выносные линии
           WS.Canvas.Line(Round(DimAr[multy_selected_dim[count]].x1 + ShiftX + Screen_X),
                          Round(DimAr[multy_selected_dim[count]].y1 + ShiftY + Screen_Y),
                          Round(DimAr[multy_selected_dim[count]].x3 + ShiftX + Screen_X),
                          Round(DimAr[multy_selected_dim[count]].y3 + ShiftY + Screen_Y));
           WS.Canvas.Line(Round(DimAr[multy_selected_dim[count]].x2 + ShiftX + Screen_X),
                          Round(DimAr[multy_selected_dim[count]].y2 + ShiftY + Screen_Y),
                          Round(DimAr[multy_selected_dim[count]].x4 + ShiftX + Screen_X),
                          Round(DimAr[multy_selected_dim[count]].y4 + ShiftY + Screen_Y));

           //Вывод текста размера
           //TextText := FloatToStr(SimpleRoundTo(DimAr[count].nadp,-DimAr[count].NumDigit));
           //TextWidth := Vrem.Canvas.TextWidth(TextText);
           //TextHeight := Vrem.Canvas.TextHeight(TextText);
           //WS.Canvas.Brush.Style := bsSolid;
           //WS.Canvas.Font.Color:= clBlue;
           //WS.Canvas.TextOut(Round(DimAr[i].nadpCX - TextWidth/2), Round(DimAr[i].nadpCY-TextHeight / 2),
           //                    TextText);
        end;
    end;
end;

//Отрисовка рамки при выборе рамкой
procedure DrawFrame(x,y : integer);
begin
    Form1.WS.Canvas.Draw(0,0,imgTemp);

    Form1.WS.Canvas.Brush.Style := bsClear;
    Form1.WS.Canvas.Pen.Color := clRed;
    Form1.WS.Canvas.Rectangle(Round(point1.X + Screen_x),Round(point1.Y + Screen_y),
                              Round(X),Round(Y));
end;

//Отрисовка заготовки прямоугольника при указании размеров мышью
procedure PrepareDrawRectangle(x_rect, y_rect, x_vrem, y_vrem : double;
                               x,y : integer);

var
  VremColor : TColor;

begin
    Form1.WS.Canvas.Draw(0,0,imgTemp);

    VremColor := Form1.WS.Canvas.Pen.Color;
    Form1.WS.Canvas.Pen.Color := Cvet;

    Form1.WS.Canvas.line(Round(x_rect+screen_x),Round(y_rect+screen_y),
                         Round(x_vrem+screen_x),Round(y_rect+screen_y));
    Form1.WS.Canvas.line(Round(x_vrem+screen_x),Round(y_rect+screen_y),
                         Round(x_vrem+screen_x),Round(y_vrem+screen_y));
    Form1.WS.Canvas.line(Round(x_vrem+screen_x),Round(y_vrem+screen_y),
                         Round(x_rect+screen_x),Round(y_vrem+screen_y));
    Form1.WS.Canvas.line(Round(x_rect+screen_x),Round(y_vrem+screen_y),
                         Round(x_rect+screen_x),Round(y_rect+screen_y));

    Form1.WS.Canvas.Pen.Color := clPurple;

    Form1.WS.Canvas.TextOut(x+10,y-10,'B = '+FloatToStr(abs(x_rect-x_vrem))+
                                    '; H = '+FloatToStr(abs(y_rect-y_vrem)));

    Form1.WS.Canvas.Pen.Color := VremColor;
end;

//Отрисовка заготовки отрезка при указании конечной точки мышью
Procedure PrepareDrawLine(x_vrem, y_vrem : double; x,y : integer);

var
  Angle_Vrem : double = 0;
  VremColor : TColor;

begin
    Form1.WS.Canvas.Draw(0,0,imgTemp);

    if (lines[k].x1<x_vrem) and
       (lines[k].y1>y_vrem) then
        angle_vrem :=  ArcTan(abs(lines[k].y1-y_vrem)/abs(lines[k].x1-x_vrem))
    else
    if (lines[k].x1 = x_vrem) and
       (lines[k].y1>y_vrem) then angle_vrem:=DegToRad(90)
    else
    if (lines[k].x1>x_vrem) and
       (lines[k].y1>y_vrem) then
        angle_vrem :=  abs(ArcTan((y_vrem-lines[k].y1)/(x_vrem-lines[k].x1))-
                       DegToRad(90-ArcTan((y-lines[k].y1)/(x-lines[k].x1)))) +
                       DegToRad(90)
    else
    if (lines[k].x1>x_vrem) and
       (lines[k].y1=y_vrem) then angle_vrem :=  DegToRad(180)
    else
    if (lines[k].x1>x_vrem) and
       (lines[k].y1<y_vrem) then
        angle_vrem :=  abs(ArcTan((y_vrem-lines[k].y1)/(x_vrem-lines[k].x1))-
                       DegToRad(90-ArcTan((y_vrem-lines[k].y1)/(x_vrem-lines[k].x1)))) +
                       DegToRad(90)
    else
    if (lines[k].x1 = x_vrem) and
       (lines[k].y1<y_vrem) then angle_vrem:=DegToRad(270)
    else
    if (lines[k].x1<x_vrem) and
       (lines[k].y1<y_vrem) then
        angle_vrem :=  abs(ArcTan((y_vrem-lines[k].y1)/(x_vrem-lines[k].x1))-
                       DegToRad(90-ArcTan((y_vrem-lines[k].y1)/(x_vrem-lines[k].x1)))) +
                       DegToRad(270);


    angle_vrem := RadToDeg(angle_vrem);

    angle_vrem := SimpleRoundTo(angle_vrem,-3);

    VremColor := Form1.WS.Canvas.Pen.Color;
    Form1.WS.Canvas.Pen.Color := Cvet;

    If (orto_graph) then
       begin
            If abs(Lines[k].x1-x_vrem)>abs(Lines[k].y1-y_vrem) then
               begin
                    Form1.WS.Canvas.Line(Round(Lines[k].x1+Screen_x),
                                   Round(Lines[k].y1+Screen_y),
                                   x, Round(Lines[k].y1+Screen_y));
                    //Индикатор рядом с указателем мыши
                    Form1.WS.Canvas.Pen.Color := clPurple;

                    Form1.WS.Canvas.TextOut(x+10,y-10,
                                    'Длина = ' +
                                    SimpleRoundTo((abs(lines[k].x1-x_vrem)),-2).ToString);
               end
            else
               begin
                    Form1.WS.Canvas.Line(Round(Lines[k].x1+Screen_x),
                                   Round(Lines[k].y1+Screen_y),
                                   Round(Lines[k].x1+Screen_x), y);
                    //Индикатор рядом с указателем мыши
                    Form1.WS.Canvas.TextOut(x+10,y-10,
                                            'Длина = ' +
                                            SimpleRoundTo((abs(lines[k].y1-y_vrem)),-2).ToString);
               end;
       end
    else
        begin
             Form1.WS.Canvas.Line(Round(Lines[k].x1+Screen_x),Round(Lines[k].y1+Screen_y),x, y);
             //Индикатор рядом с указателем мыши
             Form1.WS.Canvas.Pen.Color := clPurple;

             Form1.WS.Canvas.TextOut(x+10,y-10,'Угол = '+FloatToStr(angle_vrem)+
                                     ' / Длина = ' +
                                     SimpleRoundTo(sqrt(power(lines[k].x1-x,2) +
                                     power(lines[k].y1-y,2)),-2).ToString);
        end;
    Form1.WS.Canvas.Pen.Color := VremColor;
end;

//Отрисовка заготовки отрезка при указании СЛЕДУЮЩЕЙ точки мышью
Procedure PrepareDrawNextLine(x_vrem, y_vrem : double; x,y : integer);

var
  Angle_Vrem : double = 0;
  VremColor : TColor;

begin
    Form1.WS.Canvas.Draw(0,0,imgTemp);

    if (last_point.x<x) and
       (last_point.y>y) then
       angle_vrem :=  ArcTan(abs(last_point.y-y)/abs(last_point.x-x))
    else
    if (last_point.x = x) and
       (last_point.y>y) then angle_vrem:=DegToRad(90)
    else
    if (last_point.x>x) and
       (last_point.y>y) then
        angle_vrem :=  abs(ArcTan((y-last_point.y)/(x-last_point.x))-
                       DegToRad(90-ArcTan((y-last_point.y)/(x-last_point.x)))) +
                       DegToRad(90)
    else
    if (last_point.x>x) and
       (last_point.y=y) then angle_vrem :=  DegToRad(180)
    else
    if (last_point.x>x) and
       (last_point.y<y) then
        angle_vrem :=  abs(ArcTan((y-last_point.y)/(x-last_point.x))-
                       DegToRad(90-ArcTan((y-last_point.y)/(x-last_point.x)))) +
                       DegToRad(90)
    else
    if (last_point.x = x) and
       (last_point.y<y) then angle_vrem:=DegToRad(270)
    else
    if (last_point.x<x) and
       (last_point.y<y) then
        angle_vrem :=  abs(ArcTan((y-last_point.y)/(x-last_point.x))-
                       DegToRad(90-ArcTan((y-last_point.y)/(x-last_point.x)))) +
                       DegToRad(270);


    angle_vrem := RadToDeg(angle_vrem);


    angle_vrem := SimpleRoundTo(angle_vrem,-3);

    VremColor := Form1.WS.Canvas.Pen.Color;
    Form1.WS.Canvas.Pen.Color := Cvet;

    If (orto_graph) then
     begin
          If abs(last_point.x-x_vrem)<abs(last_point.y-y_vrem) then
          begin
               Form1.WS.Canvas.Line(Round(last_point.x+Screen_x),
                                    Round(last_point.y+Screen_y),
                                    Round(last_point.x)+Screen_x, y);
               Form1.WS.Canvas.TextOut(x+10,y-10,
                                       'Длина = ' +
                                       SimpleRoundTo(abs(y_vrem-last_point.y),-2).ToString);
          end
          else
          begin
              Form1.WS.Canvas.Line(Round(last_point.x+Screen_x),
                                   Round(last_point.y+Screen_y),
                                   x,Round(last_point.y+Screen_y));
              //Индикатор рядом с указателем мыши
              Form1.WS.Canvas.Pen.Color := clPurple;

              Form1.WS.Canvas.TextOut(x+10,y-10,
                                      'Длина = ' +
                                      SimpleRoundTo(abs(x_vrem-last_point.x),-2).ToString);
          end;
     end
    else
    begin
        Form1.WS.Canvas.Line(Round(last_point.x+Screen_x),Round(last_point.y)+ Screen_y,
                             Round(x),Round(y));
        //Индикатор рядом с указателем мыши
        Form1.WS.Canvas.Pen.Color := clPurple;

        Form1.WS.Canvas.TextOut(x+10,y-10,'Угол = '+FloatToStr(angle_vrem)+
                                ' / Длина = ' +
                                SimpleRoundTo(sqrt(power(x_vrem-last_point.x,2) +
                                power(y_vrem-last_point.y,2)),-2).ToString);
    end;

    Form1.WS.Canvas.Pen.Color := VremColor;
end;

//Отрисовка заготовки окружности при построении по 2 ординатам
Procedure PrepareDrawCirc2Ord(x_vrem, y_vrem : double; x,y : integer);

var
  Rad : double;
  Center : MyTPoint;
  VremColor : TColor;

begin
    Form1.WS.Canvas.Draw(0,0,imgTemp);

    center.X := abs(Point1.X+X_vrem) / 2;
    center.Y := abs(Point1.Y+Y_vrem) / 2;
    rad := sqrt(sqr(Point1.X-X_vrem)+sqr(Point1.Y-Y_vrem));

    VremColor := Form1.WS.Canvas.Pen.Color;
    Form1.WS.Canvas.Pen.Color := Cvet;

    Form1.WS.Canvas.Brush.Style := bsClear;
    Form1.WS.Canvas.Ellipse(Round(center.X-rad / 2)+screen_x,Round(center.Y-rad/2)+screen_y,
                            Round(center.X+rad/2)+screen_x,Round(center.Y+rad/2)+screen_y);

    Form1.WS.Canvas.Pen.Color := clPurple;
    Form1.WS.Canvas.TextOut(x+10+Screen_x,y-10+Screen_y,'R = '+FloatToStr(rad));

    Form1.WS.Canvas.Pen.Color := VremColor;
end;

//Отрисовка заготовки окружности при построении по центру и радиусу
Procedure PrepareDrawCircCentrRad(x_vrem, y_vrem : double; x,y : integer);

var
  Rad : double;
  VremColor : TColor;

begin
    Form1.WS.Canvas.Draw(0,0,imgTemp);

    VremColor := Form1.WS.Canvas.Pen.Color;
    Form1.WS.Canvas.Pen.Color := Cvet;

    rad := Round(sqrt(power(circles[k_cir].x-x_vrem,2)+power(circles[k_cir].y-y_vrem,2)));

    Form1.WS.Canvas.Brush.Style := bsClear;
    Form1.WS.Canvas.Ellipse(Round(circles[k_cir].x+Screen_x-rad),
                            Round(circles[k_cir].y+Screen_y-rad),
                            Round(circles[k_cir].x+Screen_x+rad),
                            Round(circles[k_cir].y+Screen_y+rad));

    Form1.WS.Canvas.Pen.Color := clPurple;
    Form1.WS.Canvas.TextOut(x+10,y-10,'R = '+FloatToStr(rad));

    Form1.WS.Canvas.Pen.Color := VremColor;
end;

//Отрисовка шаблона объектов при перетаскивании за точку
Procedure PrepareDrawDotMoved(x_vrem, y_vrem : double; x,y : integer);

var
  angle_vrem : double = 0;

begin
    Form1.WS.Cursor := crHandPoint;

    If (select_type = line) then //выбрана линия
    begin
        near_points(x_vrem,y_vrem,4, line, drag_selected);

        If (Drag_Point.x = lines[drag_selected].x1) and
           (Drag_Point.y = lines[drag_selected].y1) then
        begin
           lines[drag_selected].x1:=x_vrem;
           lines[drag_selected].y1:=y_vrem;
        end
        else
        begin
           lines[drag_selected].x2:=x_vrem;
           lines[drag_selected].y2:=y_vrem;
        end;

        Drag_Point.x := x_vrem;
        Drag_Point.y := y_vrem;

        if (lines[drag_selected].x1<>lines[drag_selected].x2) then
           angle_vrem :=  ArcTan(abs(lines[drag_selected].y1-lines[drag_selected].y2)/abs(lines[drag_selected].x1-lines[drag_selected].x2))
         else
           angle_vrem:=DegToRad(90);
         angle_vrem := RadToDeg(angle_vrem);

         if ((lines[drag_selected].x2>lines[drag_selected].x1) and
            (lines[drag_selected].y2>lines[drag_selected].y1)) or
            ((lines[drag_selected].x1>lines[drag_selected].x2) and
            (lines[drag_selected].y1>lines[drag_selected].y2))
            then angle_vrem := -angle_vrem;

        if angle_vrem<0 then angle_vrem:=360-abs(angle_vrem);

        angle_vrem := SimpleRoundTo(angle_vrem,-3);
        Form1.StatusBar1.Panels.Items[0].Text:='L='+FloatToStr(
              sqrt(power(lines[Drag_selected].x1-lines[Drag_selected].x2,2) +
                    power(lines[Drag_selected].y1-lines[Drag_selected].y2,2))) +
              ' : Угол = '+ FloatToStr(angle_vrem);
        Form1.StatusBar1.Panels.Items[0].Text:='Отрезок';
        Form1.StatusBar1.Panels.Items[1].Text := 'X = ' + x_vrem.ToString;
        Form1.StatusBar1.Panels.Items[2].Text := 'Y = ' + y_vrem.ToString;

        clear;
        Form1.WS.Canvas.Draw(0,0,imgTemp);

        //Индикатор рядом с указателем мыши
        Form1.WS.Canvas.TextOut(x+10,y-10,'Угол = '+FloatToStr(angle_vrem)+
                               ' / Длина = ' +
                               SimpleRoundTo(sqrt(power(lines[Drag_selected].x1-lines[Drag_selected].x2,2) +
                               power(lines[Drag_selected].y1-lines[Drag_selected].y2,2)),-2).ToString);
        exit;
    end;
    If (select_type = circle) then //выбрана окружность
    begin
        near_points(x_vrem,y_vrem,4, circle, drag_selected);

        circles[drag_selected].x:=x_vrem;
        circles[drag_selected].y:=y_vrem;

        Drag_Point.x := x_vrem;
        Drag_Point.y := y_vrem;

        Form1.StatusBar1.Panels.Items[0].Text:='Окружность';
        Form1.StatusBar1.Panels.Items[1].Text := 'X = ' + x_vrem.ToString;
        Form1.StatusBar1.Panels.Items[2].Text := 'Y = ' + y_vrem.ToString;

        clear;
        Form1.WS.Canvas.Draw(0,0,imgTemp);

        //Индикатор рядом с указателем мыши
        Form1.WS.Canvas.TextOut(x+10,y-10,'X = '+  FloatToStr(X_vrem)+
                                          '/ Y = ' + FloatToStr(Y));
        //Reset_mode;
        exit;
    end;
end;

//Отрисовка шаблона размера при перетаскивании за точку
Procedure PrepareDrawDotMoved(x_vrem, y_vrem : double; movedPoint : byte);

var
  VremAngle : double;
  VremDist  : double;
  x1 : double;

begin
    Form1.WS.Cursor := crHandPoint;

    vremDist := sqrt(sqr(DimAr[Drag_selected].x1-DimAr[Drag_selected].x3)+
                     sqr(DimAr[Drag_selected].y1-DimAr[Drag_selected].y3));

    If (movedPoint = 1) or     //Перемещение за 1 или 2 точку
       (movedPoint = 2) then
    begin
      If movedPoint = 1 then //Перемещение за 1 точку
      begin
        DimAr[Drag_selected].x1 := x_vrem;
        DimAr[Drag_selected].y1 := y_vrem;
      end
      else
      begin                  //Перемещение за 1 точку
        DimAr[Drag_selected].x2 := x_vrem;
        DimAr[Drag_selected].y2 := y_vrem;
      end;

      //угол наклона перпендикуляра
      VremAngle := CalcAngle(DimAr[Drag_selected].x1,DimAr[Drag_selected].y1,
                             DimAr[Drag_selected].x2,DimAr[Drag_selected].y2)-DegToRad(180);
      //Установить угол в зависимости от указанного направления (+180 град)
      If (SideDot(DimAr[Drag_selected].x1,DimAr[Drag_selected].y1,
                  DimAr[Drag_selected].x2,DimAr[Drag_selected].y2,
                  DimAr[Drag_selected].x3,DimAr[Drag_selected].y3) < 0) and
         (SideDot(DimAr[Drag_selected].x1,DimAr[Drag_selected].y1,
                  DimAr[Drag_selected].x2,DimAr[Drag_selected].y2,
                  DimAr[Drag_selected].x1+vremDist*sin(VremAngle),
                  DimAr[Drag_selected].y1+vremDist*cos(VremAngle)) > 0)
         then VremAngle += DegToRad(180);

      DimAr[Drag_selected].x3 := DimAr[Drag_selected].x1+VremDist*sin(VremAngle);
      DimAr[Drag_selected].y3 := DimAr[Drag_selected].y1+VremDist*cos(VremAngle);

      DimAr[Drag_selected].x4 := DimAr[Drag_selected].x2+vremDist*sin(VremAngle);
      DimAr[Drag_selected].y4 := DimAr[Drag_selected].y2+vremDist*cos(VremAngle);

      DimAr[Drag_selected].nadpCX := min(DimAr[Drag_selected].x3,DimAr[Drag_selected].x4) +
                                     abs(DimAr[Drag_selected].x3 - DimAr[Drag_selected].x4)/2;
      DimAr[Drag_selected].nadpCY := min(DimAr[Drag_selected].y3,DimAr[Drag_selected].y4) +
                                     abs(DimAr[Drag_selected].y3 - DimAr[Drag_selected].y4)/2;
      DimAr[Drag_selected].nadp := sqrt(sqr(DimAr[Drag_selected].x1-DimAr[Drag_selected].x2)+
                                        sqr(DimAr[Drag_selected].y1-DimAr[Drag_selected].y2));
      clear;
      exit;
    end;

    If (movedPoint = 3) or     //Перемещение за 3 или 4 точку
       (movedPoint = 4) then
    begin
      //угол наклона перпендикуляра для наклонного размера
      if (DimAr[Drag_selected].x3 <> DimAr[Drag_selected].x4) and
         (DimAr[Drag_selected].y3 <> DimAr[Drag_selected].y4) then
      begin
        VremAngle := CalcAngle(DimAr[Drag_selected].x1,DimAr[Drag_selected].y1,
                               DimAr[Drag_selected].x2,DimAr[Drag_selected].y2)-DegToRad(180);
      end
      else      //для ОРТО размера
      begin
          if (DimAr[Drag_selected].x3 > DimAr[Drag_selected].x1) and
             (DimAr[Drag_selected].x4 > DimAr[Drag_selected].x2) then
             VremAngle := DegToRad(90)
             else
          if (DimAr[Drag_selected].x3 < DimAr[Drag_selected].x1) and
             (DimAr[Drag_selected].x4 < DimAr[Drag_selected].x2) then
             VremAngle := DegToRad(270)
             else
          if (DimAr[Drag_selected].y3 > DimAr[Drag_selected].y1) and
             (DimAr[Drag_selected].y4 > DimAr[Drag_selected].y2) then
             VremAngle := DegToRad(0)
             else
             VremAngle := DegToRad(180);
      end;

        //Установить угол в зависимости от указанного направления (+180 град)
        If (SideDot(DimAr[Drag_selected].x1,DimAr[Drag_selected].y1,
                    DimAr[Drag_selected].x2,DimAr[Drag_selected].y2,
                    DimAr[Drag_selected].x3,DimAr[Drag_selected].y3) < 0) and
           (SideDot(DimAr[Drag_selected].x1,DimAr[Drag_selected].y1,
                    DimAr[Drag_selected].x2,DimAr[Drag_selected].y2,
                    DimAr[Drag_selected].x1+vremDist*sin(VremAngle),
                    DimAr[Drag_selected].y1+vremDist*cos(VremAngle)) > 0)
           then VremAngle += DegToRad(180);


      //If movedPoint = 3 then //Перемещение за 3 точку
      //begin
        vremDist := sqrt(sqr(x_vrem-DimAr[Drag_selected].x1)+
                         sqr(y_vrem-DimAr[Drag_selected].y1));
      //end
      //else
      //begin                  //Перемещение за 4 точку
      //  vremDist:= sqrt(sqr(x_vrem-DimAr[Drag_selected].x2)+
      //                   sqr(y_vrem-DimAr[Drag_selected].y2));
      //end;

      If vremDist < 5 then exit;

      if (DimAr[Drag_selected].x3 <> DimAr[Drag_selected].x4) and
         (DimAr[Drag_selected].y3 <> DimAr[Drag_selected].y4) then
      begin
        DimAr[Drag_selected].x3 := DimAr[Drag_selected].x1+VremDist*sin(VremAngle);
        DimAr[Drag_selected].y3 := DimAr[Drag_selected].y1+VremDist*cos(VremAngle);
        DimAr[Drag_selected].x4 := DimAr[Drag_selected].x2+VremDist*sin(VremAngle);
        DimAr[Drag_selected].y4 := DimAr[Drag_selected].y2+VremDist*cos(VremAngle);
      end
      else
      begin
         if DimAr[Drag_selected].x3 = DimAr[Drag_selected].x4 then
         begin
               VremDist := abs(DimAr[Drag_selected].x1-x_vrem);
               x1 := DimAr[Drag_selected].x1+VremDist*sin(VremAngle);
               if (abs(x1 - DimAr[Drag_selected].x1) < 5) or
                  (abs(x1 - DimAr[Drag_selected].x2) < 5) then VremDist := 5;
               DimAr[Drag_selected].x3 := DimAr[Drag_selected].x1+VremDist*sin(VremAngle);
               DimAr[Drag_selected].x4 := DimAr[Drag_selected].x3;
         end
         else if DimAr[Drag_selected].y3 = DimAr[Drag_selected].y4 then
         begin
               VremDist := abs(DimAr[Drag_selected].y1-y_vrem);
               DimAr[Drag_selected].y3 := DimAr[Drag_selected].y1+(VremDist)*cos(VremAngle);
               DimAr[Drag_selected].y4 := DimAr[Drag_selected].y3;
         end;
      end;

      DimAr[Drag_selected].nadpCX := min(DimAr[Drag_selected].x3,DimAr[Drag_selected].x4) +
                                     abs(DimAr[Drag_selected].x3 - DimAr[Drag_selected].x4)/2;
      DimAr[Drag_selected].nadpCY := min(DimAr[Drag_selected].y3,DimAr[Drag_selected].y4) +
                                     abs(DimAr[Drag_selected].y3 - DimAr[Drag_selected].y4)/2;

      if (DimAr[Drag_selected].x3 <> DimAr[Drag_selected].x4) and
         (DimAr[Drag_selected].y3 <> DimAr[Drag_selected].y4) then
      DimAr[Drag_selected].nadp := sqrt(sqr(DimAr[Drag_selected].x1-DimAr[Drag_selected].x2)+
                                        sqr(DimAr[Drag_selected].y1-DimAr[Drag_selected].y2));

      if (DimAr[Drag_selected].x3 = DimAr[Drag_selected].x4) then
       DimAr[Drag_selected].nadp := abs(DimAr[Drag_selected].y3-DimAr[Drag_selected].y4);
      if (DimAr[Drag_selected].y3 = DimAr[Drag_selected].y4) then
       DimAr[Drag_selected].nadp := abs(DimAr[Drag_selected].x3-DimAr[Drag_selected].x4);

      clear;
      exit;
    end;
end;

//Установка курсора мыши
Procedure SetMouseCursor(x_vrem, y_vrem : double);

var
  count : integer;
  midX, midY : double;

begin


    If (not Select_mode) and (not multy_select_mode) then
       Form1.WS.Cursor:=crDefault else
                            Form1.WS.Cursor:=crHandpoint;

    for count:=0 to Arr_length-1 do
        begin
          if not Form1.SnapCheck.Checked then exit;

          //Проверка по массиву линий

          //Вычисление середины отрезка
          midX := min(Lines[count].x1,Lines[count].x2)+
                  ((max(Lines[count].x1,Lines[count].x2)-
                  min(Lines[count].x1,Lines[count].x2)) / 2);
          midY := min(Lines[count].y1,Lines[count].y2)+
                  ((max(Lines[count].y1,Lines[count].y2)-
                  min(Lines[count].y1,Lines[count].y2)) / 2);

          if (sqrt(sqr(abs(x_vrem-Lines[count].x1))+sqr(abs(y_vrem-Lines[count].y1)))<4) or
             (sqrt(sqr(abs(x_vrem-Lines[count].x2))+sqr(abs(y_vrem-Lines[count].y2)))<4) or
             (sqrt(sqr(abs(x_vrem-midX))+sqr(abs(y_vrem-midY)))<4) then
             Form1.WS.Cursor := crCross;


          //Проверка по массиву кругов
          //Центр
           if (sqrt(sqr(abs(x_vrem-Circles[count].x))+sqr(abs(y_vrem-Circles[count].y)))<4) then
           Form1.WS.Cursor := crCross;
           //Лев.орд.
           if (sqrt(sqr((x_vrem-Circles[count].x-Circles[count].r))+sqr(abs(y_vrem-Circles[count].y)))<4) then
           Form1.WS.Cursor := crCross;
           //Прав.орд.
           if (sqrt(sqr((x_vrem-Circles[count].x+Circles[count].r))+sqr(abs(y_vrem-Circles[count].y)))<4) then
           Form1.WS.Cursor := crCross;
           //Верх.орд.
           if (sqrt(sqr(abs(x_vrem-Circles[count].x))+sqr((y_vrem-Circles[count].y-Circles[count].r)))<4) then
           Form1.WS.Cursor := crCross;
           //Нижн.орд.
           if (sqrt(sqr(abs(x_vrem-Circles[count].x))+sqr((y_vrem-Circles[count].y+Circles[count].r)))<4) then
           Form1.WS.Cursor := crCross;

          //Проверка по массиву дуг
          if (sqrt(sqr(abs(x_vrem-Arcs[count].x1))+sqr(abs(y_vrem-Arcs[count].y1)))<4) then
           Form1.WS.Cursor := crCross;
          if (sqrt(sqr(abs(x_vrem-Arcs[count].x2))+sqr(abs(y_vrem-Arcs[count].y2)))<4) then
           Form1.WS.Cursor := crCross;

          //Проверка по массиву точек
          if (sqrt(sqr(abs(x_vrem-points[count].x))+sqr(abs(y_vrem-points[count].y)))<4) then
           Form1.WS.Cursor := crCross;
        end;

      //Проверка по массиву размеров//////////////////////////////////////////
      for count := 0 to High(DimAr) do
      begin
          if (sqrt(sqr(x_vrem-DimAr[count].x1)+sqr(y_vrem-DimAr[count].y1))<4) then
              Form1.WS.Cursor := crCross;
          if (sqrt(sqr(x_vrem-DimAr[count].x2)+sqr(y_vrem-DimAr[count].y2))<4) then
              Form1.WS.Cursor := crCross;
          if (sqrt(sqr(x_vrem-DimAr[count].x3)+sqr(y_vrem-DimAr[count].y3))<4) then
                        Form1.WS.Cursor := crCross;
          if (sqrt(sqr(x_vrem-DimAr[count].x4)+sqr(y_vrem-DimAr[count].y4))<4) then
              Form1.WS.Cursor := crCross;
      end;
end;

end.

